/*
    Function:  ABROWSE()
    System:    GRUMPFISH LIBRARY
    Author:    Greg Lief
    Copyright (c) 1993, Greg Lief
    Dialect:   CA-Clipper 5.x
    Compile instructions: clipper abrowse /n /w

    If you recompile this file, at a minimum you MUST use the /N
    command-line parameter.

    If you are using Clipper 5.2, you may wish to recompile this file
    with the "/dCLIPPER52" command-line parameter to take advantage of
    the TBrowse:forceStable() method.

    If you do not plan to browse nested arrays more than one level deep,
    you may wish to recompile this file with the "/dNO_NESTED_ARRAYS"
    command-line parameter.  This will improve performance.

    If you want to use footing separators (which are disabled by
    default), you should recompile this file with the
    "/dFOOTING_SEPARATORS" command-line parameter.

    If you want to change the way that nested sub-arrays are visually
    represented, simply change the ARRAY_DESCRIPTION manifest constant.

*/

#include "boxget.ch"
#include "grump.ch"
#include "inkey.ch"
#include "setcurs.ch"

static nEle        // "record pointer" for browsing the array

#define ARRAY_DESCRIPTION     "{ ... }"


#ifdef TESTING          // begin test stub

#include "abrowse.ch"

function test
local a := { ;
             { ;
               1, 2, { ;
                       { "gregory ", 32 }, ;
                       { "jennifer", 31 }, ;
                       { "justin",   10 }  ;
                      } ;
             }, ;
             { 4, 5, 6 }, ;
             { 7, 8, 9 }, ;
             {10,11,12 }, ;
             {13,14,15 }, ;
             {16,17,18 }, ;
             {19,20,21 }  ;
           }
abrowse(,,,,a,,, {,,10}, 'the title', .t., .t.)
inkey(0)

// then again for DIRECTORY() array, using ABROWSE user-defined command
abrowse array directory() headings { "name", "size", "date", "time", "attr" } ;
                          widths {12} celledit
return nil

#endif  // TESTING ...  end test stub



function ABrowse(nTop, nLeft, nBottom, nRight, aData, aHeadings, nInitial, ;
                 aWidths, cTitle, lCellEdit, lCanMaintain)
local x
local oColumn
local nKey
local nFind
local nDepth
local nColSpacing
local nWidth
local nOldEle
local nMaxEle       := len(aData)
local oBrowse       := TBrowseNew()
local lOldscore     := set(_SET_SCOREBOARD, .f.)   // ugh...
local nColumns
local nMaxrow       := maxrow()
local nMaxcol       := maxcol()
local lClearSearch  := .f.
local cSearchStr    := ''
local lHeadings     := (aHeadings <> NIL)
local nMaxHeadLines := 0
local lAscend
local aTemp
local lContinue
#ifdef MOUSE
local nWhich
local nMRow
local nMCol
local aPos
#endif

// determine # of columns based on length of first nested array
if valtype(aData[1]) == "A"
   nColumns := len(aData[1])
else
   nColumns := 1
endif

// reset array pointer to either 1 or the contents of nInitial
if nInitial == NIL
   nEle := 1
else
   nEle := nInitial
endif

default cTitle to ''
default lCellEdit to .f.
default lCanMaintain to .f.
default aHeadings to array(nColumns)
default aWidths to array(nColumns)
if len(aHeadings) < nColumns
   asize(aHeadings, nColumns)
endif
if len(aWidths) < nColumns
   asize(aWidths, nColumns)
endif

gfsaveenv(.t., 0)
oBrowse:colorSpec := ColorSet(C_APICK_UNSELECTED, .T.) + ',' + ;
                     ColorSet(C_APICK_CURRENT, .T.)
if lHeadings
   oBrowse:headSep   := chr(205) + chr(209) + chr(205)
endif
#ifdef FOOTING_SEPARATORS
   oBrowse:footSep   := chr(205) + chr(207) + chr(205)
#endif
oBrowse:colSep    := chr(32) + chr(179) + chr(32)
oBrowse:goTopBlock    := { || nEle := 1 }
oBrowse:goBottomBlock := { || nEle := nMaxEle }
oBrowse:skipBlock := { | nSkipCnt | gilligan(@nEle, nSkipCnt, nMaxEle) }
for x := 1 to nColumns
   if nColumns > 1
      oColumn := TBColumnNew(, makeblock(aData, x, lCellEdit))
   else
      oColumn := TBColumnNew(, ;
                 { | _1 | if(_1 == NIL, aData[nEle], aData[nEle] := _1) } )
   endif
   if aHeadings[x] <> NIL
      oColumn:heading := aHeadings[x]
      // determine how many rows this heading will occupy by
      // looking for line breaks (semi-colons) within it...
      nMaxHeadLines   := max(nMaxHeadLines, ;
                             len( aHeadings[x] ) - ;
                             len( strtran(aHeadings[x], ';', '') ) + 1 )
   endif
   if aWidths[x] <> NIL
      oColumn:width := aWidths[x]
   endif
   oBrowse:AddColumn(oColumn)
next

// dynamically size the window if parameters were not passed
nDepth := len(aData) + if(lHeadings, nMaxHeadLines + 1, 0)

if nTop == NIL
   if nDepth > nMaxrow
      nTop := 1
   else
      nTop := int(nMaxrow - nDepth) / 2
      if int(nTop) <> nTop
         nTop += .5
      endif
   endif
endif
oBrowse:nTop := nTop

if nBottom == NIL
   if nDepth > nMaxrow
      nBottom := nMaxrow - 1
   else
      nBottom := nTop + nDepth - 1
   endif
endif
oBrowse:nBottom := nBottom

if nLeft == NIL .or. nRight == NIL
   nWidth := 0
   x := 0
   nColSpacing := len(oBrowse:colSep)
   do while ++x <= nColumns .and. nWidth < nMaxcol
      nWidth += oBrowse:colWidth(x) + nColSpacing
   enddo
   if nWidth < nMaxcol
      default nLeft to int(nMaxcol - nWidth) / 2
      default nRight to (nLeft + nWidth)
   else
      default nLeft to 1
      default nRight to nMaxcol - 1
   endif
endif
oBrowse:nLeft := nLeft
oBrowse:nRight := nRight

shadowbox(oBrowse:nTop - 1, oBrowse:nLeft - 1, oBrowse:nBottom + 1, ;
          oBrowse:nRight + 1, 1, cTitle, , oBrowse:colorSpec)

// draw arrows for mouse usage
#ifdef MOUSE

@ oBrowse:nBottom + 1, oBrowse:nLeft  - 1 say chr(27)
@ oBrowse:nBottom + 1, oBrowse:nRight     say chr(26)
@ oBrowse:nBottom + 1, oBrowse:nRight + 1 say chr(25)
@ oBrowse:nTop    - 1, oBrowse:nRight + 1 say chr(24)

#endif

do while nKey <> K_ESC
   do while ( nKey := inkey() ) == 0 .and. ! oBrowse:stabilize()
   enddo
   if nKey == 0
      if oBrowse:hitTop
         waiton( { "Beginning of data!" }, .f.)
         inkey(2)
         waitoff()
         oBrowse:hitTop := .f.
      elseif oBrowse:hitBottom
         waiton( { "End of data!" }, .f.)
         inkey(2)
         waitoff()
         oBrowse:hitBottom := .f.
      endif
      nKey := ginkey(0, "KEY")
   endif

   #ifdef MOUSE

   if nKey == K_LEFT_DBLCLICK
      if GFLastRow() == nMRow .and. GFLastCol() == nMCol
         nKey := K_ENTER
      else
         nKey := K_LEFT_DOWN
      endif
   endif

   if nKey == K_LEFT_DOWN
      aPos := getmousepos()
      do case
         // did they click on the up arrow?
         case aPos[1] == oBrowse:nTop-1 .and. aPos[2] == oBrowse:nRight+1
            nKey := K_PGUP
         // did they click on the down arrow?
         case aPos[1] == oBrowse:nBottom+1 .and. aPos[2] == oBrowse:nRight+1
            nKey := K_PGDN
         // did they click on the left arrow?
         case aPos[1] == oBrowse:nBottom+1 .and. aPos[2] == oBrowse:nLeft-1
            if oBrowse:leftVisible == oBrowse:freeze + 1
               nKey := K_LEFT
            else
               nKey := K_CTRL_LEFT
            endif
         // did they click on the right arrow?
         case aPos[1] == oBrowse:nBottom+1 .and. aPos[2] == oBrowse:nRight
            if oBrowse:rightVisible == oBrowse:colCount
               nKey := K_RIGHT
            else
               nKey := K_CTRL_RIGHT
            endif
      endcase
   endif

   do case
      case nKey == K_RIGHT_DOWN
         nKey := K_ESC

      case nKey == K_LEFT_DOWN
         nMRow := GFLastRow()
         nMCol := GFLastCol()
         if nMRow >= oBrowse:nTop  .and. nMRow <= oBrowse:nBottom .and. ;
                     nMCol >= oBrowse:nLeft .and. nMCol <= oBrowse:nRight
            lClearSearch := ! empty(cSearchStr)

            oBrowse:rowPos := GFLastRow() - oBrowse:nTop + ;
                              if(lHeadings, - nMaxHeadLines, 1)
            oBrowse:refreshAll()

            nWhich := oBrowse:nLeft + 1
            for x := 1 to oBrowse:colCount
               oColumn := oBrowse:getColumn( x )

               if nMCol >= nWhich .and. ;
                  nMCol <= nWhich + oBrowse:colWidth( x )
                  oBrowse:colPos := x
                  exit
               endif

               nWhich += oBrowse:colWidth( x )

               if oColumn:colSep != NIL
                  nWhich += len( oColumn:colSep )
               elseif oBrowse:colSep != NIL
                  nWhich += len( oBrowse:colSep )
               endif
            next
         endif
   endcase
   #endif

   do case
      case nKey == K_UP
         lClearSearch := ! empty(cSearchStr)
         oBrowse:up()
      case nKey == K_DOWN
         lClearSearch := ! empty(cSearchStr)
         oBrowse:down()
      case nKey == K_LEFT
         oBrowse:left()
      case nKey == K_RIGHT
         oBrowse:right()
      case nKey == K_PGUP
         lClearSearch := ! empty(cSearchStr)
         oBrowse:pageUp()
      case nKey == K_PGDN
         lClearSearch := ! empty(cSearchStr)
         oBrowse:pageDown()
      case nKey == K_CTRL_PGUP
         lClearSearch := ! empty(cSearchStr)
         oBrowse:goTop()
      case nKey == K_CTRL_PGDN
         lClearSearch := ! empty(cSearchStr)
         oBrowse:goBottom()
      case nKey == K_HOME
         oBrowse:home()
      case nKey == K_END
         oBrowse:_end()
      case nKey == K_CTRL_HOME
         oBrowse:panHome()
      case nKey == K_CTRL_END
         oBrowse:panEnd()
      case nKey == K_CTRL_LEFT
         oBrowse:panLeft()
      case nKey == K_CTRL_RIGHT
         oBrowse:panRight()

      case nKey > 31 .and. nKey < 255      // search
         if (nFind := Scan(aData, oBrowse:colPos, ;
                           cSearchStr + chr(nKey), nColumns)) > 0
            cSearchStr += chr(nKey)
            // if moving backwards (up) through the array,
            // we have to handle it manually
            if nEle > nFind
               for x := 1 to nEle - nFind
                  oBrowse:up()
               next
            elseif nEle <> nFind
               nEle := nFind
               oBrowse:refreshAll()
               #ifdef CLIPPER52
                  oBrowse:forceStable()
               #else
                  do while ! oBrowse:stabilize()
                  enddo
               #endif
            endif
            showstring(cSearchStr, oBrowse)
         endif

      case nKey == K_BS                    // truncate search string
         if ! empty(cSearchStr)
            cSearchStr := substr(cSearchStr, 1, len(cSearchStr) - 1)
            if (nFind := Scan(aData, oBrowse:colPos, cSearchStr, nColumns)) > 0
               nEle := nFind
               oBrowse:refreshAll()
               #ifdef CLIPPER52
                  oBrowse:forceStable()
               #else
                  do while ! oBrowse:stabilize()
                  enddo
               #endif
            endif
            showstring(cSearchStr, oBrowse)
         endif

      case nKey == K_INS .and. lCanMaintain .and. oBrowse:stable
         if yes_no2( { "You are about to add a new array element", ;
                       "Are you sure you want to do this?" },, ;
                       " Add ", " Cancel ")
            asize(aData, ++nMaxEle)
            aTemp := aData[nEle]     // save copy of current data
            ains(aData, nEle)
            if nColumns == 1
               aData[nEle] := blankout(aTemp)
            else
               aData[nEle] := array(nColumns)
               // initialize empty values for this new array element
               aeval(aTemp, { | e, k | aData[nEle][k] := blankout(e) } )
            endif
            oBrowse:refreshAll()
         endif

      // delete current array element
      case nKey == K_DEL .and. lCanMaintain .and. oBrowse:stable
         if yes_no2({ ;
                     if(len(aData) > 1, ;
                     "You are about to delete the current array element", ;
                     "You are about to delete the last element in this array"),;
                     "Are you sure you want to do this?" ;
                    },, " Delete ", " Cancel ")
            akill(aData, nEle)
            if --nMaxEle == 0
               nKey := K_ESC   // force exit from main loop!
            else
               oBrowse:refreshAll()
            endif
         endif

      // sort array based on current column
      case nKey == K_ALT_S .and. lCanMaintain
         lAscend := yes_no2( { "Sort in ascending or descending order?" },, ;
                               " Ascending ", " Descending ")
         if lastkey() <> K_ESC
            x := oBrowse:colPos
            if lAscend
               asort(aData, , , { | a1, a2 | a1[x] < a2[x] } )
            else
               asort(aData, , , { | a1, a2 | a1[x] > a2[x] } )
            endif
            oBrowse:refreshAll()
         endif

      case nKey == K_ENTER
         x := eval(oBrowse:getColumn(oBrowse:colPos):block)
         if valtype(x) == "C" .and. x == ARRAY_DESCRIPTION
            if empty(aData[nEle][oBrowse:colPos])
               if (x := AddFirstEle()) <> NIL
                  aadd(aData[nEle][oBrowse:colPos], x)
                  nOldEle := nEle           // save current "record pointer"
                  ABrowse(,,,,aData[nEle][oBrowse:colPos],,,,,lCellEdit,lCanMaintain)
                  nEle := nOldEle           // restore previous "record pointer"
               endif


            else
               nOldEle := nEle           // save current "record pointer"
               ABrowse(,,,,aData[nEle][oBrowse:colPos],,,,,lCellEdit,lCanMaintain)
               nEle := nOldEle           // restore previous "record pointer"
            endif
         elseif lCellEdit .and. oBrowse:stable .and. editcell(oBrowse)
            oBrowse:refreshCurrent()
         endif

   endcase

   // clear search string (necessary for vertical movement)
   if lClearSearch
      lClearSearch := .f.
      cSearchStr := []
      showstring(cSearchStr, oBrowse)
   endif

enddo
gfrestenv()
set(_SET_SCOREBOARD, lOldscore)
return nEle


/*
     Function: BlankOut()
     Purpose:  Retrieve "blank" value based on data type
*/
static function BlankOut(xData)
local cType := valtype(xData)
local xRetval
do case
   case cType == "C"
      xRetval := space(len(xData))
   case cType == "D"
      xRetval := ctod('')
   case cType == "N"
      xRetval := 0
   case cType == "L"
      xRetval := .f.
endcase
return xRetval


/*
     Function: AddFirstEle()
     Purpose:  Add first element to an empty array
*/
static function AddFirstEle(a)
local x := space(40)
local retval
boxget x prompt "This array is empty -- "    +      ;
                "enter value for first element"     ;
         picture "@S10" color colorset(C_GETS, .t.) ;
         boxcolor colorset(C_MESSAGE, .t.)
if ! empty( x := trim(x) )
   do case
      case val(x) > 0 .or. x == "0"
         retval := val(x)
      case x $ "nNfF"
         retval := .f.
      case x $ "yYtT"
         retval := .t.
      case ! empty( stod(x) )
         retval := stod(x)
      otherwise
         retval := x
   endcase
endif
return retval


/*
     Function: MakeBlock()
     Purpose:  Use detached locals to resolve loop counter reference
*/
static function makeblock(a, e, lCellEdit)
local bRetval

#ifndef NO_NESTED_ARRAYS

if lCellEdit
   bRetval := { | _1 | if(valtype(a[nEle, e]) == "A", ARRAY_DESCRIPTION, ;
                       if(_1 == NIL, a[nEle, e], a[nEle, e] := _1)) }
else
   bRetval := { || if(valtype(a[nEle, e]) == "A", ARRAY_DESCRIPTION, a[nEle, e]) }
endif

#else

if lCellEdit
   bRetval := { | _1 | if(_1 == NIL, a[nEle, e], a[nEle, e] := _1) }
else
   bRetval := { || a[nEle, e] }
endif

#endif

return bRetval


/*
   Function: ShowString()
   Purpose:  Display the search string (or restore the bottom row
             of the box if the search string is empty)
*/
static function showstring(cSearchStr, oBrowse)
static cOldScreen
if cOldScreen == NIL
   cOldScreen := savescreen(oBrowse:nBottom + 1, oBrowse:nLeft, ;
                            oBrowse:nBottom + 1, oBrowse:nRight)
endif
if len(cSearchStr) > 0
   @ oBrowse:nBottom + 1, oBrowse:nLeft + ;
                          int((oBrowse:nRight - oBrowse:nLeft) / 2) - 5 ;
     ssay "[" + padr(substr(cSearchStr, max(1, len(cSearchStr) - 7)), 8) + "]"
else
   restscreen(oBrowse:nBottom + 1, oBrowse:nLeft, ;
              oBrowse:nBottom + 1, oBrowse:nRight, cOldScreen)
endif
return nil

* end static function ShowString()
*--------------------------------------------------------------------*


/*
     Function: Scan()
     Purpose:  Perform case-insensitive ASCAN() against the appropriate
               dimension (column) of the nested arrays
*/
static function Scan(aArray, nDimension, cString, nColumns)
local lOldExact := set(_SET_EXACT, .f.)
local nRetval
if nColumns > 1
   nRetval := ascan(aArray, { | a | valtype(a[nDimension]) == "C" .and. ;
                        ltrim(upper(a[nDimension])) = upper(cString) }, 1)
else
   nRetval := ascan(aArray, { | e | valtype(e) == "C" .and. ;
                            ltrim(upper(e)) = upper(cString) }, 1)
endif
set(_SET_EXACT, lOldExact)
return nRetval

* end static function Scan()
*--------------------------------------------------------------------*


/*
   Function: EditCell()
   Purpose:  Edit one of the array elements
*/
static function EditCell(oBrowse)
local x
local cPicture
local nKey
local oColumn    := oBrowse:getColumn(oBrowse:colPos)
local cDataType  := valtype( eval(oColumn:block) )
local bOldinsert := setkey( K_INS, {|| setcursor( ;
                   if(readinsert(! readInsert()), SC_NORMAL, SC_SPECIAL1))} )
local lReadexit  := readexit(.t.)
// set initial cursor size based on current mode
local nOldcursor := setcursor( if(readInsert(), SC_SPECIAL1, SC_NORMAL) )

// widen entry fields for character fields
if cDataType == "C"
   x := padr( eval(oColumn:block), 128 )
   cPicture := "@K@S" + ltrim(str( ;
                                   if(oColumn:width <> NIL, ;
                                      oColumn:width,        ;
                                      oBrowse:colWidth(oBrowse:colPos)) ;
                                  ) )
else
   x := eval(oColumn:block)
   cPicture := "@K"
endif

readmodal( { getnew(Row(), Col(), { | _1 | if(_1 == NIL, x, x := _1) }, ;
             oColumn:heading, cPicture, oBrowse:colorSpec) } )
setcursor(nOldcursor)
setkey(K_INS, bOldinsert)
readexit(lReadexit)
nKey := lastkey()
#ifdef MOUSE
if nKey <> K_ESC .and. nKey <> K_RIGHT_DOWN
#else
if nKey <> K_ESC
#endif
   if cDataType == "C"
      x := trim(x)
   endif
   eval(oColumn:block, x)
endif
// if user exited edit with up or down arrow, pass it through
if nKey == K_UP .or. nKey == K_DOWN
   keyboard chr(nKey)
endif
return (nKey <> K_ESC)

* end static function EditCell()
*--------------------------------------------------------------------*

// end of file ABROWSE.PRG
