/*
    Program: BLANKSCR4()
    System:  GRUMPFISH LIBRARY
    Author:  Greg Lief
    Copyright (c) 1988-93, Greg Lief
    CA-Clipper 5.01 Version
    Compile instructions: clipper blank5 /n /w
    *Mouseable*
    Yet another screen-blanker... this time, the sliding blocks.
    Note: Compatible with 25- or 50-line video mode

    Calls: GFRAND()      (function in RAND.PRG)
*/

#include "box.ch"
#include "grump.ch"
#include "inkey.ch"

static cTemp

#define TOP       1
#define LEFT      2
#define BOTTOM    3
#define RIGHT     4
#define CONTENTS  5

#define UP        1
#define DOWN      3

#define DELAY    400

function blankscr4(nDelay, cPassword)
local w, x, y
local z := 0
local nBlocks := if(maxrow() < 25, 40, 80)
local aBlocks := array(nBlocks)
local key := 0
local nDirection
local nOldDirection
local nStart := seconds()
local cOldScrn
local bCondition

while nextkey() = 0 .and. seconds() - nStart < nDelay
end

if nextkey() == 0        // i.e., do while loop timed out with no keypress
   GFSaveEnv(.t., 0)
   w := if(maxrow() < 25, 20, 45)
   for x := 0 to w step 5
      for y := 0 to 70 step 10
         aBlocks[++z] := { x, y, x + 4, y + 9, savescreen(x, y, x + 4, y + 9) }
      next
   next

   // determine first block to be cleared
   y := x := gfrand(nBlocks) + 1
   scroll(aBlocks[x][TOP], aBlocks[x][LEFT], aBlocks[x][BOTTOM], ;
          aBlocks[x][RIGHT])
   if cPassword == NIL
      bCondition := { || ginkey() == 0 }
   else
      cTemp      := ''
      cPassword  := upper(cPassword)
      bCondition := { || password(cPassword) }
   endif

   #ifdef MOUSE
   HideMouseCursor()
   #endif
   while eval(bCondition)
      do while y == x
         nDirection := gfrand(4) + 1
         do case
            case nDirection == UP .and. nOldDirection <> DOWN .and. x > 8
               y -= 8
               for z := 1 to 5
                  dispbegin()
                  restscreen(aBlocks[y][TOP] + z, aBlocks[y][LEFT], ;
                             aBlocks[y][BOTTOM] + z, aBlocks[y][RIGHT], ;
                             aBlocks[y][CONTENTS])
                  scroll(aBlocks[y][TOP] + z - 1, aBlocks[y][LEFT], ;
                         aBlocks[y][TOP] + z - 1, aBlocks[y][RIGHT])
                  dispend()
                  w := 1
                  do while ++w < DELAY
                  enddo
               next
            case nDirection == DOWN .and. nOldDirection <> UP ;
                                   .and. x < nBlocks - 7
               y += 8
               for z := 1 to 5
                  dispbegin()
                  restscreen(aBlocks[y][TOP] - z, aBlocks[y][LEFT], ;
                             aBlocks[y][BOTTOM] - z, aBlocks[y][RIGHT], ;
                             aBlocks[y][CONTENTS])
                  scroll(aBlocks[y][BOTTOM] - z + 1, aBlocks[y][LEFT], ;
                         aBlocks[y][BOTTOM] - z + 1, aBlocks[y][RIGHT])
                  dispend()
                  w := 1
                  do while ++w < DELAY
                  enddo
               next
            case nDirection == LEFT .and. nOldDirection <> RIGHT .and. x % 8 <> 1
               y--
               for z := 2 to 10 step 2
                  dispbegin()
                  restscreen(aBlocks[y][TOP], aBlocks[y][LEFT] + z, ;
                             aBlocks[y][BOTTOM], aBlocks[y][RIGHT] + z, ;
                             aBlocks[y][CONTENTS])
                  scroll(aBlocks[y][TOP], aBlocks[y][LEFT] + z - 2, ;
                         aBlocks[y][BOTTOM], aBlocks[y][LEFT] + z - 1)
                  dispend()
                  w := 1
                  do while ++w < DELAY
                  enddo
               next
            case nDirection == RIGHT .and. nOldDirection <> LEFT .and. x % 8 <> 0
               y++
               for z := 2 to 10 step 2
                  dispbegin()
                  restscreen(aBlocks[y][TOP], aBlocks[y][LEFT] - z, ;
                             aBlocks[y][BOTTOM], aBlocks[y][RIGHT] - z, ;
                             aBlocks[y][CONTENTS])
                  scroll(aBlocks[y][TOP], aBlocks[y][RIGHT] - z + 1, ;
                         aBlocks[y][BOTTOM], aBlocks[y][RIGHT] - z + 2)
                  dispend()
                  w := 1
                  do while ++w < DELAY
                  enddo
               next
         endcase
      enddo
      aBlocks[x][CONTENTS] := aBlocks[y][CONTENTS]
      x := y
      nOldDirection := nDirection
   end
   #ifdef MOUSE
   ShowMouseCursor()
   #endif

   GFRestEnv()
endif
return nil


static function password(cPassword)

local nKey := inkey()
local lRetVal := .t.

if nKey > 0
   do case
      case nKey <> K_ENTER
         cTemp += upper(chr(nKey))

      case nKey == K_ENTER
         if cTemp == cPassword
            lRetVal := .f.   // forces exit from screen blank loop above
         else
            cTemp := ''
            tone(MUSIC_RASPBERRY,1)
         endif

      case nKey == K_BS .and. ! empty(cTemp)
         cTemp := left(cTemp, len(cTemp) - 1)
   endcase
endif

return lRetVal


* eof blank5.prg
