/*
    Program: GRUMPBROW()
    System: GRUMPFISH LIBRARY
    Author: Greg Lief
    Copyright (c) 1988-93, Greg Lief
    CA-Clipper 5.x Version
    Compile instructions: clipper brow /n /w

    If you recompile this file, at a minimum you MUST use the /N
    command-line parameter.

    If you are using Clipper 5.2, you may wish to recompile BROW.PRG
    with the "/dCLIPPER52" command-line parameter to take advantage of
    the TBrowse:picture instance variable and the TBrowse:forceStable()
    method.

    If you plan to use the "additional options" menu, you must specify
    the "/dEXTRA_OPTIONS" command-line parameter when recompiling
    BROW.PRG.

    If you plan to use the screen painter, you must specify the
    "/dREMBRANDT" command-line parameter when recompiling BROW.PRG.

    If you plan to use the code generator, you must specify the
    "/dCODEGEN" command-line parameter when recompiling BROW.PRG.

    If you do not want to start with a basic field template in
    the screen painter, you should specify the "/dNO_TEMPLATE"
    command-line parameter when recompiling BROW.PRG.

    Syntax:  GRUMPBROW([<security>, <top>, <left>, <bottom>, <right>,    ;
             <box>, <fields>, <heads>, <pictures>, <alternates>,         ;
             <low value>, <high value>] [, <vertical movement block>]    ;
             [, <horizontal movement block>] [, <extra options array>]   ;
             [, <extra options key>] [, <extra options menu title> ]     ;
             [, <exit key> ] [, <index descriptions array> ]             ;
             [, <pre-validation array> ]  [, <post-validation array>]    ;
             [, <auto refresh time>] [, <# of columns to lock> ]         ;
             [, <carry flag> ] [, <memo width> ] [, <title for window> ] ;
             [, <memo height> ] [, <TBrowse colorspec> ]                 ;
             [, <colorBlock array> ] [, <initialization codeblock> ]     ;
             [, <gotop logical flag> ] [, <alternate key array> ] )

    For more specifics on this dizzying parameter list, please refer to
    the Grumpfish Norton Guide file.

    GRUMPBROW() returns a character string that can contain none, some,
    or all of the following letters: "A" (indicates that a record was
    added), "E" (indicates that a record was edited), "D" (indicates
    that a record was deleted). Note that these apply only to the
    default add/edit/delete routines -- if you use your own alternate
    routines, you can keep track of that yourself.

*/

#include "apick.ch"
#include "boxget.ch"
#include "grump.ch"
#include "inkey.ch"
#include "dbstruct.ch"
#include "error.ch"
#include "setcurs.ch"

// the reason for the next manifest constant is that some people
// have reported cursor weirdness on EGA monitors... if you need
// to change the cursor size, you can do so right here
#define CURSOR_ON   SC_NORMAL

// manifest constants to delineate structure of security array
#define S_ADD         1
#define S_DELETE      2
#define S_EDIT        3
#define S_QUERY       4
#define S_SEARCH      5
#define S_VIEW        6
#define S_CELLEDIT    7
#define S_OUTPUT      8
#define S_LOCKCOLUMN  9
#define S_SUBSET     10
#define S_SWAPINDEX  11
#define S_PACK       12

#define S_ITEMS      12


// the following PLUS & MINUS definitions are used in the code
// that handles expanding and shrinking columns...
#define K_PLUS       43
#define K_MINUS      45
#ifndef K_ALT_MINUS
   #define K_ALT_MINUS 386     // Alt-Minus (Non-keypad)
#endif
#ifndef K_ALT_PLUS
   #define K_ALT_PLUS  387     // Alt-Plus (Non-keypad)
#endif

// manifest constants relevant to movement, which determine whether
// vBlock or hBlock should be evaluated after stabilization
#define NO_MOVEMENT     0
#define VERTICAL        1
#define HORIZONTAL      2

#xtranslate getArray       => cargo\[1]
#xtranslate boxArray       => cargo\[2]
#xtranslate indexBlock     => cargo\[3]
#xtranslate queryCondition => cargo\[4]
#xtranslate startValues    => cargo\[5]
#xtranslate endValues      => cargo\[6]
#xtranslate origSkipBlock  => cargo\[7]
#xtranslate memoWidth      => cargo\[8]
#xtranslate memoHeight     => cargo\[9]

#define CARGO_LENGTH                 10

#xtranslate TextAt(<r>, <c>) => substr(savescreen(<r>, <c>, <r>, <c>), 1, 1)
#xtranslate ColorAt(<r>, <c>) => substr(savescreen(<r>, <c>, <r>, <c>), 2, 1)

// manifest constants for main browse window coordinates
#define TopRow      boxcoords[1]
#define LeftColumn  boxcoords[2]
#define BottomRow   boxcoords[3]
#define RightColumn boxcoords[4]

// manifest constants for memo window coordinates
#define MEMOTOP     aMemoCoords[1]
#define MEMOLEFT    aMemoCoords[2]
#define MEMOBOTTOM  aMemoCoords[3]
#define MEMORIGHT   aMemoCoords[4]

#xtranslate GetRow(<xx>)       =>  browse:getArray\[<xx>, 1]
#xtranslate GetCol(<xx>)       =>  browse:getArray\[<xx>, 2]
#xtranslate GetLength(<xx>)    =>  browse:getArray\[<xx>, 3]
#xtranslate GetName(<xx>)      =>  browse:getArray\[<xx>, 4]
#xtranslate GetPicture(<xx>)   =>  browse:getArray\[<xx>, 5]

#xtranslate BoxTop(<xx>)       =>  browse:boxArray\[<xx>, 1]
#xtranslate BoxLeft(<xx>)      =>  browse:boxArray\[<xx>, 2]
#xtranslate BoxBottom(<xx>)    =>  browse:boxArray\[<xx>, 3]
#xtranslate BoxRight(<xx>)     =>  browse:boxArray\[<xx>, 4]
#xtranslate BoxString(<xx>)    =>  browse:boxArray\[<xx>, 5]
#xtranslate BoxColor(<xx>)     =>  browse:boxArray\[<xx>, 6]
#xtranslate BoxFill(<xx>)      =>  browse:boxArray\[<xx>, 7]

// indentation of generated source code (my preference is three)
#define INDENT(x) space(x * 3)

// directive to provide default keys for Grumpbrow() behaviors
#xtranslate DefaultKey( <n>, <c>, <msg> )     ;
                                           => ;
          if valtype( aKeys\[<n>\] ) <> "A"   ;
          ;   aKeys\[<n>\] := { <c>, <msg> }  ;
          ; end

// to delineate structure of nested arrays in aKeys array
#define KEY_VALUE   1
#define KEY_PROMPT  2


function GrumpBrow(cSecurity, ntop, nleft, nbottom, nright, lBox, ;
                   aTFields, aHeads, aPics, aAlternate,           ;
                   starting, ending, vblock, hblock, aExtraopts,  ;
                   nExtrakey, cExtratitle, xExitkey, aIndexdesc,  ;
                   aValids, aWhens, nRefresh, nLockcols, lCarry,  ;
                   nMemowidth, cTitle, nMemoHeight, cColorspec,   ;
                   aColorBlocks, bInitial, lGoTop, aKeys)
local cBoxcolor
local cArrowcolor
local security[S_ITEMS]
local xx
local nMenuTop
local a_
local mfield
local buffer
local lreadexit
local mtype
local lPainter
local key
local subkey
local browse
local mget
local column
local marker
local lOldScore
local keepgoing
local nRow
local nCol
local searchstr
local aFields
local aStru
local scrnbuff
local indexbuff
local lOldInsert
local nDirection := NO_MOVEMENT
local lDescend := .f.
local cKey := upper(indexkey(0))
local midrow
local midcol
local aMemoCoords
local bOldskip, bOldgotop, bOldgobott       // original movement blocks
local nFields
local aPicLens := {}          // array holding lengths of PICTURE clauses
local lSetDeleted := ! set(_SET_DELETED)   // used for showing deleted status
local lConfigure := .f.       // used when shrinking/expanding columns
local nPtr1
local nPtr2
local bExit
local cFilter

// next three flags are for the return value
local lDelete := .f.
local lAdd    := .f.
local lEdit   := .f.

// nOrder holds the current index order.  My reason for storing this
// to a variable is to optimize performance of the Gilligan() skip
// function, which will be used in the event that you have a data subset.
local nOrder

#ifdef EXTRA_OPTIONS

local aOptions
local aActions

#endif

// lNocase is a flag to denote case-insensitive searches
local lNocase := "UPPER(" $ cKey

#ifdef MOUSE
local oColumn
local nColumn
local nWhich
local nMRow
local nMCol
local aHotSpots := {}
local aPos
local nOldRowPos
local nOldColPos
local nPadding
local nOldLeftVis  := 0
local nOldRightVis := 0
local nHeadLines   := 0

// these delineate the structure of the aHotSpots array
#define BUTTON_LEFT      1
#define BUTTON_RIGHT     2
#define BUTTON_KEYPRESS  3
#define RESET_MOUSE_POS  4   // used only with aHotSpots in screen painter

#endif

// no database active... ergo, no point in continuing
if empty(alias())
   return ''
endif

GFSaveEnv(.t., 0)                // shuts off cursor

#ifdef EXTRA_OPTIONS

// initialize options/blocks arrays if "extra options" array was passed
default nExtrakey to K_ALT_F10
if aExtraopts <> NIL
   buffer := len(aExtraopts)
   aOptions := array(buffer)
   aActions := array(buffer)
   for xx := 1 to buffer
      aOptions[xx] := aExtraopts[xx][1]
      aActions[xx] := aExtraopts[xx][2]
   next
endif

#endif

default xExitkey to K_ESC
default cSecurity to ''
default ntop to 0
default nleft to 0
default nbottom to maxrow() - 1
default nright to maxcol()
default lBox to .t.
default lCarry to .f.
default lGoTop to .f.
if aAlternate == NIL
   aAlternate := array(8)
elseif len(aAlternate) < 8
   asize(aAlternate, 8)
endif
if aKeys == NIL
   aKeys := array(S_ITEMS)
elseif len(aKeys) < S_ITEMS
   asize(aKeys, S_ITEMS)
endif

nbottom := min(nbottom, maxrow() - 1)
midrow := ntop + int(nbottom - ntop + 1) / 2
midcol := nleft + int(nright - nleft + 1) / 2

if cColorspec == NIL
   cColorspec  := ColorSet(C_GRUMPBROW_SAY, .T.) + ',' + ;
                  ColorSet(C_GRUMPBROW_GET, .T.) + ',,,'+ ;
                  ColorSet(C_GRUMPBROW_SAY, .T.)
   cBoxcolor   := ColorSet(C_GRUMPBROW_BOX, .T.)
   cArrowcolor := ColorSet(C_GRUMPBROW_GET, .T.)
else
   xx          := at(',', cColorspec)
   cBoxcolor   := left(cColorspec, xx - 1)
   cArrowcolor := ltrim(substr(cColorspec, xx + 1))
endif
afill(security, .f.)
cSecurity := upper(cSecurity)
if "A" $ cSecurity
   security[S_ADD] := .t.
   DefaultKey(S_ADD, asc("A"), "[A]dd")
endif
if "D" $ cSecurity
   security[S_DELETE] := .t.
   DefaultKey(S_DELETE, asc("D"), "[D]elete")
endif
if "E" $ cSecurity .and. ! ("C" $ cSecurity)
   security[S_EDIT] := .t.
   DefaultKey(S_EDIT, asc("E"), "[E]dit")
   /*
      Determine if user will be able to edit the cell by pressing Enter.
      By default, if you pass "E" so that they can edit, they will be able
      to edit an individual cell.  However, if you don't want them to be able
      to do this, pass an "N" as part of the security string.  (Note that
      you can also pass "C" to enable individual cell editing - see below)
   */
   security[S_CELLEDIT] := ! ("N" $ cSecurity)
endif
// passing "C" as part of the security parameter allows cell editing
// and disables full-screen editing
if "C" $ cSecurity
   security[S_CELLEDIT] := .t.
   security[S_EDIT] := .f.
endif
if "Q" $ cSecurity
   security[S_QUERY] := .t.
   DefaultKey(S_QUERY, asc("Q"), "[Q]uery")
endif
if "K" $ cSecurity
   security[S_PACK] := .t.
   DefaultKey(S_PACK, asc("K"), "pac[K]")
endif
if "S" $ cSecurity .and. ! empty(cKey)
   xx := 0
   lDescend := ( "DESCEND(" == left(upper(cKey), 8) )
   do while ! empty( buffer := indexkey(++xx) ) .and. ! security[S_SEARCH]
      // strip out the descend() function if it's part of the key
      if "DESCEND(" == left(upper(buffer), 8)
         buffer := strtran(buffer, 'DESCEND(', '')
         buffer := left(buffer, len(buffer) - 1)
      endif
      if ! security[S_SEARCH]
         security[S_SEARCH] := ;
               (valtype(eval( &("{ || " + buffer + "}") )) == "C")
      endif
   enddo
   if security[S_SEARCH]
      DefaultKey(S_SEARCH, asc("S"), "[S]earch")
   endif
endif
if "V" $ cSecurity
   security[S_VIEW] := .t.
   DefaultKey(S_VIEW, asc("V"), "[V]iew")
endif
if "L" $ cSecurity
   security[S_LOCKCOLUMN] := .t.
   DefaultKey(S_LOCKCOLUMN, asc("L"), "[L]ock")
endif
if "O" $ cSecurity
   security[S_OUTPUT] := .t.
   DefaultKey(S_OUTPUT, asc("O"), "[O]utput")
endif
if ! empty(cKey) .and. "F" $ cSecurity
   security[S_SUBSET] := .t.
   DefaultKey(S_SUBSET, K_ALT_S, "[Alt-S]ubset")
endif
if ! empty( indexkey(2) ) .and. "I" $ cSecurity
   security[S_SWAPINDEX] := .t.
   DefaultKey(S_SWAPINDEX, K_ALT_I, "[Alt-I]ndex")
endif

// must pass a "P" in the security parameter to allow screen painting
lPainter := ("P" $ cSecurity)

// create multi-dimensional FIELDS array based on the fields
// array you passed as a parameter
if aTFields <> NIL
   aFields := aclone(aTFields)
   // create array aTFields containing all fields in this database
   aStru := dbstruct()
   nFields := len(aFields)
   for xx := 1 to nFields

      // if it's a code block, no need to create a field subarray
      if valtype(aFields[xx]) == "B"
         // do nothing

      // find this field in the database to create field subarray
      elseif (mfield := ascan(aStru, { | a | ;
                        upper(a[DBS_NAME]) == upper(aFields[xx]) } ) )  > 0
         aFields[xx] := aStru[mfield]
      else    // you called for a field that's not in the .dbf... bye-bye
         err_msg( { "Invalid field '" + aFields[xx] + ;
                    "' -- GRUMPBROW terminating"} )
         GFRestEnv()
         return ''
      endif
   next
else
   aFields := dbstruct()
   nFields := len(aFields)
endif

// either initialize blank colorblock array or verify its length
if aColorBlocks == NIL
   aColorBlocks := array(nFields)
elseif len(aColorBlocks) < nFields
   asize(aColorBlocks, nFields)
endif

// create array aHeads (column headings) if necessary
if aHeads == NIL
   // dump all field names from array aFields into aHeads
   aHeads := array(nFields)
   aeval(aFields, { | a, b | aHeads[b] := if(valtype(a) == "A", a[1], '') } )
endif

// create array aPics (picture clauses) if necessary
if aPics == NIL
   /*
      okay - create the array and fill it with garbage hi bit ASCII
      why?  because we must go through the following FOR..NEXT to
      establish the array holding PICTURE lengths, regardless of
      whether or not they passed the aPics array.  I didn't want to
      duplicate the code because that is totally wasteful, so I resorted
      to CHR(254) as an indicator that we had to create the array here.
   */
   aPics := array(nFields)
   afill(aPics, chr(254))
elseif ( xx := len(aPics) ) < nFields   // PICTURE array is too small!
   // resize PICTURE array to match # of fields
   asize(aPics, nFields)
   // and insert CHR(254) in the newly added elements for
   // "self-initialization" below
   for buffer := xx + 1 to nFields
      aPics[buffer] := chr(254)
   next
endif

// create aPicLens array to hold the length of each GET.
aPicLens := array(nFields)
for xx := 1 to nFields
   if valtype(aFields[xx]) == "A"
      a_ := makepicture(aFields[xx], aPics[xx])
      aPics[xx] := a_[1]
      aPicLens[xx] := a_[2]
   else
      aPicLens[xx] := 10   // default for non-fields
   endif
next

// create a browse object
browse := TBrowseDB(ntop + 1, nleft + 1, nbottom - 1, nright - 1)
browse:cargo := array(CARGO_LENGTH)
browse:getArray := {}
browse:boxArray := {}
browse:origSkipBlock := browse:skipBlock

// set memo width -- if passed as parameter, must be > 12
// in either case, cannot be larger than the entire browse window
if nMemowidth <> NIL
   browse:memoWidth := min( max(nMemowidth, 12), nright - nleft - 4 )
else
   browse:memoWidth := min( 40, nright - nleft - 4 )
endif

// set memo height -- if passed as parameter, must be > 4
// in either case, cannot be larger than the entire browse window
if nMemoHeight <> NIL
   browse:memoHeight := min( max(nMemoHeight, 4), nbottom - ntop )
else
   browse:memoHeight := min( 8, nbottom - ntop )
endif

// if there's an active index, store reference block in cargo
if ! empty(cKey)
   browse:indexBlock := &("{ || " + cKey + "}")
endif

// initialize coordinates for memo window, using coordinates of
// primary window as limiting factors
aMemoCoords := { max(midrow - browse:memoHeight/2, ntop), ;
                 max(midcol - browse:memoWidth/2 - 1, nleft),  ;
                 min(midrow + browse:memoHeight/2, nbottom), ;
                 min(midcol + browse:memoWidth/2 + 1, nright) }

// initialize data subset arrays
xx := 0
do while ! empty(indexkey(++xx))
enddo
browse:startValues := array(max(xx - 1, 1))
browse:endValues   := array(max(xx - 1, 1))
default aIndexdesc to array(max(xx - 1, 1))

// initialize subset start/end for current index if they were specified
// note that nOrder is used in Gilligan() for data subsets
if ( nOrder := indexord() ) > 0
   browse:startValues[nOrder] := starting
   browse:endValues[nOrder]   := ending
endif

// save original movement blocks in case they need to be reset later
bOldgotop := browse:goTopBlock
bOldgobott := browse:goBottomBlock
bOldskip := browse:skipBlock

// set up movement blocks if hi/low values were passed as parameters
if ! empty(cKey) .and. ;
         (browse:startValues[nOrder] <> NIL .or. browse:endValues[nOrder] <> NIL)
   pseudofilt(browse, .f., bOldgotop, bOldgobott, bOldskip, nOrder)
endif
setcolor(cColorspec)

browse:colorSpec := cColorspec
browse:headSep := ""
browse:colSep  := "  "
for xx := 1 to nFields

   #ifdef MOUSE

   // determine # of rows in header for this column
   if ! empty(aHeads[xx]) .and. ";" $ aHeads[xx]
      nHeadLines := max(nHeadLines, ;
                        len(aHeads[xx]) - len(strtran(aHeads[xx], ";")) )
   endif

   #endif

   // if there is a code block in the aFields array rather than a
   // subarray of field info, plug the block directly into the column
   if valtype(aFields[xx]) == "B"
      column := TBColumnNew(aHeads[xx], aFields[xx])
   else

      // memos must be treated differently than regular fields...
      // also note that character strings longer than 255 must be
      // treated as memos, because TBrowse chokes on them!
      if aFields[xx][DBS_TYPE] == "M" .or. aFields[xx][DBS_LEN] > 255
         column := TBColumnNew(aHeads[xx], &("{ || if(empty(" + ;
                   aFields[xx][DBS_NAME] + "), '<memo>', '<MEMO>') }"))
      else
         column := TBColumnNew(aHeads[xx])
         // handle picture clause for this column (if there is one)
         if aPics[xx] == chr(254)
            column:block := fieldwblock(aFields[xx][DBS_NAME], select())
         else
            // if we are using Clipper 5.2, we can take advantage
            // of the new TBColumn:picture instance variable
            // instead of slugging it out with TRANSFORM()
            #ifdef CLIPPER52
               column:block   := fieldwblock(aFields[xx][DBS_NAME], select())
               column:picture := aPics[xx]
            #else
               // use TRANSFORM() to simulate PICTURE.
               column:block := &("{ | | transform(" + aFields[xx][DBS_NAME] + ;
                                 ", '" + aPics[xx] + "') }")
              /*
                 Now load cargo with a two element array.  Element 1
                 contains the "un-transformed" retrieval code block
                 for this field, and element 2 contains the desired
                 PICTURE clause.  These are used by GETNEW() when
                 editing a cell directly (see below).
              */
              column:cargo := { &("{ | x | if( pcount() == 0, " + ;
                     alias() + "->" + aFields[xx][DBS_NAME] + ", " + ;
                     alias() + "->" + aFields[xx][DBS_NAME] + " := x) }"), ;
                     aPics[xx] }
            #endif
         endif
      endif
   endif

   // assign colorBlock if one was specified for this column
   if aColorblocks[xx] <> NIL
      column:colorBlock := aColorblocks[xx]
   endif

   browse:AddColumn( column )
   // initialize column width so that columns can shrink/expand
   column:width := browse:colWidth(xx)
next

#ifdef MOUSE
nHeadLines++
#endif

// initialize alternate cell editing array to proper length if needed
if valtype(aAlternate[S_CELLEDIT]) <> "A"
   aAlternate[S_CELLEDIT] := array(nFields)
elseif len(aAlternate[S_CELLEDIT]) < nFields
   asize(aAlternate[S_CELLEDIT], nFields)
endif

// lock the desired number of columns
if nLockcols <> NIL
   browse:freeze := nLockcols
endif

// draw box around browse window if desired
if lBox
   ShadowBox(ntop, nleft, nbottom, nright, 1, cTitle,, cBoxcolor)
endif

// draw arrows for mouse usage
#ifdef MOUSE

@ nbottom, nleft    say chr(27) color cArrowcolor
@ nbottom, nright-1 say chr(26) color cArrowcolor
@ nbottom, nright   say chr(25) color cArrowcolor
@ ntop,    nright   say chr(24) color cArrowcolor

#endif

// determine total length of all key prompts
xx := 0
aeval(aKeys, { | a | if(valtype(a) == "A" .and. ! empty(a[KEY_PROMPT]), ;
                        xx += len(a[KEY_PROMPT]) + 1, ) } )
if xx > 0    // draw prompts & set up hot spots only if there ARE key prompts!
   setpos( if( lBox, nBottom, maxrow() ), midcol - int(xx / 2) )
   for xx := 1 to S_ITEMS
      if valtype( aKeys[xx] ) == "A"
         // if using mouse, we must establish the hotspot for this option
         nPtr1 := at("[", aKeys[xx][KEY_PROMPT])
         nPtr2 := at("]", aKeys[xx][KEY_PROMPT])
         #ifdef MOUSE
         if nPtr1 > 0 .and. nPtr2 > nPtr1
            aadd(aHotSpots, { col() + nPtr1, ;
                              col() + nPtr2 - 2, ;
                              aKeys[xx][KEY_VALUE] } )
         endif
         #endif
         if nPtr1 == 0 .or. nPtr2 == 0
            dispout(aKeys[xx][KEY_PROMPT] + ' ', cBoxColor)
         else
            dispout(substr(aKeys[xx][KEY_PROMPT], 1, nPtr1), cBoxColor)
            dispout(substr(aKeys[xx][KEY_PROMPT], nPtr1 + 1, nPtr2 - nPtr1 - 1), ;
                    cArrowColor)
            dispout(substr(aKeys[xx][KEY_PROMPT], nPtr2) + ' ', cBoxColor)
         endif
      endif
   next
endif

// initialize or resize column pre- and post-validation arrays
default aValids to array(browse:colCount)
if len(aValids) < browse:colCount
   asize(aValids, browse:colCount)
endif
default aWhens to array(browse:colCount)
if len(aWhens) < browse:colCount
   asize(aWhens, browse:colCount)
endif

// execute initialization codeblock if one was specified
if valtype(bInitial) == "B"
   eval(bInitial, browse)
endif

lOldScore := set(_SET_SCOREBOARD, .F.)    // I hate it!

if security[S_SWAPINDEX] .and. ! empty( indexkey(0) )
   indexbuff := savescreen(browse:nTop - 1, browse:nLeft, ;
                           browse:nTop - 1, browse:nRight)
   @ browse:nTop - 1, browse:nLeft say "[ Viewing by " + ;
     if(aIndexdesc[nOrder] <> NIL, ;
     aIndexdesc[nOrder], indexkey(0)) + " ]"
endif

// Set up codeblock to determine exit condition based on whether
// the exit key is in fact a single key or multiple keys (in an array).
// This allows faster performance when using a single exit key because
// we can avoid the performance hit of using the ASCAN() function.
if valtype( xExitKey ) == "N"
   bExit := { || key <> xExitKey }
else // valtype( xExitKey ) == "A"
   bExit := { || ascan(xExitKey, key) == 0 }
endif

// abandon all hope ye who enter here...
do while eval(bExit)
   // do not allow cursor to move into frozen columns
   if browse:colPos <= browse:freeze
      browse:colPos := browse:freeze + 1
   endif
   dispbegin()
   do while (key := inkey()) == 0 .and. ! browse:stabilize()
   enddo
   dispend()
   if key == 0

      // if there is a box around the browse window, and SET DELETED
      // is currently off, show deleted status at top right corner
      nRow := row()
      nCol := col()
      if lBox .and. lSetDeleted
         @ ntop, nright - 11 say if(deleted(), "<deleted>", "") ;
                             color cBoxcolor
         setpos(nRow, nCol)
      endif

      // evaluate vertical or horizontal movement blocks
      // if applicable
      if vblock <> NIL .and. nDirection == VERTICAL
         eval(vblock, browse)
      elseif hblock <> NIL .and. nDirection == HORIZONTAL
         eval(hblock, browse)
      endif

      #ifndef MOUSE

      // draw arrows if data off to left or right
      // must take frozen columns into account
      if browse:leftvisible > browse:freeze + 1
         @ nbottom, nleft say chr(17) + chr(196) ;
                          color cArrowcolor
      else
         @ nbottom, nleft say if(lBox, chr(200)+chr(205), space(2)) ;
                          color cBoxcolor
      endif
      if browse:rightvisible < browse:colCount
         @ nbottom, nright - 1 say chr(196) + chr(16) ;
                          color cArrowcolor
      else
         @ nbottom, nright - 1 say if(lBox, chr(205)+chr(188), space(2)) ;
                          color cBoxcolor
      endif

      #endif

      setpos(nRow, nCol)

      if browse:hitTop
         waiton( { "Beginning of data!" }, .f.)
         inkey(2)
         waitoff()
      elseif browse:hitBottom
         waiton( { "End of data!" }, .f.)
         inkey(2)
         waitoff()
      endif

      if nRefresh <> NIL

         #ifdef MOUSE
         ShowMouseCursor()
         #endif

         xx := seconds()
         /*
           If auto refresh time was specified, use another loop so we
           can refresh while waiting for a keypress Note: we must use
           INKEY() rather than GINKEY() because GINKEY() turns mouse
           cursor on and off which causes "caffeine cursor".
         */
         do while ( key := inkey() ) == 0
            if seconds() - xx > nRefresh
               xx := seconds()
               browse:refreshAll()
               dispbegin()
               #ifdef CLIPPER52
                  browse:forceStable()
               #else
                  do while ! browse:stabilize()
                  enddo
               #endif
               dispend()
               #ifdef MOUSE
               HideMouseCursor()
               ShowMouseCursor()
               #endif
            endif
         enddo

         // check for hot-key
         xx := setkey(key)
         if xx <> NIL
            eval(xx, procname(), procline(), "KEY", browse)
         endif

         #ifdef MOUSE
         HideMouseCursor()
         #endif

      else
         key := ginkey(0, "KEY", browse)
      endif
   endif

   #ifdef MOUSE
   if key == K_LEFT_DBLCLICK
      if GFLastRow() == nMRow .and. GFLastCol() == nMCol
         key := K_ENTER
      else
         key := K_LEFT_DOWN
      endif
   endif

   // check whether they clicked on one of the buttons
   if ( key == K_LEFT_DOWN .or. key == K_LEFT_DBLCLICK )
      aPos := getmousepos()
      do case
         // did they click on the up arrow?
         case aPos[1] == nTop .and. aPos[2] == nRight
            key := K_PGUP
         // did they click on the down arrow?
         case aPos[1] == nBottom .and. aPos[2] == nRight
            key := K_PGDN
         // did they click on the left arrow?
         case aPos[1] == nBottom .and. aPos[2] == nLeft
            if browse:leftVisible == browse:freeze + 1
               key := K_LEFT
            else
               key := K_CTRL_LEFT
            endif
         // did they click on the right arrow?
         case aPos[1] == nBottom .and. aPos[2] == nRight - 1
            if browse:rightVisible == browse:colCount
               key := K_RIGHT
            else
               key := K_CTRL_RIGHT
            endif
         // did they clicked on one of the hot spots?
         case aPos[1] == if(lBox, nBottom, maxrow() )
            xx := ascan( aHotSpots, { | b | aPos[2] >= b[BUTTON_LEFT] .and. ;
                                            aPos[2] <= b[BUTTON_RIGHT] } )
            if xx > 0
               key := aHotSpots[xx][BUTTON_KEYPRESS]
            endif
            nDirection := NO_MOVEMENT
      endcase
   endif

   do case
      case key == K_RIGHT_DOWN
         key := K_ESC

      case key == K_LEFT_DOWN
         nMRow := GFLastRow()
         nMCol := GFLastCol()

         /*
           TBrowse will always center the columns in the window, so we
           must first determine the amount of left padding.
           NOTE: nOldLeftVis and nOldRightVis allow us to keep track of
           our previous column boundaries and thus only recalculate the
           padding when the screen has been panned horizontally.
         */
         if nOldLeftVis <> browse:leftVisible .or. ;
                 nOldRightVis <> browse:rightVisible
            nOldLeftVis  := browse:leftVisible
            nOldRightVis := browse:rightVisible
            nPadding := 0
            // determine the width of any frozen columns
            for nColumn := 1 to browse:freeze
               nPadding += browse:colWidth( nColumn )
            next
            // now add the width of all other unfrozen columns
            nColumn := browse:leftVisible - 1
            do while ++nColumn <= browse:rightVisible
               nPadding += browse:colWidth(nColumn)
               if nColumn <> browse:rightVisible
                  oColumn := browse:getColumn(nColumn)
                  // take column separators into account!
                  if oColumn:colSep <> NIL
                     nPadding += len( oColumn:colSep )
                  elseif browse:colSep <> NIL
                     nPadding += len( browse:colSep )
                  endif
              endif
            enddo
            nPadding := int((browse:nRight - browse:nLeft + 1 - nPadding) / 2)
         endif
         if nMRow >= browse:nTop .and. nMRow <= browse:nBottom
            nOldRowPos := browse:rowPos
            nOldColPos := browse:colPos
            nWhich  := browse:nLeft + nPadding
            for xx := 1 to browse:freeze
               nWhich += browse:colWidth( xx )
            next

            for xx := browse:leftVisible to browse:colCount
               oColumn := browse:getColumn( xx )

               if nMCol >= nWhich .and. ;
                  nMCol <= nWhich + browse:colWidth( xx )
                  browse:rowPos := nMRow - browse:nTop - nHeadLines
                  browse:refreshAll()

                  browse:colPos := xx
                  exit
               endif

               nWhich += browse:colWidth( xx )

               if oColumn:colSep <> NIL
                  nWhich += len( oColumn:colSep )
               elseif browse:colSep <> NIL
                  nWhich += len( browse:colSep )
               endif
            next
            if browse:rowPos <> nOldRowPos
               nDirection := VERTICAL
            elseif browse:colPos <> nOldColPos
               nDirection := HORIZONTAL
            else
               nDirection := NO_MOVEMENT
            endif
         else
            nDirection := NO_MOVEMENT
         endif
   endcase
   #endif

   // convert key to upper-case for testing purposes below
   if key > 96 .and. key < 123
      key -= 32
   endif

   do case

#ifdef EXTRA_OPTIONS

      case key == nExtrakey .and. aOptions <> NIL
         xx := 1
         nMenuTop := max(ntop, midrow - (len(aOptions) / 2) - 1 )
         do while ( xx := lite_menu(nMenuTop,,aOptions,,,,xx,cExtratitle) ) <> 0
            eval(aActions[xx], browse)
         enddo

#endif

      case key == K_LEFT
         browse:left()
         nDirection := HORIZONTAL

      case key == K_RIGHT
         browse:right()
         nDirection := HORIZONTAL

      case key == K_UP
         browse:up()
         nDirection := VERTICAL

      case key == K_DOWN
         browse:down()
         nDirection := VERTICAL

      case key == K_PGUP
         browse:pageUp()
         nDirection := VERTICAL

      case key == K_PGDN
         browse:pageDown()
         nDirection := VERTICAL

      case key == K_HOME .and. browse:indexBlock <> NIL
         JumpUp(browse, norder)
         nDirection := VERTICAL

      case key == K_END .and. browse:indexBlock <> NIL
         JumpDown(browse, norder)
         nDirection := VERTICAL

      case key == K_CTRL_HOME
         browse:panHome()
         nDirection := HORIZONTAL

      case key == K_CTRL_END
         browse:panEnd()
         nDirection := HORIZONTAL

      case key == K_CTRL_PGUP
         browse:goTop()
         nDirection := VERTICAL

      case key == K_CTRL_PGDN
         browse:goBottom()
         nDirection := VERTICAL

      case key == K_CTRL_LEFT
         browse:panLeft()
         nDirection := HORIZONTAL

      case key == K_CTRL_RIGHT
         browse:panRight()
         nDirection := HORIZONTAL

      // direct edit or view memos
      case key == K_ENTER .and. browse:stable .and.             ;
                  (security[S_CELLEDIT] .or.                    ;
                  (valtype(aFields[browse:colPos]) == "A" .and. ;
                  (aFields[browse:colPos][DBS_TYPE] == "M" .or. ;
                  aFields[browse:colPos][DBS_LEN] > 255) ) )

         // determine if alternate cell-editing block was specified
         if valtype(aAlternate[S_CELLEDIT][browse:colPos]) == "B"

            eval(aAlternate[S_CELLEDIT][browse:colPos], browse)

         else

            // set INS key to toggle shape of cursor as well as insert mode
            lOldInsert := setkey( K_INS, {|| setcursor( ;
                         if(readinsert(! readInsert()), SC_NORMAL, SC_SPECIAL1))} )

            // process memos differently
            if valtype(aFields[browse:colPos]) == "A" .and.           ;
                       ( aFields[browse:colPos][DBS_TYPE] == "M" .or. ;
                         aFields[browse:colPos][DBS_LEN] > 255 )

               buffer := ShadowBox(MEMOTOP, MEMOLEFT, MEMOBOTTOM, MEMORIGHT, ;
                                   2, aHeads[browse:colPos])

               // set initial cursor size based on current mode
               if security[S_CELLEDIT]
                  setcursor( if(readInsert(), SC_SPECIAL1, SC_NORMAL) )
               endif

               #ifdef MOUSE
               // set right mouse click to emulate ESC
               setkey(K_RIGHT_DOWN, { || StuffKey(chr(K_ESC)) } )
               #endif

               mfield := memoedit(fieldget(;
                                  fieldpos(aFields[browse:colPos][DBS_NAME])), ;
                                  MEMOTOP + 1, MEMOLEFT + 1, MEMOBOTTOM - 1, ;
                                  MEMORIGHT - 1, security[S_CELLEDIT])

               #ifdef MOUSE
               setkey(K_RIGHT_DOWN, NIL)
               #endif

               if security[S_CELLEDIT]
                  setcursor(SC_NONE)
                  if lastkey() <> K_ESC .and. rec_lock()
                     lEdit := .t.   // set flag for return value
                     fieldput(fieldpos(aFields[browse:colPos][DBS_NAME]),mfield)
                     unlock
                     browse:refreshCurrent()
                  endif
               endif
               ByeByeBox(buffer)

            // we need both Edit security access before proceeding
            elseif security[S_CELLEDIT]
               if Rec_Lock()
                  // retrieve current column object from browse object
                  column := browse:getColumn(browse:colPos)

                  // set initial cursor size based on current mode
                  setcursor( if(readInsert(), SC_SPECIAL1, SC_NORMAL) )

                  // enable up/down arrow keys to exit the read
                  lreadexit := readexit(.t.)
                  /*
                     Create corresponding GET object with GETNEW() and
                     read it now. Note the use of the TBcolumn cargo instance
                     variable.  Cargo is a two-element array.  The first
                     element contains the retrieval code block for this
                     data item.  The second contains the PICTURE clause.
                     This was initialized above.
                  */
                  if valtype(aFields[browse:colPos]) == "A"
                     #ifdef CLIPPER52    // retrieve column:picture
                        xx := getnew(Row(), Col(), column:block, ;
                              column:heading, column:picture, browse:colorSpec)
                     #else
                        xx := getnew(Row(), Col(), column:cargo[1], ;
                              column:heading, column:cargo[2], browse:colorSpec)
                     #endif
                  else
                     // "code block" column -- picture won't apply
                     xx := getnew(Row(), Col(), column:block, ;
                           column:heading, , browse:colorSpec)
                  endif
                  // initialize WHEN clause for this column if one exists
                  if valtype(aWhens[browse:colPos]) == "B"
                     xx:preBlock := aWhens[browse:colPos]
                  endif
                  // initialize VALID clause for this column if one exists
                  if valtype(aValids[browse:colPos]) == "B"
                     xx:postBlock := aValids[browse:colPos]
                  endif
                  readmodal( { xx } )
                  setcursor(SC_NONE)
                  readexit(lreadexit)
                  lEdit := ( lEdit .or. (lastkey() <> K_ESC) )

                  // Check if this field is part of an active filter...
                  // Note that any "codeblock" columns have to be
                  // checked as a matter of course...
                  cFilter := dbfilter()
                  if ! empty(cFilter) .and. ;
                       ( valtype(aFields[browse:colPos]) == "B" .or.  ;
                         upper(aFields[browse:colPos][DBS_NAME]) $    ;
                         upper(cFilter) )

                     // if so, see if filter condition is still fulfilled
                     // if not, bounce record pointer so edited record
                     // will no longer be displayed
                     if ! eval( &("{ || " + cFilter + "}"))
                        skip
                        browse:refreshAll()
                     else
                        browse:refreshCurrent()
                     endif
                  endif
                  // Check if this field is part of the active index
                  // Again, "codeblock" columns have to be checked as
                  // a matter of course...
                  if valtype(aFields[browse:colPos]) == "B" .or. ;
                         upper(aFields[browse:colPos][DBS_NAME]) $ ;
                         upper(indexkey(0))
                     browse:refreshAll()
                  else
                     browse:refreshCurrent()
                  endif

                  // if we exited with an arrow key, pass it through
                  // and start editing same field in the next record
                  xx := lastkey()
                  if xx == K_UP .or. xx == K_DOWN
                     if xx == K_UP
                        browse:up()
                     else
                        browse:down()
                     endif
                     #ifdef CLIPPER52
                        browse:forceStable()
                     #else
                        do while ! browse:stabilize()
                        enddo
                     #endif
                     keyboard chr(K_ENTER)
                  endif
                  unlock
               endif
            endif
            setkey(K_INS, lOldInsert)

         endif // valtype(aAlternate[S_CELLEDIT]) == "A" etcetera ad nauseum

      case security[S_OUTPUT] .and. key == aKeys[S_OUTPUT][KEY_VALUE]
         if ! override(aAlternate[S_OUTPUT], browse) .and. ;
                         yes_no('Output to printer')
            printit(browse)
         endif

      case security[S_PACK] .and. key == aKeys[S_PACK][KEY_VALUE]
         if yes_no("Removing deleted records may take several minutes", ;
                   "Are you sure you want to continue")
            // if current record is marked for deletion, move
            // backwards to find the closest undeleted record
            if deleted()
               do while deleted() .and. ! bof()
                  dbskip( -1 )
               enddo
            endif
            xx := recno()
            pack
            dbgoto( xx )
            browse:refreshAll()
         endif

      // note: locking only available if not all columns are visible
      case security[S_LOCKCOLUMN] .and. key == aKeys[S_LOCKCOLUMN][KEY_VALUE]
         if browse:rightVisible == browse:colCount .and. ;
                    browse:leftVisible == 1 .and. browse:freeze == 0
            err_msg( { "All columns are currently visible" } )
         else
            xx := browse:freeze
            boxget xx prompt "Lock how many columns" picture '##'
            if lastkey() <> K_ESC
               browse:freeze := xx
               browse:invalidate()
            endif
         endif

      case security[S_SEARCH] .and. key == aKeys[S_SEARCH][KEY_VALUE]
         if valtype(eval( &("{ || " + indexkey(0) + "}") )) == "C"
            if ! override(aAlternate[S_SEARCH], browse)
               // save affected portion of top row of box
               scrnbuff := savescreen(browse:nTop - 1, browse:nLeft, ;
                                      browse:nTop - 1, browse:nRight)
               @ browse:nTop-1, browse:nLeft + ;
                 (browse:nRight - browse:nLeft)/2 - 11 ssay "[" + space(20) + "]"
               subkey := ginkey(0, "KEY")
               searchstr := ''
               do while ( (subkey > 31 .and. subkey < 255) .or. subkey == K_BS )
                  marker := recno()
                  if subkey == K_BS
                     searchstr := substr(searchstr, 1, len(searchstr) - 1)
                     if len(searchstr) > 0
                        browse:goTop()
                        if lDescend
                           seek descend(searchstr)
                        else
                           seek searchstr
                        endif
                     else
                        searchstr := NIL
                        exit
                     endif
                  else
                     browse:goTop()
                     // force upper-case search if necessary (see above)
                     if lNocase .and. subkey > 96 .and. subkey < 123
                        subkey -= 32
                     endif
                     if lDescend
                        seek descend( searchstr + chr(subkey) )
                     else
                        seek searchstr + chr(subkey)
                     endif
                  endif
                  if eof()
                     go marker
                  else
                     if subkey <> K_BS
                        searchstr += chr(subkey)
                     endif
                     @ browse:nTop-1, browse:nLeft + ;
                       (browse:nRight - browse:nLeft)/2 - 10 ssay padc(searchstr, 20)

                     // if data subset is active, only display the
                     // found record if it is within the data subset
                     if browse:startValues[nOrder] <> NIL
                        if eval(browse:indexBlock) < browse:startValues[nOrder] .or. ;
                                eval(browse:indexBlock) > browse:endValues[nOrder]
                           go marker
                        endif
                     endif

                     // did record pointer move?
                     if recno() <> marker
                        // save current record number... see below
                        marker := recno()
                        // force a redisplay
                        dispbegin()
                        browse:refreshAll()
                        #ifdef CLIPPER52
                           browse:forceStable()
                        #else
                           do while ! browse:stabilize()
                           enddo
                        #endif
                        /*
                           Sometimes the refreshAll() method will not leave
                           us on the correct record.  If this was the case,
                           we'll move up until we get to the right record.
                        */
                        do while recno() <> marker
                           browse:up()
                           skip -1
                        enddo
                        dispend()
                        nDirection := VERTICAL
                     else
                        browse:hiLite()
                     endif
                  endif
                  subkey := ginkey(0, "KEY")
               enddo
               restscreen(browse:nTop - 1, browse:nLeft, ;
                          browse:nTop - 1, browse:nRight, scrnbuff)
               searchstr := NIL
            else
               browse:refreshAll()
            endif
         else
            err_msg( { "Search feature unavailable for current index" } )
         endif

      case security[S_ADD] .and. key == aKeys[S_ADD][KEY_VALUE]
         if ! override(aAlternate[S_ADD], browse)
            lAdd := gfbrecview('A', aFields, aHeads, aPics, browse, ;
                               aPicLens, aWhens, aValids, lCarry)
         endif
         browse:refreshAll()

      case security[S_DELETE] .and. key == aKeys[S_DELETE][KEY_VALUE]
         if ! override(aAlternate[S_DELETE], browse)

            if yes_no('This record will be '               + ;
                       if(deleted(), 'recalled', 'deleted'), ;
                      'Do you want to do this')
               if rec_lock()
                  if deleted()
                     dbRecall()
                  else
                     dbDelete()
                     eval(browse:skipBlock, -1)
                     lDelete := .t.  // set flag for return value
                  endif
                  browse:refreshall()
                  dbUnlock()
               else
                  err_msg( { NETERR_MSG } )
               endif
            endif
         else
            browse:refreshall()
         endif

      case security[S_EDIT] .and. browse:stable .and. key == aKeys[S_EDIT][KEY_VALUE]
         if ! override(aAlternate[S_EDIT], browse)
            if gfbrecview('E', aFields, aHeads, aPics, browse, ;
                               aPicLens, aWhens, aValids)
               lEdit := .t.    // set flag for return value
               browse:refreshAll()
            endif
         else
            browse:refreshAll()
         endif

#ifdef REMBRANDT

      case (key == 112 .or. key == 80) .and. lPainter      // screen painter
         gfbrecview('S', aFields, aHeads, aPics, browse, aPicLens)

#endif

      case security[S_QUERY] .and. key == aKeys[S_QUERY][KEY_VALUE]
         if ! override(aAlternate[S_QUERY], browse)
            keepgoing := .t.
            if ! empty(browse:queryCondition)
               if yes_no("Use most recent query search criteria")
                  keepgoing := .f.
                  marker := recno()
                  // browse:hitBottom won't work in this situation
                  // which is why I am using temporary variable XX
                  do while ( xx := eval(browse:skipBlock, 1) <> 0 ) ;
                      .and. ! eval(browse:queryCondition) .and. checkescape()
                  enddo
                  if ! xx
                     err_msg( { "No more matches" } )
                     go marker
                  else
                     browse:refreshAll()
                     nDirection := VERTICAL
                  endif
               endif
            endif
            if keepgoing
               gfbrecview('Q', aFields, aHeads, aPics, browse, aPicLens)
               browse:refreshAll()
               nDirection := VERTICAL
            endif
         else
            browse:refreshAll()
            nDirection := VERTICAL
         endif

      case security[S_VIEW] .and. key == aKeys[S_VIEW][KEY_VALUE]
         if ! override(aAlternate[S_VIEW], browse)
            gfbrecview('V', aFields, aHeads, aPics, browse, aPicLens)
         else
            browse:refreshAll()
         endif

      case security[S_SWAPINDEX] .and. key == aKeys[S_SWAPINDEX][KEY_VALUE]
         if indexord() == len(browse:startValues)
            nOrder := 1
         else
            nOrder++
         endif
         dbsetorder(nOrder)
         cKey := indexkey(0)

         // clear previous index description
         restscreen(browse:nTop - 1, browse:nLeft, ;
                    browse:nTop - 1, browse:nRight, indexbuff)

         // display index description, using the one from the index
         // description array if it was passed as a parameter
         @ browse:nTop - 1, browse:nLeft say "[ Viewing by " + ;
           if(aIndexdesc[nOrder] <> NIL, ;
           aIndexdesc[nOrder], indexkey(0)) + " ]"

         // reset case-insensitive flag
         lNocase := "UPPER(" $ upper(cKey)

         // reset descend flag
         lDescend := ( "DESCEND(" == left(upper(cKey), 8) )

         // reset the cargo block referring to the indexkey
         browse:indexBlock := &("{ || " + cKey + "}")

         // if there is a data subset previously established
         // for this index, set it up
         if browse:startValues[nOrder] <> NIL .or. ;
                   browse:endValues[nOrder] <> NIL
            pseudofilt(browse, .f., bOldgotop, bOldgobott, bOldskip, nOrder)
         else
            browse:goTopBlock    := bOldgotop
            browse:goBottomBlock := bOldgobott
            browse:skipBlock     := bOldskip
            // move record pointer to top of file if requested
            if lGoTop
               browse:goTop()
            endif
         endif
         browse:refreshAll()

      case security[S_SUBSET] .and. key == aKeys[S_SUBSET][KEY_VALUE]
         pseudofilt(browse, .t., bOldgotop, bOldgobott, bOldskip, nOrder)

      case (key == K_PLUS .or. key == K_MINUS .or. key == K_ALT_PLUS .or. ;
            key == K_ALT_MINUS)
         column := browse:getColumn(browse:colPos)
         do case
            case key == K_PLUS       // expand current column by one
               column:width++
               lConfigure := .t.
            case key == K_MINUS      // shrink current column by one
               if column:width > 1
                  lConfigure := .t.
                  column:width--
               endif
            case key == K_ALT_PLUS   // expand current column by five
               column:width += 5
               lConfigure := .t.
            case key == K_ALT_MINUS  // shrink current column by one
               if column:width > 5
                  lConfigure := .t.
                  column:width -= 5
               endif
         endcase
         if lConfigure
            browse:configure() // force browse to update this column's width
            // we must force a full stabilization here because if the
            // user is permitted to break the stabilize loop (as above)
            // unpredictable results will occur (same for shrinking below)
            dispbegin()
            #ifdef CLIPPER52
               browse:forceStable()
            #else
               do while ! browse:stabilize()
               enddo
            #endif
            dispend()
            lConfigure := .f.
         endif

   endcase
enddo
GFRestEnv()
set(_SET_SCOREBOARD, lOldScore)
return if(lAdd, 'A', '') + if(lEdit, 'E', '') + if(lDelete, 'D', '')

* end function GrumpBrow()
*--------------------------------------------------------------------*


/*
   Override(): See if a procedure/function name was passed to override
               the generic add/edit/delete/search routines -- if so,
               and if it exists in the application, run that instead
               of the generic routine
*/
static function Override(melement, browse)
local mproc, ret_val := .f., xx
if ! empty(melement)
   // first check if they specified a code block, and if so, evaluate
   // the block passing the browse object as a parameter
   if valtype(melement) == "B"
      ret_val := .t.
      eval(melement, browse)
   else
      if ! type(melement) $ 'UE'
         ret_val := .t.
         eval( &("{ | | " + melement + "}") )
      endif
   endif
endif
return ret_val

* end static function Override()
*--------------------------------------------------------------------*


/*
    GfbRecView(): The heart of the order, so to speak.  This is
                  where all SAYs and GETs are SAID and GOTTEN
*/
static function GfbRecView(mode, aFields, aHeads, aPics, browse, ;
                           aPicLens, aWhens, aValids, lCarry)
local lOldInsert
local picstring
local mstring
local mfield
local mrow
local handle
local pic_len
local xx
local yy
local zz
local nleft := maxcol() - 38     // used to limit long char strings below
local marker
local buffer
local oldcolor
local is_memo
local mtype
local key
local oldscrn    := savescreen()
local picclause
local getlist    := {}
local mscrnfile  := alias() + '.gfs'
local mainloop   := .t.
local curr_fld
local bytes      := (maxrow() + 1) * (maxcol() + 1) * 2
local nFields   := len(aFields)
local scatter    := array(nFields)
local nBoxes  := 0
local firstfield := 1
local goingdown
local curr_get
local ret_val    := .f.   // necessary to determine if add/edit was finalized

#ifdef MOUSE
local aPos
local nNextRecCol
local nPrevRecCol
#endif

setcolor(browse:colorSpec)
scroll()
if mode <> "S"
   @ 0, 2 ssay '[ ' + if(mode == 'Q', "Query database", ;
            if(mode == 'A', "Add" , if(mode == "E", "Edit", "View")) + ;
            "ing record") + ' ]'
endif
if mode $ "AE"
   scrncenter(maxrow(), 'Ctrl-W = save edits   Esc = exit without saving')
elseif mode == "Q"
   scrncenter(maxrow(), 'Ctrl-W = process query    Esc = exit without processing')
elseif mode == 'V'
   scrncenter(maxrow(), '[N]ext record   [P]revious record   any other key = next screen (or exit)')
   #ifdef MOUSE
   nNextRecCol := col() - 72
   nPrevRecCol := col() - 56
   #endif
endif

asize(browse:getArray, 0)

// if we have a pre-saved screen file, read it in now
if file(mscrnfile)
   if ( handle := fopen(mscrnfile) ) == -1
      err_msg( { "Could not open " + mscrnfile } )
   else
      buffer := space(bytes)
      // first paint the static text by restoring the saved screen
      fread(handle, @buffer, bytes)
      restscreen(,,,,buffer)
      buffer := [ ]
      // then read next byte, which will tell us how many fields
      // are stored in this file
      fread(handle, @buffer, 1)
      nFields := bin2i(buffer)
      // now read in that number of fields
      asize(browse:getArray, nFields)
      for yy := 1 to nFields
         gfreadline(@buffer, handle)
         browse:getArray[yy] := { bin2i(substr(buffer, 1, 1)), ;   // row pos
                           bin2i(substr(buffer, 2, 1)), ;   // column position
                           bin2i(substr(buffer, 3, 1)), ;   // row position
                                 substr(buffer, 4, 10), ;   // field name
                                 substr(buffer, 14)   }     // PICTURE clause
      next
      // okay, now read next byte, which will tell us how many boxes
      // are stored in this file  -- note that we are limited to 20
      buffer := [ ]
      fread(handle, @buffer, 1)
      nBoxes := min(bin2i(buffer), 20)
      buffer := space(14)
      // now read in that number of boxes
      asize(browse:boxArray, nBoxes)
      for xx := 1 to nBoxes
         fread(handle, @buffer, 14)
         browse:boxArray[xx] := { bin2i(substr(buffer, 1, 1)),  ;   // top row
                           bin2i(substr(buffer, 2, 1)),  ;   // left column
                           bin2i(substr(buffer, 3, 1)),  ;   // bottom row
                           bin2i(substr(buffer, 4, 1)),  ;   // right column
                                 substr(buffer, 5, 8) ,  ;   // box string
                           bin2i(substr(buffer, 13, 1)), ;   // box color
                                 substr(buffer, 14, 1) }     // fill box? Y/N
      next
      fclose(handle)
   endif
endif

// determine topmost row at which to paint SAYs and GETs
mrow := max(int( ( maxrow() - nFields ) / 2 ), 1)
curr_fld := 1
dispbegin()
do while curr_fld <= nFields .and. mainloop
   if file(mscrnfile)
      mfield := GetName(curr_fld)
      pic_len := GetLength(curr_fld)
      picclause := GetPicture(curr_fld)
   elseif valtype(aFields[curr_fld]) == "A"
      mfield := aFields[curr_fld][DBS_NAME]
      // limit width of long character strings for edit/display
      if aFields[curr_fld][DBS_TYPE] == "C" .and. aFields[curr_fld][DBS_LEN] > nleft
         pic_len   := nleft
         picclause := "@S" + ltrim(str(nleft))
      else
         pic_len := aPicLens[curr_fld]
         picclause := aPics[curr_fld]
      endif
   else
      mfield := NIL
   endif

// the NO_TEMPLATE manifest constant means don't present user with
// any fields in paint mode
#ifndef NO_TEMPLATE
   if mfield <> NIL
#else
   if mfield <> NIL .and. mode <> "S"
#endif

      mtype := type(mfield)
      // determine initial value for each get field
      do case
         // if you went nuts and changed the structure on me, there might
         // be a field that is undefined.  Fortunately for you, Mr. Grump
         // is here to protect you from the deadly DOS drop (but cut out
         // your monkey business, because next time I might not be around)
         case mtype == 'U'
            err_msg( { trim(mfield) + " is undefined - please delete the " + ;
                       mscrnfile + " file" } )
            restscreen(,,,,oldscrn)
            return .f.

         case mtype == 'D'
            scatter[curr_fld] := if(mode == "S", padr(mfield, 8),   ;
                  if((mode == "A" .and. ! lCarry) .or. mode == "Q", ;
                      ctod(''), fieldget(fieldpos(mfield))))
         case mtype == 'M'
            // embed an ascii 255 at the front of this -- we need this when
            // in the get loop down below so that we know this is a memo!
            scatter[curr_fld] := if(mode == "S", padr(mfield, 6), chr(255) + ;
                  if((mode == "A" .and. ! lCarry) .or. mode $ "MQ",          ;
                      [], fieldget(fieldpos(mfield))))
         case mtype == 'C'
            scatter[curr_fld] := if(mode == "S", padr(mfield, pic_len), ;
                  if((mode == "A" .and. ! lCarry) .or. mode == "Q", ;
                      space(len(fieldget(fieldpos(mfield)))), ;
                      fieldget(fieldpos(mfield))))
         case mtype == 'L'
            scatter[curr_fld] := if(mode == "S", substr(mfield, 1, 1), ;
                  if((mode == "A" .and. ! lCarry) .or. mode == "Q",    ;
                      .f., fieldget(fieldpos(mfield))))
         otherwise
            scatter[curr_fld] := if(mode == "S", padr(mfield, pic_len), ;
                  if((mode == "A" .and. ! lCarry) .or. mode == "Q",     ;
                      0, fieldget(fieldpos(mfield))))
      endcase

      // do generic says and gets only if we didn't use a pre-saved screen file
      if ! file(mscrnfile)
         @ curr_fld + mrow, 37 - len(aHeads[curr_fld]) say aHeads[curr_fld]
         if mode == "S"
            aadd(browse:getArray, { curr_fld + mrow, 39, pic_len, mfield, picclause } )
            setcolor("i")
         endif
         // if this is a memo field, we want to simply display the
         // word "<memo>" (defined above as picstring) rather than
         // showing the memo.  the memo will be displayed when they
         // cursor to it.
         if type(mfield) == "M"
            @ curr_fld + mrow, 39 say "<memo>"
         else
            // truncate character fields that would spill off the right side
            if type(mfield) == "C" .and. len(scatter[curr_fld]) > 40
               @ curr_fld + mrow, 39 say substr(scatter[curr_fld], 1, 40) ;
                                         picture picclause
            else
               if mode == "S"
                  @ curr_fld + mrow, 39 say scatter[curr_fld]
               else
                  @ curr_fld + mrow, 39 say scatter[curr_fld] picture picclause
               endif
            endif
         endif
         if mode == "S"
            setcolor(browse:colorSpec)
         endif
      else
         // gotta display the fields in inverse for maintenance mode
         if mode == "S"
            setcolor("i")
         endif
         // pull get coordinates and info out of gets array
         // note: if this is a memo field, we want to simply display the
         // word "<memo>" rather than the memo
         if type(mfield) == "M"
            @ GetRow(curr_fld), GetCol(curr_fld) say "<memo>"
         else
            @ GetRow(curr_fld), GetCol(curr_fld) say scatter[curr_fld] ;
                                picture picclause
         endif
         if mode == "S"
            setcolor(browse:colorSpec)
         endif
      endif

   endif

   // we have either reached the last field or the bottom of the screen
   // whichever came first -- time to stop for the pause that refreshes
   if row() == maxrow() - 1 .or. curr_fld == nFields
      dispend()
      /*
         GOINGDOWN is a flag that indicates in which direction we are
         moving through the gets.  if it is true, then we are indeed
         moving downward.  if false, we are moving upward.  It is
         important only because when we escape from editing a memo,
         we will not know where to go unless we have such a flag.
         Trust me -- I went through all kinds of gyrations on this one.
         NOTE: its value gets changed via GFBACKAGET() and GFSKIPAGET().
      */
      goingdown := .t.
      do case

#ifdef REMBRANDT

         case mode == "S"
            gfbmaint(aFields, browse, aPics, aPicLens, aHeads)
            // we must force an exit from the main loop or else
            // the sucker will bomb horribly
            mainloop := .f.
#endif

         case mode <> "V"
            setkey(K_UP, { || curr_get := gfbackaget(firstfield, curr_get, .t.), ;
                              goingdown := .f. } )
            setkey(K_DOWN, { || goingdown := gfskipaget() } )

            // set INS key to toggle shape of cursor as well as insert mode
            lOldInsert := setkey( K_INS, {|| setcursor( ;
                      if(readinsert(! readInsert()), SC_NORMAL, SC_SPECIAL1))} )

            // set initial cursor size based on current mode
            setcursor( if(readInsert(), SC_SPECIAL1, SC_NORMAL) )

            for curr_get = firstfield to curr_fld
               // skip any "non-fields"
               if scatter[curr_get] <> NIL
                  /*
                     now for the tricky part - figuring out if this is a memo.
                     memos will look like character strings, but since we were
                     thoughtful enough to embed the chr(255), that will assist
                     us in making the determination now
                  */
                  is_memo := if(valtype(scatter[curr_get]) <> "C", .f., ;
                             substr(scatter[curr_get], 1, 1) == chr(255))
                  if ! is_memo
                     if ! file(mscrnfile)
                        // truncate characters that would spill off right side
                        if valtype(scatter[curr_get]) == "C" .and. ;
                                          len(aPics[curr_get]) > 40
                           @ curr_get + mrow, 39 get scatter[curr_get] ;
                                         picture "@S40 " + aPics[curr_get]
                        else
                           @ curr_get + mrow, 39 get scatter[curr_get] ;
                                                   picture aPics[curr_get]
                        endif
                     else
                        @ GetRow(curr_get), GetCol(curr_get) get ;
                               scatter[curr_get] picture GetPicture(curr_get)
                     endif
                     // tack on any applicable WHEN or VALID clauses
                     if mode $ "AE"
                        if valtype(aWhens[curr_get]) == "B"
                           atail(getlist):preBlock := aWhens[curr_get]
                        endif
                        if valtype(aValids[curr_get]) == "B"
                           atail(getlist):postBlock := aValids[curr_get]
                        endif
                     endif

                     read

                     // check last keypress
                     key := lastkey()
                     do case

                        // proceed downward
                        case key == K_ENTER .and. goingdown
                           goingdown := .t.

                        // jump to end of this screen
                        case key == K_PGDN
                           curr_get := curr_fld

                        // back to first get
                        case key == K_PGUP
                           curr_get := firstfield - 1

                        // (save and then) fall out
                        #ifdef MOUSE
                        case key == K_CTRL_W .or. key == K_ESC .or. ;
                             key == K_RIGHT_DOWN .or. key == K_LEFT_DBLCLICK
                        #else
                        case key == K_CTRL_W .or. key == K_ESC
                        #endif
                           mainloop := .f.          // force exit from main loop
                           curr_get := curr_fld     // force exit from this loop
                     endcase
                  else
                     yy := int( maxcol() / 2 )
                     zz := browse:memoWidth / 2
                     buffer := shadowbox(mrow + curr_get, ;
                                         yy - zz - 1, ;
                                         mrow + curr_get + 4, ;
                                         yy + zz + 1, ;
                                         2, aHeads[curr_get], .f., , ;
                                         "Ctrl-W to save, or Esc to exit")
                     // shut off up and down arrows for the memoedit
                     setkey(K_UP, NIL)
                     setkey(K_DOWN, NIL)

                     #ifdef MOUSE
                     // set right mouse click to emulate ESC
                     setkey(K_RIGHT_DOWN, { || StuffKey(chr(K_ESC)) } )
                     #endif

                     scatter[curr_get] := chr(255) + ;
                         memoedit(substr(scatter[curr_get], 2), ;
                                  mrow + curr_get + 1, yy - zz, ;
                                  mrow + curr_get + 3, yy + zz, .t.)

                     #ifdef MOUSE
                     setkey(K_RIGHT_DOWN, NIL)
                     #endif

                     // okay, now define them again
                     setkey(K_UP, { || ;
                            curr_get := gfbackaget(firstfield, curr_get, .t.), ;
                            goingdown := .f. } )
                     setkey(K_DOWN, { || goingdown := gfskipaget() } )
                     if ! goingdown
                        curr_get := gfbackaget(firstfield, @curr_get)
                     endif
                     byebyebox(buffer)
                  endif
               endif
            next
            setcursor(SC_NONE)
            setkey(K_INS, lOldInsert)

            // undefine left and right arrows for the memoedit
            setkey(K_UP, NIL)
            setkey(K_DOWN, NIL)
         otherwise
            #ifdef MOUSE
            ShowMouseCursor()
            #endif
            key := ginkey(0)
            #ifdef MOUSE

            if key == K_LEFT_DOWN .or. key == K_LEFT_DBLCLICK
               aPos := GetMousePos()
               if aPos[1] == maxrow()
                  if aPos[2] == nNextRecCol
                     key := asc("N")
                  elseif aPos[2] == nPrevRecCol
                     key := asc("P")
                  endif
               endif
            elseif key == K_RIGHT_DOWN
               mainloop := .f.
            endif

            #else

            if key > 96 .and. key < 123
               key -= 32
            endif

            #endif
            if key == asc("N") .or. key == asc("P")
               // reset field pointer so we continue in this loop
               curr_fld := 0
               if key == asc("N")            // jump to NEXT record
                  if eval(browse:skipBlock, 1) == 0
                     err_msg( { "You are at the end of the file" } )
                  endif
               else                     // jump to PREVIOUS record
                  if eval(browse:skipBlock, -1) == 0
                     err_msg( { "You are at the beginning of the file" } )
                  endif
               endif
            endif
      endcase
      if curr_fld > 0
         mrow -= (maxrow() - 2)
      elseif curr_fld == 0 .and. mrow < 0
         mrow := 1
      endif

      // if we still have another pageful of gets to get,
      // then by all means clear out the previous bunch
      if mainloop .and. curr_fld <> nFields
         firstfield := curr_fld + 1
         dispbegin()
         // only clear screen if we don't have a presaved screen file!
         // otherwise, all additional text/boxes will be obliterated
         if ! file(mscrnfile)
            scroll(1, 0, maxrow() - 1, maxcol(), 0)
         endif
      endif
   endif
   curr_fld++
enddo
// note that if last field on the edit screen is a memo, user
// might have pressed ESC to exit... we nonetheless should let
// them proceed into the logic below to save their edits
if mode $ "AEQ" .and. (lastkey() <> K_ESC .or. is_memo)
   if mode == 'Q'                                 // query-by-example
      // loop through all fields and build the query string
      browse:queryCondition := []
      for xx := 1 to curr_fld - 1
         // skip "non-fields"
         if scatter[xx] <> NIL
            mfield := if(file(mscrnfile), GetName(xx), aFields[xx][DBS_NAME])
            mtype := type(mfield)
            // first get rid of the dratted chr(255) littering the front of a memo
            if ( mtype := type(mfield) ) == "M"
               scatter[xx] := substr(scatter[xx], 2)
            endif
            if ! empty(scatter[xx])
               do case
                  case mtype == "N"
                     picstring := mfield + " == " + ltrim(str(scatter[xx]))
                  case mtype == "L"
                     picstring := if(! scatter[xx], "! ", "") + mfield
                  case mtype == "D"
                     picstring := mfield + " == ctod('" + dtoc(scatter[xx]) + "')"
                  otherwise
                     /*
                        if user specified "..whatever.." in character field,
                        then they want to find the first occurrence containing
                        wwhatever; consequently, we must first check for this
                        situation by looking for two periods
                     */
                     if substr(scatter[xx], 1, 2) == '..'
                        picstring := 'upper([' + ;
                                     trim(strtran(scatter[xx], '..', '')) + ;
                                     ']) $ upper(' + mfield + ')'
                     else
                        // note the case-insensitive search - feel free to
                        // change this if you need to by stripping out the
                        // references to "UPPER("
                        picstring := "upper(" + mfield + ") = [" + ;
                                      upper(trim(scatter[xx])) + "]"
                     endif
               endcase
               browse:queryCondition += ;
                   if(! empty(browse:queryCondition), " .and. ", "") + picstring
            endif
         endif
      next
      marker := recno()
      if ! empty(browse:queryCondition)

         // compile this string to a code block, which will speed up the
         // process a tad over the traditional macro expansion rigmarole
         browse:queryCondition := MakeBlock(browse:queryCondition)

         // browse:hitBottom won't work in this situation
         // which is why I am using the temporary variable YY
         do while ( yy := eval(browse:skipBlock, 1) <> 0 ) ;
             .and. ! eval(browse:queryCondition) .and. checkescape()
         enddo
         if ! yy
            err_msg( { "No matches found" } )
            browse:queryCondition := []  // reset for next time
            go marker
         endif
      endif
   else     // add/edit mode: gather memory variables back into fields
      if yes_no(if(mode == "A", "Add this new record", "Save your edits"))
         if if(mode == "A", Add_Rec(), Rec_Lock())
            ret_val := .t.
            for xx := 1 to curr_fld - 1
               // skip "non-fields"
               if scatter[xx] <> NIL
                  mfield := if(file(mscrnfile), GetName(xx), aFields[xx][DBS_NAME])
                  // note: memos will still have a chr(255) stuck on the front
                  // and we do not want to save that, so let's act accordingly
                  if type(mfield) == "M"
                     fieldput(fieldpos(mfield), substr(scatter[xx], 2))
                  else
                     fieldput(fieldpos(mfield), scatter[xx])
                  endif
               endif
            next
         endif
      endif
   endif
endif
unlock
restscreen(,,,, oldscrn)
return ret_val

* end static function GFBRecView()
*--------------------------------------------------------------------*


/*
   GfBackAGet(): trap up arrow in the READ to
                 move back up by one GET
*/
static function gfbackaget(firstfield, curr_get, lhotkey)
/*
   we might have gotten here in one of two ways:
   (a) up arrow, in which case there would have
       been three parameters passed
   (b) direct call when escaping from a memofield
       and moving upwards through the get list
   we only need to stuff the keyboard in the first situation
*/
if lhotkey <> NIL
   keyboard chr(K_ENTER)
endif
return max(curr_get - 2, firstfield - 1)

* end static function GFBackAGet()
*--------------------------------------------------------------------*


/*
   GfSkipAGet(): trap down arrow in the READ to
                 move down one GET
*/
static function gfskipaget
keyboard chr(K_ENTER)
return .t.

* end static function GFSkipAGet()
*--------------------------------------------------------------------*


#ifdef REMBRANDT

/*
   GfbMaint(): Data Entry Screen Painter/Code Generator
*/
static function gfbmaint(aFields, browse, aPics, aPicLens, aHeads)
local key
local nRow := 0
local nCol := 0
local mfile
local buffer
local xx
local yy
local zz
local mstart
local nLeft
local nBottom
local nRight
local lChanged := .f.
local oldf1
local handle
local mfield
local mstring
local mtype
local ngets
local lOldInsert := readinsert(.f.)
local mcolor
local oldprintf
local oldconsole
local firstfield
local nMaxRow := maxrow()
local nMaxCol := maxcol()
local bytes := (nMaxRow + 1) * (nMaxCol + 1) * 2
local getlist := {}
local ccolor := browse:colorSpec
local nBoxes := len(browse:boxArray)
local aFldNames
local aMemos
local nMemos
local lContinue

#ifdef MOUSE

local aPos
local cButtonBuff := savescreen(nMaxRow, 0, nMaxRow, nMaxCol)
local aHotSpots := { {  0,  9, K_ALT_B, .t. }, ;
                     { 11, 26, K_ALT_P, .f. }, ;
                     { 28, 41, K_ALT_F, .t. }, ;
                     { 43, 49, K_ALT_E, .t. }, ;
                     { 51, 56, K_ALT_H, .f. } }
#ifdef CODEGEN
   aadd(aHotSpots, { 58, 72, K_ALT_G, .f. } )
#endif

dispbegin()
@ nMaxRow, 0  say " Draw Box " color "I"
@ nMaxRow, 6  say "B" color "R/W"
@ nMaxRow, 11 say " Change Palette " color "I"
@ nMaxRow, 19 say "P" color "R/W"
@ nMaxRow, 28 say " Insert Field " color "I"
@ nMaxRow, 36 say "F" color "R/W"
@ nMaxRow, 43 say " Erase " color "I"
@ nMaxRow, 44 say "E" color "R/W"
@ nMaxRow, 51 say " Help " color "I"
@ nMaxRow, 52 say "H" color "R/W"
#ifdef CODEGEN
   @ nMaxRow, 58 say " Generate Code " color "I"
   @ nMaxRow, 59 say "G" color "R/W"
#endif

dispend()

#endif

// disable f1 key so that they will only get my help screen!
oldf1 := setkey(K_F1, nil)
setcursor(CURSOR_ON)
do while key <> K_F10
   // show current screen position at top right corner
   @ 0, nMaxCol - 6 ssay padr(ltrim(str(nRow, 2)) + ', ' + ;
                         ltrim(str(nCol, 3)), 7)
   setpos(nRow, nCol)

   #ifdef MOUSE
   //ShowMouseCursor()
   #endif

   key := ginkey(0)

   #ifdef MOUSE

   // first check whether they clicked on one of the buttons
   if ( key == K_LEFT_DOWN .or. key == K_LEFT_DBLCLICK )
      aPos := getmousepos()
      if aPos[1] == nMaxRow
         xx := ascan( aHotSpots, { | b | aPos[2] >= b[BUTTON_LEFT] .and. ;
                                         aPos[2] <= b[BUTTON_RIGHT] } )
         if xx > 0
            key := aHotSpots[xx][BUTTON_KEYPRESS]
            // move mouse cursor back to screen cursor if requested
            if aHotSpots[xx][RESET_MOUSE_POS]
               setmousepos(nRow, nCol)
            endif
         endif
      endif
   endif

   #endif

   do case

      // help!
      case key == K_ALT_H .or. key == K_F1
         gfbhelpme()

      // erase mode (mouse only)
      #ifdef MOUSE
      case key == K_ALT_E
         gfberase(browse)
      #endif

      // insert field
      case key == K_ALT_F
         // load field names array first time only
         if aFldNames == NIL
            yy := len(aFields)
            aFldNames := {}
            for xx := 1 to yy
               // only use "actual" fields (no code blocks)
               if valtype(aFields[xx]) == "A"
                  aadd(aFldNames, aFields[xx][DBS_NAME])
               endif
            next
         endif
         apick xx array aFldNames
         if xx > 0
            yy := makepicture(aFields[xx], chr(254))
            if ! OnAGet(nRow, nCol, nCol + yy[2] - 1, browse)
               aadd(aPics, yy[1])
               aadd(aPicLens, yy[2])
               aadd(browse:getArray, { nRow, nCol, yy[2], aFldNames[xx], yy[1]} )
               @ nRow, nCol say padr(aFldNames[xx], yy[2]) color 'I'
               lChanged := .t.
            else
               err_msg( { "GETs cannot be stacked" } )
            endif
         endif

      // alphanumeric or something printable
      case key > 31 .and. key < 255
         if isitaget(nRow, nCol, browse)
            err_msg( { "You cannot type over a GET" } )
         else
            // if insert mode is on, push everything else on this row over
            if readinsert()
               /*
                  we don't want to move any gets, but how to do this?
                  simple... we jump merrily through every character on
                  this row and check for its color.  if its color is
                  inverse (112), then we know it must be a get and we
                  skip it.
               */
               mstart := 0
               for xx := 0 to nMaxCol - nCol
                  if isitaget(nRow, nCol + xx, browse)
                     restscreen(nRow, nCol + mstart + 1, nRow, nCol + xx - 1, ;
                          savescreen(nRow, nCol + mstart, nRow, nCol + xx - 2))
                     // now loop through until we find the next non-get
                     do while xx++ < nMaxCol + 1 - nCol .and. ;
                                     isitaget(nRow, nCol+xx, browse)
                     enddo
                     mstart := xx
                  endif
               next

               // if there were no gets, mstart will still be zero.
               if mstart == 0
                  restscreen(nRow, nCol + 1, nRow, nMaxCol, ;
                             savescreen(nRow, nCol, nRow, nMaxCol - 1))
               endif
            endif
            lChanged := .t.
            @ nRow, nCol ssay chr(key)
            if nCol == nMaxCol
               /*
                 Because Clipper 5.x doesn't assume a fixed screen of 25 x 80,
                 it allows you to keep typing off the right side of the screen.
                 Therefore, we must restrict this sort of nonsense ourselves
                 by forcing a wrap from the bottom row to the top.
               */
               if nRow == nMaxRow
                  nRow := 0
               else
                  nRow++
               endif
               nCol := 0
            else
               nCol++
            endif
         endif

      // move a field or resize a box
      #ifdef MOUSE
      case key == K_ENTER .or. key == K_LEFT_DOWN
         aPos := getmousepos()
         if key == K_ENTER
            lContinue := isitaget(nRow, nCol, browse)
         else
            lContinue := isitaget(aPos[1], aPos[2], browse)
            nRow := aPos[1]
            nCol := aPos[2]
            setpos(nRow, nCol)
         endif
      #else
      case key == K_ENTER
         lContinue := isitaget(nRow, nCol, browse)
      #endif
         if lContinue .and. gfbmovefld(browse, key)
            lChanged := .f.
            nRow := row()
            nCol := col()
            // let's re-sort the gets array so that they will fall in the
            // order that they appear on the screen (rather than how they
            // appear in the .dbf)
            asort(browse:getArray,,, { | x, y | if(x[1] == y[1], ;
                                         x[2] < y[2], x[1] < y[1] ) } )
         else
            // are we on a box outline?? only the shadow knows
            if nBoxes > 0
               if ( xx := isitabox(browse) ) > 0
                  browse:boxArray[xx] := gfbsizebox(browse, xx)
                  lChanged := .f.
               endif
            endif
         endif

      case key == K_BS .and. nCol > 0
         nCol--
         // wipe out this space if we are not on a get
         if ! isitaget(nRow, nCol, browse)
            @ nRow, nCol ssay [ ]
            lChanged := .t.
         endif
         setpos(nRow, nCol)

      #ifdef MOUSE
      case key == K_DEL .or. key == K_LEFT_DBLCLICK
         if key == K_LEFT_DBLCLICK
            aPos := getmousepos()
            nRow := aPos[1]
            nCol := aPos[2]
            setpos(nRow, nCol)
         endif
      #else
      case key == K_DEL
      #endif
         // first, check if we are on a get field
         if isitaget(nRow, nCol, browse)
            if yes_no("Are you sure you want to delete this field")
               // first find this element
               xx := gfbwhichget(nRow, nCol, browse)
               if xx > 0
                  // wipe it off the screen
                  scroll(GetRow(xx), GetCol(xx), GetRow(xx), GetCol(xx) + ;
                         GetLength(xx) - 1, 0)
                  // blast it out of the array
                  akill(browse:getArray, xx)
                  lChanged := .t.
               endif
            endif
            // if we were on a box outline, redraw the box
            if (xx := isitabox(browse) ) > 0
               @ BoxTop(xx), BoxLeft(xx), BoxBottom(xx), BoxRight(xx) ;
                 box BoxString(xx) color BoxColor(xx)
            endif
         // next, check if we are on a box outline
         elseif ( xx := isitabox(browse) ) > 0
            if yes_no("Are you sure you want to delete this box")
               // wipe it off the screen
               @ BoxTop(xx), BoxLeft(xx), BoxBottom(xx), BoxRight(xx) ;
                 box space(8)
               // then blast it from the array
               adel(browse:boxArray, xx)
               // finally, resize box array and decrement box counter
               asize(browse:boxArray, --nBoxes)
               lChanged := .t.
            endif
         else
            /*
               note that we don't want to move any gets, so we must check
               each character on this row to see if it is part of a get.
               however, so that we do not have to go through this loop
               on lines without gets, we do a quick scan of the array
               to see if there is a get on this row
            */
            xx := ascan(browse:getArray, { | a | a[1] == nRow} )
            if xx > 0      // yes, there is at least one get on this row
               mstart := 0
               for xx := 0 to nMaxCol - nCol
                  if isitaget(nRow, nCol + xx, browse)
                     restscreen(nRow, nCol + mstart, nRow, nCol + xx - 2, ;
                              savescreen(nRow, nCol+mstart+1, nRow, nCol+xx-1))
                     @ nRow, nCol + xx - 1 ssay [ ]
                     // now loop through until we find the next non-get
                     do while xx++ < nMaxCol + 1 - nCol .and. ;
                                     isitaget(nRow, nCol+xx, browse)
                     enddo
                     mstart := xx
                  endif
               next
            else       // no gets on this row
               restscreen(nRow, nCol, nRow, nMaxCol - 1, ;
                          savescreen(nRow, nCol + 1, nRow, nMaxCol))
               @ nRow, nMaxCol ssay [ ]
            endif
            lChanged := .t.
         endif

      // toggle insert mode
      case key == K_INS
         setcursor( if( readinsert(! readinsert()), 1, 3) )

      // color palette to change active color
      case key == K_ALT_P
         setcolor(colorpal())

      // draw box
      case key == K_ALT_B
         if gfbdrawbox(nRow, nCol, browse)
            nBoxes++
            lChanged := .t.
         endif

#ifdef CODEGEN

      // generate code (note: Shift-F10 is for backwards compatibility
      // with Grumpfish Library version 3.x)
      case key == K_ALT_G .or. key == K_SH_F10
         mfile := upper(alias())
         boxget mfile prompt 'Enter filename for generated code:' picture '@K@!'
         mfile += ".PRG"
         #ifdef MOUSE
         HideMouseCursor()
         if lastkey() <> K_ESC .and. lastkey() <> K_RIGHT_DOWN .and. ;
                 if(! file(mfile), .t., yes_no(mfile + " already exists", "Overwrite it"))
         #else
         if lastkey() <> K_ESC .and. if(! file(mfile), .t., ;
                         yes_no(mfile + " already exists", "Overwrite it"))
         #endif
            buffer := savescreen()
            dispbegin()
            gfbcleanup(browse)

            #ifdef MOUSE
            // remove buttons from bottom row so they don't sneak into
            // the generated code!
            restscreen(nMaxRow, 0, nMaxRow, nMaxCol, cButtonBuff)
            #endif

            oldprintf := set(_SET_PRINTFILE, mfile)
            set print on

            // first, get rid of position indicator so it doesn't get
            // stuck in the generated code (gasp!)
            scroll(0, nMaxCol - 6, 0, nMaxCol, 0)

            // next, let's get rid of the boxes so that they don't get
            // displayed as part of the text
            for xx := 1 to nBoxes
               @ BoxTop(xx), BoxLeft(xx), BoxBottom(xx), BoxRight(xx) box space(8)
            next
            oldconsole := set(_SET_CONSOLE, .F.)
            // standard header -- change it as you see fit
            QQOut("/*")
            QOut(INDENT(1) + "Program: " + mfile)
            QOut(INDENT(1) + "Date:    " + dtoc(date()))
            QOut(INDENT(1) + "Time:    " + time())
            QOut(INDENT(1) + "Dialect: CA-Clipper 5.x")
            QOut(INDENT(1) + "Generated by GrumpBrow()")
            QOut(INDENT(1) + "Copyright (c) 1990-3 Greg Lief")
            QOut(INDENT(1) + "Notes: data entry screen for " + alias() + ".dbf")
            QOut(INDENT(1) + "expects to be passed a parameter (mode),")
            QOut(INDENT(1) + "which could be either [A]dd, [E]dit, [V]iew")
            QOut("*/")
            QOut()
            QOut('#include "inkey.ch"')
            QOut('#include "setcurs.ch"')
            QOut()
            // must maintain a counter separate from XX, because
            // if we skip any memos the counter value will be off
            // in the generated code, and chaos will ensue...
            yy := 0
            ngets := len(browse:getArray)
            for xx := 1 to nGets
               if type(GetName(xx)) <> "M"
                  QOut("#define m" + padr(GetName(xx), 12) + " aScatter[" + ;
                       ltrim(str(++yy)) + "]")
               endif
            next
            QOut()
            QOut("function addedit(mode)")
            QOut("local getlist := {}")
            QOut("local aScatter := {}")
            QOut("local lContinue")
            QOut("local cAlias := '" + alias() + "'")
            QOut("local nOldArea")
            QOut("local marker")
            QOut("local lHad2Open := .f.")
            QOut("local lOldScore := set(_SET_SCOREBOARD, .F.)   // ugh...")
            QOut("local lOldInsert := setkey( K_INS, {|| setcursor( ;")
            QOut("      if(readinsert(! readInsert()), SC_NORMAL, SC_SPECIAL1))} )")
            QOut("local aFieldNames := { ")
            firstfield := .t.
            aMemos := {}
            for xx := 1 to nGets
               if type(GetName(xx)) <> "M"
                  if firstfield
                     firstfield := .f.
                  else
                     QQOut(", ;")
                     QOut(space(23))
                  endif
                  QQOut("'" + trim(GetName(xx)) + "'")
               else
                  // save both actual name of memo field and
                  // column heading (for title of memoedit() box)
                  aadd(aMemos, { getname(xx), aHeads[xx] } )
               endif
            next
            QQOut("}")
            nMemos := len(aMemos)
            if nMemos > 0
               QOut()
               QOut("// initialize memo locals and hot keys for memo editing")
               for xx := 1 to nMemos
                  zz := ltrim(str(xx + 1))
                  QOut("local m" + aMemos[xx][1])
                  QOut("local oldf" + zz + " := setkey(K_F" + upper(zz) + ", " + ;
                       "{ | bBlah | bBlah := setkey(K_F" + upper(zz) + ;
                       ", NIL), ;")
                  QOut(space(22) + "memedit(@m" + aMemos[xx][1] + ",,,,,'" + ;
                       aMemos[xx][2] + "'), setkey(K_F" + upper(zz) + ;
                       ", bBlah) } )" )
               next
            endif
            QOut()
            QOut("if select(cAlias) == 0")
            QOut(INDENT(1) + "nOldArea := select()")
            QOut(INDENT(1) + "use (cAlias) new")
            QOut(INDENT(1) + "lHad2Open := .t.")
            QOut("endif")
            QOut("gfsaveenv(.t.)")
            QOut("scroll()")
            QOut()
            QOut("// static text")
            // draw the boxes, dude (in Saudi Arabia, "dude" means "worm",
            // so that was not necessarily a term of endearment)
            for xx := 1 to nBoxes
               QOut("@ " + ltrim(str(BoxTop(xx)))  + ", " + ;
                           ltrim(str(BoxLeft(xx))) + ", " + ;
                           ltrim(str(BoxBottom(xx))) + ", " + ;
                           ltrim(str(BoxRight(xx))) + " box '" + BoxString(xx) + ;
                           if(BoxFill(xx) == "Y", " ", "") + ;
                           "' color '" + color_n2s(BoxColor(xx)) + "'")
            next
            // now draw all other text
            QOut("setcolor('" + ccolor + "')")
            for xx := 0 to nMaxrow
               mstring := [ ]
               yy := 0
               do while yy <= nMaxCol .and. asc(TextAt(xx, yy)) < 33
                  yy++
               enddo
               if yy <= nMaxCol
                  do while yy <= nMaxCol
                     mstring := TextAt(xx, yy)
                     mcolor := ColorAt(xx, yy)
                     zz := yy + 1
                     do while ColorAt(xx, zz) == mcolor .and. zz <= nMaxCol
                        mstring += TextAt(xx, zz)
                        zz++
                     enddo
                     if ! empty(mstring)
                        if color_s2n(ccolor) <> bin2i(mcolor)
                           ccolor := color_n2s(mcolor)
                           QOut("setcolor('" + ccolor + "')")
                        endif
                        QOut("@ " + str(xx, 2) + ", " + str(yy, 2) + ;
                                  " say [" + trim(mstring) + "]")
                     endif
                     yy := zz
                  enddo
               endif
            next

            QOut()
            QOut("// use phantom record to grab initial values if adding")
            QOut("if mode == 'A'")
            QOut(INDENT(1) + "marker := recno()")
            QOut(INDENT(1) + "go 0")
            QOut("endif")
            QOut()
            QOut("// initialize field duplicates")
            QOut("aeval(aFieldNames, { | a | aadd(aScatter, " + ;
                 "(cAlias)->(fieldget((cAlias)->(fieldpos(a))))) } )")

            // memos must be handled as variables due to pass-by-reference
            if nMemos > 0
               QOut()
               QOut("// initialize memo duplicates")
               for xx := 1 to nMemos
                  QOut("m" + aMemos[xx][1] + " := (cAlias)->" + aMemos[xx][1])
               next
            endif

            /* do the gets */
            QOut()
            QOut("// go GET 'em")
            for xx := 1 to nGets
               if type(GetName(xx)) <> "M"
                  QOut("@ " + str(GetRow(xx), 2) + ", " + ;
                       str(GetCol(xx), 2) + " get m" + GetName(xx) + ;
                       " picture '" + GetPicture(xx) + "'")
               endif
            next

            /* basic stuff */
            QOut("if mode <> 'V'")
            QOut(INDENT(1)+"setcursor( if(readInsert(), SC_SPECIAL1, SC_NORMAL) )")
            QOut(INDENT(1) + "read")
            /* do the replaces */
            QOut(INDENT(1) + "// do the replaces if they didn't escape out")
            QOut(INDENT(1) + "if lastkey() <> K_ESC")
            QOut(INDENT(2) + "if mode == 'A'")
            QOut(INDENT(3) + "lContinue := (cAlias)->( add_rec() )")
            QOut(INDENT(2) + "else")
            QOut(INDENT(3) + "lContinue := (cAlias)->( rec_lock() )")
            QOut(INDENT(2) + "endif")
            QOut(INDENT(2) + "if lContinue")
            QOut(INDENT(3) + "aeval(aFieldNames, { |a,x| " + ;
                          "(cAlias)->(fieldput((cAlias)->(fieldpos(a)), aScatter[x])) } )")
            QOut(INDENT(3) + "(cAlias)->(dbunlock())")
            QOut(INDENT(2) + "endif")

            // memos must be handled as variables due to pass-by-reference
            if nMemos > 0
               QOut()
               QOut(INDENT(2) + "// replace all memos")
               for xx := 1 to nMemos
                  QOut(INDENT(2) + "(cAlias)->" + aMemos[xx][1] + " := m" + ;
                                   aMemos[xx][1])
               next
               QOut()
            endif
            QOut(INDENT(1) + "// if in add mode, reset record pointer")
            QOut(INDENT(1) + "elseif mode == 'A'")
            QOut(INDENT(2) + "(cAlias)->(dbgoto(marker))")
            QOut(INDENT(1) + "endif")
            QOut("else")
            QOut(INDENT(1) + "getlist := {}")
            QOut(INDENT(1) + "inkey(0)")
            QOut("endif")
            QOut("gfrestenv()")
            QOut("setkey(K_INS, lOldInsert)")
            QOut("set(_SET_SCOREBOARD, lOldScore)")
            if nMemos > 0
               QOut()
               QOut("// de-activate memo editing hot keys")
               for xx := 1 to nMemos
                  zz := ltrim(str(xx + 1))
                  QOut("setkey(K_F" + upper(zz) + ", oldf" + zz + ")")
               next
            endif
            QOut("if lHad2Open")
            QOut(INDENT(1)+"dbCloseArea()")
            QOut(INDENT(1)+"dbSelectArea(nOldArea)")
            QOut("endif")
            QOut("return NIL")
            QOut()
            QOut("* end of file " + mfile)

            set print off
            set(_SET_PRINTFILE, oldprintf)
            set(_SET_CONSOLE, oldconsole)
            restscreen(,,,, buffer)
            dispend()
            waiton("Code successfully generated to " + mfile)
            inkey(1)
            waitoff()
            lChanged := .f.
         endif

#endif // CODEGEN

      #ifdef MOUSE
      case key == K_ESC .or. key == K_RIGHT_DOWN
      #else
      case key == K_ESC
      #endif
         if ! lChanged .or. yes_no('Your changes will be lost', ;
                                   'Are you sure you want to exit')
            exit
         endif

      otherwise
         gfarrowkey(key, nMaxCol, @nRow, @nCol)

   endcase
enddo
setkey(K_F1, oldf1)     // re-enable f1 key to whatever it was before this
readinsert(lOldInsert)

#ifdef MOUSE
if key <> K_ESC .and. key <> K_RIGHT_DOWN
#else
if key <> K_ESC
#endif
   gfbcleanup(browse)
   mfile := alias() + ".gfs"
   if ( handle := fcreate(mfile) ) == -1
      err_msg( { "Could not create " + mfile } )
   else
      if fwrite(handle, savescreen(), bytes) <> bytes
         err_msg( { "Error writing to " + mfile } )
      endif
      // save number of fields
      ngets := len(browse:getArray)
      fwrite(handle, chr(ngets))
      for xx := 1 to nGets
         fwrite(handle, chr(GetRow(xx)) + chr(GetCol(xx)) + ;
                        chr(GetLength(xx)) + padr(GetName(xx), 10)  + ;
                        GetPicture(xx) + CRLF)
      next
      // save number of boxes
      fwrite(handle, chr(nBoxes))
      if nBoxes > 0
         for xx := 1 to nBoxes
            fwrite(handle, chr(BoxTop(xx)) + chr(BoxLeft(xx)) + ;
                    chr(BoxBottom(xx)) + chr(BoxRight(xx)) + ;
                    BoxString(xx) + chr(BoxColor(xx)) + BoxFill(xx) )
         next
      endif
      fclose(handle)
   endif
endif
setcursor(SC_NONE)
return NIL

* end static function GFBMaint()
*--------------------------------------------------------------------*


/*
   IsItAGet(): are we sitting on a GET?
*/
static function IsItAGet(r, c, browse)
local nGets := len(browse:getArray)
local xx := 0
do while ++xx <= nGets .and. ( GetRow(xx) <> r .or. ;
                               c < GetCol(xx) .or. ;
                               c > GetCol(xx) + GetLength(xx) )
enddo
return (xx <= nGets)

* end static function IsItAGet()
*--------------------------------------------------------------------*


/*
   OnAGet(): is current GET overlapping another one?
*/
static function OnAGet(t, l, r, browse, nCurrGet)
local nGets := len(browse:getArray)
local xx := 0
local nRight
default nCurrGet to 0
do while ++xx <= nGets
   if GetRow(xx) == t .and. nCurrGet <> xx .and.                            ;
        ( ( l < GetCol(xx) .and. r > GetCol(xx) + GetLength(xx)   - 1) .or. ;
          ( l >= GetCol(xx) .and. l <= GetCol(xx) + GetLength(xx) - 1) .or. ;
          ( r >= GetCol(xx) .and. r <= GetCol(xx) + GetLength(xx) - 1)  )
      exit
   endif
enddo
return (xx <= nGets)

* end static function IsItAGet()
*--------------------------------------------------------------------*


#ifdef MOUSE

static function gfberase(browse)
local key
local aPos
local nTop := row()
local nLeft := col()
local nBottom := nTop
local nRight := nLeft
local buffer
local xx
local nGets  := len(browse:getArray)
local nBoxes := len(browse:boxArray)
setmousepos(nTop, nLeft)
enableDrag(.T.)       // Enable mouse dragging
buffer := savescreen(nTop, nLeft, nBottom, nRight)
do while ( key := inkey() ) <> K_LEFT_UP
   aPos := GetMousePos()
   if aPos[ 1 ] == nBottom .and. aPos[ 2 ] == nRight
      loop
   elseif aPos[ 1 ] < nTop .or. ;
          aPos[ 1 ] > maxrow() - 1 .or. ;
          aPos[ 2 ] < nLeft .or. ;
          aPos[ 2 ] > maxcol()
      SetMousePos( nBottom, nRight )
      loop
   endif
   dispbegin()
   restscreen(nTop, nLeft, nBottom, nRight, buffer)
   nBottom := aPos[1]
   nRight := aPos[2]
   buffer := savescreen(nTop, nLeft, nBottom, nRight)
   gfattr(nTop, nLeft, nBottom, nRight, 8)
   dispend()
enddo
enableDrag( .F. )       // Disable mouse dragging

dispbegin()
scroll(nTop, nLeft, nBottom, nRight)
// redraw all GETs
for xx := 1 to nGets
   @ GetRow(xx), GetCol(xx) ssay padr(GetName(xx), GetLength(xx)) color 'I'
next
// redraw all boxes
for xx := 1 to nBoxes
   @ BoxTop(xx), BoxLeft(xx), BoxBottom(xx), BoxRight(xx) ;
     box BoxString(xx) color BoxColor(xx)
next
dispend()

return nil

#endif


/*
   GfbCleanup(): clear GET fields in preparation for
                 saving .GFS file or generating code
*/
static function gfbcleanup(browse)
local xx, ngets := len(browse:getArray)
// we must go through the gets array and wipe all inverse
// gets off the screen -- if we leave them here, they will be
// restored when adding/editing only to be wiped out again,
// which creates an annoying little visual blip.
setcolor(browse:colorSpec)
for xx := 1 to nGets
   // if we have more than one screen of gets, we must add error-
   // trapping here so that the thing don't crash!!
   if valtype(browse:getArray[xx]) == "U"
      exit
   else
      scroll(GetRow(xx), GetCol(xx), GetRow(xx), GetCol(xx) + ;
             GetLength(xx) - 1, 0)
   endif
next
// get rid of the coordinates at the top right corner
scroll(0, maxcol() - 6, 0, maxcol(), 0)
return nil

* end static function GFBCleanUp()
*--------------------------------------------------------------------*


/*
   GfbHelpMe(): help screen for screen painting module
*/
static function gfbhelpme
gfsaveenv(.t., 0, '+w/b')          // shut off cursor & change color
@ 0, 0, maxrow(), maxcol() box BOXFRAMES[5] color "+gr/b"
@ 1, 19 ssay "GRUMPBROWSE() SCREEN PAINTER - ACTIVE KEYS" color "+gr/b"
@ 2, 16 ssay "insert field at current location"
@ 3, 16 ssay "draw box"
@ 4, 16 ssay "change current color"
@ 5, 16 ssay "toggle insert mode on/off"
@ 6, 16 ssay "destructive backspace"
@ 7, 16 ssay "delete from cursor position"
@ 8, 16 ssay "exit without saving changes"
@ 9, 16 ssay "save screen file (.gfs) and exit"
@10, 16 ssay "generate code for this screen"
@11, 16 ssay "move left one column"
@11, 54 ssay "move right one column"
@12, 16 ssay "move to top left"
@12, 54 ssay "move to bottom right"
@13, 16 ssay "move right five columns"
@13, 54 ssay "move left five columns"
@14, 16 ssay "move to top row"
@14, 54 ssay "move to bottom row"
@15, 16 ssay "move to left column"
@15, 54 ssay "move to right column"
@17, 4 ssay "To delete a box, place the cursor on its outline and press DEL or double"
@18, 4 ssay "click.  To seize a box, place the cursor on its outline and press Enter"
@19, 4 ssay "or left click.  To delete a GET, place the cursor on it and press DEL or"
@20, 4 ssay "double click.  To move a GET, place the cursor on it and press Enter or"
@21, 4 ssay "left click."
@23, 20 ssay "press any key to return to screen painter"
setcolor("i")
@ 2,  4 ssay "Alt-F"
@ 3,  4 ssay "Alt-B"
@ 4,  4 ssay "Alt-P"
@ 5,  4 ssay "Insert"
@ 6,  4 ssay "Backspace"
@ 7,  4 ssay "Delete"
@ 8,  4 ssay "Esc"
@ 9,  4 ssay "F10"
@10,  4 ssay "Alt-G"
@11,  4 ssay "LtArrow"
@11, 42 ssay "RtArrow"
@12,  4 ssay "Home"
@12, 42 ssay "End"
@13,  4 ssay "Tab"
@13, 42 ssay "Shift-Tab"
@14,  4 ssay "PgUp"
@14, 42 ssay "PgDn"
@15,  4 ssay "Ctrl-Left"
@15, 42 ssay "Ctrl-Right"
ginkey(0)
gfrestenv()
return nil

* end static function GFBHelpMe()
*--------------------------------------------------------------------*


/*
   GfbMoveFld(): logic to drag a field around on the screen
*/
static function gfbmovefld(browse, key)
local oldcolor := setcolor()
local nRow
local nCol
local nLength
local nColor
local yy
local buffer
local buffer2
local lRetVal := .f.
local xx := gfbwhichget(row(), col(), browse)

#ifdef MOUSE
local aPos
#endif

if xx > 0
   nRow  := GetRow(xx)
   nCol  := GetCol(xx)
   nLength := GetLength(xx)
   // initialize a scrap buffer which will hold the underlying
   // screen area as we drag the field around.  if we did not
   // do this, then the field would act as a giant erase,
   // annihilating static text (and other fields) in its wake!
   nColor := color_s2n( setcolor() )
   buffer := []
   for yy = 1 to nLength
      buffer += chr(32) + chr(nColor)
   next
   buffer2 := savescreen(nRow, nCol, nRow, nCol + nLength - 1)
   #ifdef MOUSE
   if key == K_LEFT_DOWN
      setmousepos(nRow, nCol)
      enableDrag(.T.)       // Enable mouse dragging
      do while ( key := inkey() ) <> K_LEFT_UP
         aPos := GetMousePos()
         if aPos[ 1 ] == nRow .and. aPos[ 2 ] == nCol
            loop
         elseif aPos[ 1 ] < 0 .or. ;
                aPos[ 1 ] > maxrow() .or. ;
                aPos[ 2 ] < 0 .or. ;
                aPos[ 2 ] + nLength - 1 > maxcol()
            SetMousePos( nRow, nCol )
            loop
         endif

         restscreen(nRow, nCol, nRow, nCol + nLength - 1, buffer)
         nRow := aPos[1]
         nCol := aPos[2]
         buffer := savescreen(nRow, nCol, nRow, nCol + nLength - 1)
         restscreen(nRow, nCol, nRow, nCol + nLength - 1, buffer2)
      enddo
      enableDrag( .F. )       // Disable mouse dragging
      key := K_ENTER
   else

   #endif

   key := 0
   do while key <> K_ENTER .and. key <> K_ESC
      @ 0, maxcol() - 6 ssay padr(ltrim(str(nRow, 2)) + ', ' + ;
                         ltrim(str(nCol, 3)), 7)
      setpos(nRow, nCol)
      key := ginkey(0)
      if key <> K_ENTER .and. key <> K_ESC
         setcolor(oldcolor)
         dispbegin()
         restscreen(nRow, nCol, nRow, nCol + nLength - 1, buffer)
         gfarrowkey(key, maxcol() + 1 - nLength, @nRow, @nCol)
         buffer := savescreen(nRow, nCol, nRow, nCol + nLength - 1)
         @ nRow, nCol say space(nLength) color "I"
         dispend()
         buffer2 := savescreen(nRow, nCol, nRow, nCol + nLength - 1)
      endif
   enddo

   #ifdef MOUSE
   endif
   #endif

   dispbegin()
   restscreen(nRow, nCol, nRow, nCol + nLength - 1, buffer)
   if key == K_ENTER
      lRetVal := .t.
      // make sure that we are not plopping this get down over
      // another one!
      if OnAGet(nRow, nCol, nCol + nLength - 1, browse, xx)
         dispend()
         err_msg( { "GETs cannot be stacked" } )
         lRetVal := .f.
      endif
   endif
   if lRetVal
      // change this array element to reflect the new position
      GetRow(xx) := nRow
      GetCol(xx) := nCol
      restscreen(nRow, nCol, nRow, nCol + nLength - 1, buffer2)
      dispend()
   else
      // either they escaped, or they tried to place this get
      // on top of another one -- in either case we must now
      // redraw the original get
      @ GetRow(xx), GetCol(xx) ssay padr(GetName(xx), GetLength(xx)) color 'I'
   endif

endif
setcolor(oldcolor)
return lRetVal

* end static function GFBMoveFld()
*--------------------------------------------------------------------*



/*
   IsItABox(): determine whether or not we are on a box outline
               useful for resizing and deleting boxes
*/
static function isitabox(browse)
local ret_val := 0, xx, temprow := row(), tempcol := col()
local nBoxes := len(browse:boxArray)
for xx := 1 to nBoxes
   if ((BoxTop(xx) == temprow .or. BoxBottom(xx) == temprow) .and.   ;
        BoxLeft(xx) <= tempcol .and. BoxRight(xx) >= tempcol) .or.   ;
        ((BoxLeft(xx) == tempcol .or. BoxRight(xx) == tempcol) .and. ;
           BoxTop(xx) <= temprow .and. BoxBottom(xx) >= temprow)
      ret_val := xx
      exit
   endif
next
return ret_val

* end static function IsItABox()
*--------------------------------------------------------------------*


/*
   GFBWhichGet(): locate current get in get array
*/
static function gfbwhichget(nRow, nCol, browse)
local mget := 0, xx
// figure out which field it is by scanning the array
// for the value of the row we are currently on
xx := ascan(browse:getArray, { | a | a[1] == nRow} )
do while xx > 0 .and. mget == 0
   // now check the length of this field (character 3 in
   // the array element) to see if we are actually on it
   if xx > 0
      if nCol < GetCol(xx) + GetLength(xx)
         mget := xx
      else
         xx := ascan(browse:getArray, { | a | a[1] == nRow }, xx + 1)
      endif
   endif
enddo
return mget

* end static function GFBWhichGet()
*--------------------------------------------------------------------*


/*
   GfbDrawBox(): logic to drag a field around on the screen
*/
static function gfbdrawbox(nRow, nCol, browse)
local buffer
local nBoxType
local key
local nTop
local nLeft
local oldscrn
local mwidth
local fillbuff
local xx
local scrnbuff := savescreen(0, maxcol() - 16, 0, maxcol())
local lRetVal := .f.
#ifdef MOUSE
local aPos
#endif
static aBoxTypes
// only load boxtypes array if it is empty (i.e., first time through)
if aBoxTypes == NIL
   aBoxTypes := array(5)
   for xx := 1 to 5
      aBoxTypes[xx] := substr(BOXFRAMES[xx], 1, 8)
   next
endif
/*
   even though it seems wasteful, we must make a safety copy of
   the entire screen because you might decide not to keep the
   box, right?  remember the old maxim: users change their mind
   as often as Cher changes costumes
*/
oldscrn := savescreen()
setpos(nRow, nCol)
apick nBoxType array aBoxTypes title "Boxes"
if nBoxType > 0
   key := 0
   nTop := nRow
   nLeft := nCol
   buffer := savescreen(nTop, nLeft, nRow, nCol)
   do while key <> K_ENTER .and. key <> K_ESC
      // display current box coordinates at top right corner
      // but only if the box is not in the top right corner!!
      if nTop > 0 .or. nCol < maxcol() - 16
         @ 0, maxcol() - 16 ssay padr(ltrim(str(nTop, 2)) + ', '  + ;
                                      ltrim(str(nLeft, 3)) + ', ' + ;
                                      ltrim(str(nRow, 2)) + ', '  + ;
                                      ltrim(str(nCol, 3)), 16)
      endif
      setpos(nRow, nCol)
      key := ginkey(0)

      #ifdef MOUSE
      if key == K_LEFT_DOWN
         setmousepos(nRow, nCol)
         enableDrag(.T.)       // Enable mouse dragging
         do while ( key := inkey() ) <> K_LEFT_UP
            aPos := GetMousePos()
            if aPos[ 1 ] == nRow .and. aPos[ 2 ] == nCol
               loop
            elseif aPos[ 1 ] < nTop .or. ;
                   aPos[ 1 ] > maxrow() .or. ;
                   aPos[ 2 ] < nLeft .or. ;
                   aPos[ 2 ] > maxcol()
               SetMousePos( nRow, nCol )
               loop
            endif
            dispbegin()
            restscreen(nTop, nLeft, nRow, nCol, buffer)
            nRow := aPos[1]
            nCol := aPos[2]
            buffer := savescreen(nTop, nLeft, nRow, nCol)
            @ nTop, nLeft, nRow, nCol box aBoxTypes[nBoxType]
            // redisplay box coordinates status
            if nTop > 0 .or. nCol < maxcol() - 16
               @ 0, maxcol() - 16 ssay padr(ltrim(str(nTop, 2)) + ', '  + ;
                                       ltrim(str(nLeft, 3)) + ', ' + ;
                                       ltrim(str(nRow, 2)) + ', '  + ;
                                       ltrim(str(nCol, 3)), 16)
            endif
            setpos(nRow, nCol)
            dispend()
         enddo
         enableDrag( .F. )       // Disable mouse dragging
         key := K_ENTER
      else
      #endif

      if ! ((nRow == maxrow() .and. key == K_DOWN) .or. ;
            (nRow == nTop .and. key == K_UP) .or. ;
            (nCol == nLeft .and. key == K_LEFT) .or. ;
            (nCol > maxcol() - 1 .and. key == K_RIGHT))
         dispbegin()
         restscreen(nTop, nLeft, nRow, nCol, buffer)
         gfarrowkey(key, maxcol(), @nRow, @nCol)
         buffer := savescreen(nTop, nLeft, nRow, nCol)
         @ nTop, nLeft, nRow, nCol box aBoxTypes[nBoxType]
         dispend()
      endif

      #ifdef MOUSE
      endif
      #endif

   enddo
   if key == K_ESC
      restscreen(,,,, oldscrn)
      // restore original cursor position too
      setpos(nTop, nLeft)
   else
      // restore screen contents under the coordinates area (top right)
      restscreen(0, maxcol() - 16, 0, maxcol(), scrnbuff)
      lRetVal := .t.
      aadd(browse:boxArray, { nTop, nLeft, nRow, nCol, aBoxTypes[nBoxType], ;
                              color_s2n(), "N" } )
   endif
endif
return lRetVal

* end static function GFBDrawBox()
*--------------------------------------------------------------------*


/*
   GfbSizeBox(): logic to resize a box
*/
static function gfbsizebox(browse, xx)
local oldscrn
local key := 0
local buffer
local oldcolor
local lRetVal
local nTop    := BoxTop(xx)
local nLeft   := BoxLeft(xx)
local nBottom := BoxBottom(xx)
local nRight  := BoxRight(xx)
#ifdef MOUSE
local aPos
#endif

// once again, we must make a safety copy of the entire screen
// because you might decide to leave the box size as is.  picky users!
oldscrn := savescreen()

// get rid of the box for a split second and save the underlying screen
// otherwise, we are going to have a real mess on our hands!
dispbegin()
@ nTop, nLeft, nBottom, nRight box space(8)
buffer := savescreen(nTop, nLeft, nBottom, nRight)
oldcolor := setcolor(color_n2s(BoxColor(xx)))
@ nTop, nLeft, nBottom, nRight box BoxString(xx)
dispend()

do while key <> K_ENTER .and. key <> K_ESC
   setpos(nBottom, nRight)
   key := ginkey(0)

   #ifdef MOUSE
   if key == K_LEFT_DOWN
      setmousepos(nBottom, nRight)
      enableDrag(.T.)       // Enable mouse dragging
      do while ( key := inkey() ) <> K_LEFT_UP
         aPos := GetMousePos()
         if aPos[ 1 ] == nBottom .and. aPos[ 2 ] == nRight
            loop
         elseif aPos[ 1 ] < nTop .or. ;
                aPos[ 1 ] > maxrow() .or. ;
                aPos[ 2 ] < nLeft .or. ;
                aPos[ 2 ] > maxcol()
            SetMousePos( nBottom, nRight )
            loop
         endif
         dispbegin()
         restscreen(nTop, nLeft, nBottom, nRight, buffer)
         nBottom := aPos[1]
         nRight := aPos[2]
         buffer := savescreen(nTop, nLeft, nBottom, nRight)
         @ nTop, nLeft, nBottom, nRight box BoxString(xx)
         dispend()
      enddo
      enableDrag( .F. )       // Disable mouse dragging
      key := K_ENTER
   else
   #endif

   if ! ((nBottom == maxrow() .and. key == K_DOWN) .or. ;
         (nBottom == nTop .and. key == K_UP) .or. ;
         (nRight == nLeft .and. key == K_LEFT) .or. ;
         (nRight > maxcol() - 1 .and. key == K_RIGHT))
      dispbegin()
      restscreen(nTop, nLeft, nBottom, nRight, buffer)
      gfarrowkey(key, maxcol(), @nBottom, @nRight)
      buffer := savescreen(nTop, nLeft, nBottom, nRight)
      @ nTop, nLeft, nBottom, nRight box BoxString(xx)
      dispend()
   endif

   #ifdef MOUSE
   endif
   #endif

enddo
if key == K_ESC
   restscreen(,,,, oldscrn)
   lRetVal := browse:boxArray[xx]
else
   lRetVal := { nTop, nLeft, nBottom, nRight, BoxString(xx), BoxColor(xx), ;
                BoxFill(xx) }
endif
setcolor(oldcolor)
return lRetVal

* end static function GFBSizeBox()
*--------------------------------------------------------------------*


/*
   GfbArrowKey(): process arrow keypresses etcetera
*/
static function gfarrowkey(key, maxlength, nRow, mcol)
do case
   case key == K_UP
      if nRow == 0
         nRow := maxrow()
      else
         nRow--
      endif

   case key == K_DOWN
      if nRow == maxrow()
         nRow := 0
      else
         nRow++
      endif

   case key == K_LEFT .and. mcol > 0
      mcol--

   case key == K_RIGHT .and. mcol < maxlength
      mcol++

   case key == K_HOME
      nRow := mcol := 0

   case key == K_END
      nRow := maxrow()
      mcol := maxlength

   case key == K_PGUP
      nRow := 0

   case key == K_PGDN
      nRow := maxrow()

   case key == K_CTRL_LEFT
      mcol := 0

   case key == K_CTRL_RIGHT
      mcol := maxlength

   case key == K_TAB
      mcol := min(mcol + 5, maxlength)

   case key == K_SH_TAB
      mcol := max(mcol - 5, 0)

endcase
return NIL

* end static function GFBArrowKey()
*--------------------------------------------------------------------*

#endif // REMBRANDT


/*
   JumpDown(): jump to beginning of next subgroup (active index only!)
*/
static function jumpdown(browse, nOrder)
local val := eval(browse:indexBlock)
local marker := recno()
local searchval
if valtype(val) == "C"
   searchval := substr(val, 1, len(val) - 1) + chr( asc(right(val, 1)) + 1 )
elseif valtype(val) $ "DN"
   searchval := val + 1
endif

// only perform the jump if we are not passing the end of the data subset
if empty(browse:endValues[nOrder]) .or. browse:endValues[nOrder] >= searchval
   dbseek(searchval, .t.)
   if eof()
      go marker
   else
      browse:refreshAll()
   endif
endif
return nil

* end static function JumpDown()
*--------------------------------------------------------------------*


/*
   JumpUp(): jump to beginning of previous subgroup (active index only!)
*/
static function jumpup(browse, nOrder)
local val
// only perform the jump if we are not passing the start of the data subset
if empty(browse:startValues[nOrder]) .or. ;
               eval(browse:indexBlock) > browse:startValues[nOrder]
   dbseek(eval(browse:indexBlock), .t.)
   dbskip(-1)
   val := eval(browse:indexBlock)
   browse:goTop()
   dbseek(val, .t.)
   browse:refreshAll()
endif
return nil

* end static function JumpUp()
*--------------------------------------------------------------------*


// the next five functions are for viewing subsets with the
// three TBrowse navigation blocks...

static function pseudofilt(browse, lNeedInput, ;
                           bOldgotop, bOldgobott, bOldskip, nOrder)
local startval
local endval
local bOldHandler
local bNewHandler
local newblock
local getlist := {}
local oldscrn
local initval
local cpicture
local lReset := .t.
if lNeedInput
   initval := eval(browse:indexBlock)
   oldscrn := shadowbox(7, 9, 10, 70, 1, "View Subset")
   startval := if(empty(browse:startValues[nOrder]),initval,  ;
               if(valtype(browse:startValues[nOrder]) <> "C", ;
                          browse:startValues[nOrder],         ;
                          padr(browse:startValues[nOrder], 40)))
   endval   := if(empty(browse:endValues[nOrder]), initval,   ;
               if(valtype(browse:endValues[nOrder]) <> "C",   ;
                          browse:endValues[nOrder],           ;
                          padr(browse:endValues[nOrder], 40)))

   // check for long character fields... must limit to 40 spaces
   cpicture := "@K" + ;
               if(valtype(initval) == "C" .and. len(initval) > 40, "@S40", '')
   @ 8, 11 say "Enter low value:  " get startval picture cpicture ;
                                    valid changeendval(getlist)
   @ 9, 11 say "Enter high value: " get endval picture cpicture
   setcursor(CURSOR_ON)
   read
   setcursor(SC_NONE)
   byebyebox(oldscrn)

   if lastkey() <> K_ESC
      if valtype(initval) == "C"    // trim if character type
         browse:startValues[nOrder] := trim(startval)
         browse:endValues[nOrder]   := trim(endval)
      else
         browse:startValues[nOrder] := startval
         browse:endValues[nOrder]   := endval
      endif
   endif
endif

if lastkey() <> K_ESC .or. ! lNeedInput
   if ! empty(browse:startValues[nOrder]) .or. ! empty(browse:endValues[nOrder])
      bNewHandler = { | e | blockhead(e, bOldHandler) }
      bOldHandler = errorblock(bNewHandler)
      begin sequence
         if ! empty(browse:startValues[nOrder])
            browse:goTopBlock := &(" { | | dbseek(" + ;
                         convertit(browse:startValues[nOrder]) + ", .t.) }")
         else
            browse:goTopBlock := bOldgotop
         endif
         if ! empty(browse:endValues[nOrder])
            browse:goBottomBlock := &(" { | | dbseek(" +             ;
                                    convertit(browse:endValues[nOrder], .t.) + ;
                                    ", .t.) , dbskip(-1) }")
         else
            browse:goBottomBlock := bOldgobott
         endif
         browse:skipBlock := { | SkipCnt | Gilligan(SkipCnt, browse, nOrder) }
         browse:goTop()           // force "filter" to be set
         // verify that data is within range
         initval := eval(browse:indexBlock)
         lReset := ( initval < browse:startValues[nOrder] .or. ;
                     initval > browse:endValues[nOrder] )
         if lReset
            waiton("No data in specified range... viewing all data")
            inkey(2)
            waitoff()
         endif
      end
      errorblock(bOldHandler)            // reset previous error handler
   else
      browse:startValues[nOrder] := browse:endValues[nOrder] := NIL
   endif
   if lReset
      // reset original movement blocks
      browse:goTopBlock    := bOldgotop
      browse:goBottomBlock := bOldgobott
      browse:skipBlock     := bOldskip
      browse:goTop()           // force "filter" to be set
   endif
endif
return nil

* end static function PseudoFilt()
*--------------------------------------------------------------------*


/*
   Function:  ChangeEndVal()
   Purpose:   Change ending value for data subset to match starting value
*/
static function ChangeEndVal(getlist)
if getlist[1]:changed
   getlist[2]:varPut( getlist[1]:varGet() )
endif
return .t.

* end static function ChangeEndVal()
*--------------------------------------------------------------------*


/*
   Function: Gilligan()
   Purpose:  Skipblock function used for data subsets
*/
static function gilligan(skipcnt, browse, norder)
local movement := 0
local bIndex := browse:indexBlock
static lReset := .f.
do case
   // no movement
   case skipcnt == 0
      skip 0
   // moving forward
   case skipcnt > 0
      do while movement < skipcnt .and. eval(bIndex) <= browse:endValues[norder] ;
               .and. ! eof()
         skip 1
         movement++
      enddo
      // make sure that we are within range - if not, move backward
      do while (eval(bIndex) > browse:endValues[norder] .or. eof()) .and. ! bof()
         skip -1
         movement--
      enddo
      // ensure once again that data is within range
      if bof() .or. eval(bIndex) < browse:startValues[norder]
         lReset := .t.
      endif
   // moving backward
   case skipcnt < 0
      do while movement > skipcnt .and. eval(bIndex) >= browse:startValues[norder]
         skip -1
         if bof()
            exit
         endif
         movement--
      enddo
      // make sure that we are within range -- if not, move forward
      do while eval(bIndex) < browse:startValues[norder] .and. ! eof()
         skip
         movement++
      enddo
      // ensure once again that data is within range
      if eof() .or. eval(bIndex) > browse:endValues[norder]
         lReset := .t.
      endif
endcase
if lReset
   err_msg({ "No data within this subset... viewing all records" })
   browse:startValues[norder] := NIL
   browse:endValues[norder] := NIL
   browse:goTopBlock := { || dbgotop() }
   browse:goBottomBlock := { || dbgobottom() }
   browse:skipBlock := browse:origSkipBlock
   lReset := .f.
endif
return movement

* end static function Gilligan()
*--------------------------------------------------------------------*


/*
   do type conversion for character/numeric/date index keys
   in preparation for compiling goTopBlock and goBottomBlock
   2nd parameter passed to increment low value for SOFTSEEK
*/
static function convertit(val, lo)
local ret_val := valtype(val)
local cdelimiters
do case
   case ret_val == "D"
      ret_val := "CTOD('" + dtoc(val + if(lo <> NIL, 1, 0)) + "')"
   case ret_val == "N"
      ret_val := ltrim(str(val + if(lo <> NIL, 1, 0)))
   case ret_val == "C"
      // if 2nd parameter was passed, then increment the rightmost
      // character in the string by 1 ASCII value.  This is required
      // so that the record pointer will be properly positioned...
      cdelimiters := if("'" $ val, { "[", "]" }, { "'", "'" })
      if lo <> NIL
         ret_val := cdelimiters[1] + left(val, len(val) - 1) + ;
                    chr(asc(right(val, 1)) + 1) + cdelimiters[2]
      else
         ret_val := cdelimiters[1] + val + cdelimiters[2]
      endif
endcase
return ret_val

* end static function ConvertIt()
*--------------------------------------------------------------------*


/*
   BlockHead(): custom error handler for code block syntax bombs
*/
static function blockhead(e, bOldHandler)
if e:gencode() == EG_SYNTAX .or. e:gencode() == EG_ARG
   dispend()
   err_msg( { "Error in code block syntax" } )
   break
   return .t.
endif
return eval(bOldHandler, e)

* end static function BlockHead()
*--------------------------------------------------------------------*


/*
   MakePicture(): derive default picture and picture length for a field
*/
static function makepicture(aField, pic)
local piclen
local x
do case
   case aField[DBS_TYPE] == 'D'
      if pic == chr(254)
         pic := "@D"
      endif
      piclen := 8

   case aField[DBS_TYPE] == 'M'
      pic := "<memo>"
      piclen := 6

   case aField[DBS_TYPE] == 'C'
      if pic == chr(254)
         pic := replicate("X", aField[DBS_LEN])
      endif
      piclen := aField[DBS_LEN]

   case aField[DBS_TYPE] == 'L'
      if pic == chr(254)
         pic := "Y"
         piclen := 1
      else
         piclen := len(pic)
      endif

   otherwise
      if pic == chr(254)
         x := str(fieldget(fieldpos(aField[DBS_NAME])))
         if "." $ x
            pic := replicate('9', at(".", x) - 1) + "."
            pic += replicate('9', len(x) - len(pic))
         else
            pic := replicate('9', len(x))
         endif
      endif
      piclen := len(pic)
endcase
return { pic, piclen }

* end static function MakePicture()
*--------------------------------------------------------------------*

// the following manifest constant sets spacing between printed columns
#define COLUMN_SPACING   1

/*
   PrintIt(): print all or some records
*/
static function printit(obrowse)
local marker := recno()
local ncolumns := obrowse:colCount
local mainloop  := .t.
local keepgoing := .t.
local lFiltered := .f.
local x
if ! empty(obrowse:queryCondition)
   lFiltered := ! Yes_No2({ "Print all records or those matching query?" }, ;
                             maxrow()/2 - 2, " All ", " Filtered ")
endif
if printok()
   eval(obrowse:goTopBlock)
   if lFiltered
      waiton({ "Locating first matching record... press ESC to exit" } )
      do while ( mainloop := eval(obrowse:skipBlock, 1) <> 0 ) ;
         .and. ! eval(obrowse:queryCondition) .and. checkescape()
      enddo
      waitoff()
      if ! mainloop
         err_msg({ "No matching records found" })
         keepgoing := .f.
      elseif ! eval(obrowse:queryCondition)   // they escaped out
         keepgoing := .f.
      endif
   endif
   if keepgoing
      waiton({ "Printing... press ESC to exit" } )
      printheadings(obrowse)
      do while mainloop .and. checkescape()
         for x := 1 to ncolumns
            printitem( eval(obrowse:getColumn(x):block), ;
                       obrowse:colWidth(x))
         next
         if prow() > 57
            eject
            printheadings(obrowse)
         else
            devpos( prow()+1, 0 )
         endif
         if lFiltered
            do while ( mainloop := eval(obrowse:skipBlock, 1) <> 0 ) ;
                       .and. ! eval(obrowse:queryCondition)
            enddo
         else
            mainloop := ( eval(obrowse:skipBlock, 1) <> 0 )
         endif
      enddo
      if mainloop
         devout("*** interrupted ***")
      endif
      eject
      set(_SET_DEVICE, "SCREEN")
      waitoff()
   endif
   go marker
endif
return nil

* end static function Printit()
*--------------------------------------------------------------------*


/*
   CheckEscape(): test for ESC key pressed during output process
*/
static function CheckEscape
local ret_val := .t.
if inkey() == K_ESC
   // don't allow to user to ESC out of this -- force them to confirm
   ret_val := ! Yes_No2({ "Are you sure you want to cancel output?" }, ;
                           maxrow()/2 - 2, " Yes ", " No ", .f.)
endif
return ret_val

* end static function CheckEscape()
*--------------------------------------------------------------------*


/*
   PrintItem() - pad each item to column width
*/
static function printitem(data, width)
local ctype := valtype(data)
do case
   case ctype == "C"
      devout( padr(data, width + COLUMN_SPACING ) )
   case ctype == "L"
      devout( padr(if(data, "T", "F"), width + COLUMN_SPACING ) )
   case ctype == "N"
      devout( padl(ltrim(str(data)), width) )
      devout( space(COLUMN_SPACING) )
   case ctype == "D"
      devout( padl(dtoc(data), width) )
      devout( space(COLUMN_SPACING) )
endcase
return nil

* end static function PrintItem()
*--------------------------------------------------------------------*

/*
   PrintHeadings() - print column headings
*/
static function printheadings(obrowse)
local num_columns := obrowse:colCount
local x, cheading
for x := 1 to num_columns
   if ( cheading := obrowse:getColumn(x):heading ) <> NIL
      devout(padr(cheading, obrowse:colWidth(x) + COLUMN_SPACING))
   else
      devout(space(obrowse:colWidth(x) + COLUMN_SPACING))
   endif
next
devpos( prow() + 1, 0)
for x := 1 to num_columns
   devout(replicate('-', obrowse:colWidth(x)) + space(COLUMN_SPACING))
next
devpos( prow() + 1, 0 )
return nil

* eof brow.prg
