/*
    Program: DRAPES.PRG
    System: GRUMPFISH LIBRARY
    Author: Greg Lief
    Copyright (c) 1988-93, Greg Lief
    CA-Clipper 5.x Version
    Compile instructions: clipper drapes /n /w

    Procs & Fncts: PULL_DRAPE()
                 : SAVE_DRAPE()
*/

#include "grump.ch"
#include "fileio.ch"

/*
   Function: PULL_DRAPE()
   Purpose:  Draw title screen from specified memory file
*/
function Pull_Drape(cFile, nDelay)
local nHandle
local aScreen := {}
local buffer := space(4)
local xx
local yy
local midpoint
local nTop := 0
local nLeft := 0
local nBottom := maxrow()
local nRight := maxcol()
local nWidth
local nLength
local nOldCurs := setcursor(0)
default nDelay to 20
if file(cFile)
   nHandle := fopen(cFile, FO_READ)
   // determine file length
   nLength := fseek(nHandle, 0, FS_END)
   // reset pointer to beginning of file
   fseek(nHandle, FS_SET)
   // we must accommodate for .scr files created by
   // if this is not a full screen
   if nLength <> 4000
      fread(nHandle, @buffer, 4)
      nTop    := bin2i(substr(buffer, 1, 1))
      nLeft   := bin2i(substr(buffer, 2, 1))
      nBottom := bin2i(substr(buffer, 3, 1))
      nRight  := bin2i(substr(buffer, 4, 1))
   endif
   nWidth := ((nBottom - nTop) + 1) * 2
   yy := nRight - nLeft + 1
   asize(aScreen, yy)
   for xx = 1 to yy
      buffer := space(nWidth)
      fread(nHandle, @buffer, nWidth)
      aScreen[xx] := buffer
   next
   fclose(nHandle)
   midpoint := nLeft + int((nRight - nLeft + 1) / 2) + 1
   #ifdef MOUSE
   HideMouseCursor()
   #endif
   for xx = midpoint to nRight + 1
      restscreen(nTop, xx - 1, nBottom, xx - 1, aScreen[xx - nLeft])
      restscreen(nTop, nRight + 1 + nLeft - xx, nBottom, ;
                 nRight + 1 + nLeft - xx, aScreen[nRight + 2 - xx])
      for yy = 1 to nDelay
      next
   next
   #ifdef MOUSE
   ShowMouseCursor()
   #endif
endif
setcursor(nOldCurs)
return NIL

* end function Pull_Drape()
*--------------------------------------------------------------------*


/*
     Function: SAVE_DRAPE()
     Purpose:  Save title screen to specified memory file
*/
function save_drape(cFile, nTop, nLeft, nBottom, nRight)
local buffer
local nHandle := fcreate(cFile)
local xx
local ret_val := .f.
default nTop to 0
default nLeft to 0
default nBottom to maxrow()
default nRight to maxcol()
if ferror() = 0
   #ifdef MOUSE
   HideMouseCursor()
   #endif
   fwrite(nHandle, chr(nTop) + chr(nLeft) + chr(nBottom) + chr(nRight))
   ret_val := .t.
   for xx = nLeft to nRight
      buffer := savescreen(nTop, xx, nBottom, xx)
      if fwrite(nHandle, buffer) <> ((nBottom - nTop) + 1) * 2
         ret_val := .f.
         exit
      endif
   next
   fclose(nHandle)
   #ifdef MOUSE
   ShowMouseCursor()
   #endif
endif
return ret_val

* end function Save_Drape()
*--------------------------------------------------------------------*

* eof drapes.prg
