/*
    Program: ERR_MSG()
    System: GRUMPFISH LIBRARY
    Author: Greg Lief
    Copyright (c) 1988-93, Greg Lief
    CA-Clipper 5.x Version
    Compile instructions: clipper errmsg /n /w
    Compile with "/dMOUSE" for mouseability
    Displays error message and waits for user keypress
*/

#include "grump.ch"

function err_msg(aMsgs, cBoxColor, cTextColor, lSound, cTitle)
local cOldscrn
local x := 0
local nLen
local nLeftCol
local nMidRow := int( ( maxrow() + 1) / 2 )
local nTop
local cBuffer

#ifdef MOUSE
   local aPos
   local lResetMouse := .f.
#endif

GFSaveEnv(, 0)                   // shut off cursor

// establish box and text colors if not passed as parameters
default cBoxColor to ColorSet(C_ERRORBOX, .t.)
default cTextColor to ColorSet(C_ERRORMESSAGE, .t.)
default lSound to .t.


// if title message was not passed, check current
// date format to determine language
if cTitle == NIL
   cBuffer := left(set(_SET_DATEFORMAT), 8)
   if cBuffer == "dd.mm.yy"   // SET DATE GERMAN
      cTitle := "Fehler"
   /*
      SET DATE BRITISH and FRENCH have the same date format.  English
      is the default.  If you wish to French instead of English,
      recompile this PRG with the following command-line syntax:

         clipper errmsg /dFRENCH (add /dMOUSE if you need the mouse)
   */
   elseif cBuffer == "dd/mm/yy"
      #ifndef FRENCH
      cTitle := "Error"
      #else
      cTitle := "Erreur"
      #endif
   else
      cTitle := "Error"
   endif
endif

// if message passed was a character string, convert to array
if valtype(aMsgs) == "C"
   aMsgs := { aMsgs }
endif

// add keypress msg to array
#ifdef MOUSE
   aadd( aMsgs, "** press any key or click mouse to continue **" )
#else
   aadd( aMsgs, "** press any key to continue **" )
#endif

nLen := len(aMsgs)

// determine width of longest message in array
aeval(aMsgs, { | a | x := max(len(a), x) } )

nLeftCol := int(maxcol() - 5 - max(x, 35)) / 2
nTop := int( ( maxrow() + 1 - nLen ) / 2 ) - 1

dispbegin()

#ifdef MOUSE

// move mouse cursor out of screen drawing area if necessary
aPos := GetMousePos()
if aPos[1] >= nTop .and. aPos[1] <= nTop + 2 + nLen .and. aPos[2] >= nLeftCol ;
                   .and. aPos[2] <= maxcol() - nLeftCol + 1
   lResetMouse := .t.
   SetMousePos(nTop - 1, nLeftCol - 1)
endif
#endif

cOldScrn := shadowbox(nTop, nLeftCol, nTop + 1 + nLen, ;
            maxcol() - nLeftCol, 3, cTitle, , cBoxColor)

setcolor(cTextColor)
for x := 1 to nLen
   scrncenter(nTop + x, aMsgs[x])
next

dispend()

#ifdef MOUSE

if lResetMouse
   SetMousePos(aPos[1], aPos[2])
endif

#endif

if lSound
   tone(MUSIC_ERROR, 1)
   tone(MUSIC_ERROR, 1)
endif

ginkey( 0 )

GFRestEnv()
byebyebox(cOldscrn)
return nil

* end function Err_Msg()
*--------------------------------------------------------------------*

* eof errmsg.prg
