/*
    Function: GETDATE()
    System: GRUMPFISH LIBRARY
    Author: Greg Lief
    Copyright (c) 1988-93, Greg Lief
    CA-Clipper 5.x version
    Compile instructions: clipper getdate /n/w/a

    Procs & Funcs: GETDATE()

          Calls: CALENDBOX()    (function  in CALENDAR.PRG)
               : NEXTMONTH()    (function  in CALENDAR.PRG)
*/

#define GRUMP_CALENDAR   // see GRUMP.CH
#include "grump.ch"
#include "inkey.ch"

#define CURRENTYEAR     substr(dtos(dDate), 1, 4)
#define CURRENTDAY      substr(dtos(dDate), 7)

function getdate(dDate, nTop, nLeft, cMainColor, lHolidays)
local row
local tdate
local lastday
local newrow
local newcol
local tempdate
local datecoords[10]
local xx
local lRedraw
local olddelete := set(_SET_DELETED, .T.)
local nKey
local oGet
local dRetVal
local nWorkArea
local lOpenHolidays := .f.
#ifdef MOUSE
local nMRow
local nMCol
#endif

// note: many people use GETDATE() in conjunction with GRUMPBROW(),
// which remaps the UP and DOWN arrows to do something completely
// different.  Therefore, we will temporarily shut them off so that
// they do what we expect them to do in the context of this function.
local bOldUp   := setkey(K_UP, NIL)
local bOldDown := setkey(K_DOWN, NIL)

// use system date as starting date if it was not passed as parameter
// also see if there is an active GET (used below)
if empty(dDate)
   dDate := date()
   oGet  := getactive()
endif

// determine screen position and color if not passed as parameters
default nTop  to int(maxrow() / 2) - 6
default nLeft to int( maxcol() / 2) - 12
default cMainColor to ColorSet(C_CALENDAR, .T.)

default lHolidays to .t.

// only open HOLIDAYS.DBF if it is not already open!
if lHolidays .and. select("holidays") == 0
   nWorkArea := select()
   if ! file("holidays.dbf")
      dbcreate("holidays.dbf", { { "DATE", "D", 8, 0 } } )
      use holidays new
      index on holidays->date to holidays
      dbclosearea()
      lHolidays := .f.
   elseif ! file('holidays' + indexext()) .and. (lHolidays := net_use('holidays', .t.))
      index on holidays->date to holidays
      dbclosearea()
      lHolidays := lOpenHolidays := net_use('holidays', .f., 'holidays')
   else
      lHolidays := lOpenHolidays := net_use('holidays', .f., 'holidays')
   endif
   select (nWorkArea)
endif

GFSaveEnv( { nTop, nLeft, nTop + 13, nLeft + 23 }, 0 )  // shut off cursor

CalendBox(.f., nTop, nLeft, dDate, cMainColor, .t., datecoords)

#ifdef MOUSE
ShowMouseCursor()
#endif

while nKey <> K_ENTER .and. nKey <> K_ESC
   lRedraw := .f.
   nKey := ginkey(0)
   tdate := dDate                  // store highlighted date
   newrow := CURRENTDAY_ROW
   newcol := CURRENTDAY_COL

   #ifdef MOUSE
   if nKey == K_LEFT_DBLCLICK
      nMRow := GFLastRow()
      nMCol := __FixMCol( nLeft )

      if nMRow == CURRENTDAY_ROW .and. nMCol == CURRENTDAY_COL
         nKey := K_ENTER
      else
         nKey := K_LEFT_DOWN
      endif
   endif
   #endif

   do case
   #ifdef MOUSE
   case nKey == K_RIGHT_DOWN
      nKey := K_ESC
      loop

   case nKey == K_LEFT_DOWN
      nMRow := GFLastRow()
      nMCol := __FixMCol( nLeft )

      if nMRow >= nTop + 5 .and. nMRow <= nTop + 11
         if nMCol == nLeft +  1 .or. nMCol == nLeft +  4 .or. ;
            nMCol == nLeft +  7 .or. nMCol == nLeft + 10 .or. ;
            nMCol == nLeft + 13 .or. nMCol == nLeft + 16 .or. ;
            nMCol == nLeft + 19

            dDate  += ( ( ( nMRow - CURRENTDAY_ROW ) * 7 ) + ;
                        ( ( nMCol - CURRENTDAY_COL ) / 3 ) )

            newrow := nMRow
            newcol := nMCol
         endif
      endif
   #endif

   case nKey == K_DOWN .or. nKey == K_UP   // forward/backward one week
      dDate += if(nKey = 24, 7, -7)
      newrow  += if(nKey = 24, 1, -1)

   case nKey == K_LEFT         // go back one day
      dDate--
      // did we just go from sunday to saturday??
      newrow := if(dow(dDate) = 7, CURRENTDAY_ROW - 1, CURRENTDAY_ROW)
      newcol := if(dow(dDate) = 7, nLeft + 19, CURRENTDAY_COL - 3)

   case nKey == K_RIGHT       // go forward one day
      dDate++
      // did we just go from saturday to sunday??
      newrow := if(dow(dDate) = 1, CURRENTDAY_ROW + 1, CURRENTDAY_ROW)
      newcol := if(dow(dDate) = 1, nLeft + 1, CURRENTDAY_COL + 3)

   case nKey == K_PGUP        // go back one month
      if month(dDate) == 1        // going to December of previous year
         dDate := stod( str(val(CURRENTYEAR)-1, 4) + '12' + CURRENTDAY)
      else
         // check for validity of current date in previous month
         // i.e., cannot go from march 31 to february 31, etcetera
         tempdate := ctod('')
         do while empty(tempdate)
            tempdate := stod(CURRENTYEAR + if(month(dDate) < 11, '0', '') + ;
                        ltrim(str(month(dDate) - 1)) + CURRENTDAY)
            dDate--
         enddo
         dDate := tempdate
      endif

   case nKey == K_PGDN        // go forward one month
      dDate := NextMonth(dDate)

   case nKey == K_CTRL_PGUP   // go back one year
      if month(dDate) == 2 .and. day(dDate) == 29
         dDate--
      endif
      xx := dtos(dDate)
      dDate := stod( str( val(substr(xx, 1, 4)) - 1, 4) + ;
                     substr(xx, 5) )
      lRedraw := .t.

   case nKey == K_CTRL_PGDN   // go forward one year
      if month(dDate) == 2 .and. day(dDate) == 29
         dDate++
      endif
      xx := dtos(dDate)
      dDate := stod( str( val(substr(xx, 1, 4)) + 1, 4) + ;
                     substr(xx, 5) )
      lRedraw := .t.

   case nKey == K_HOME        // go to first day
      newrow := FIRSTDAY_ROW
      newcol := FIRSTDAY_COL
      dDate := stod(substr(dtos(dDate), 1, 6) + '01')

   case nKey == K_END         // go to last day
      newrow := LASTDAY_ROW
      newcol := LASTDAY_COL
      dDate := stod(substr(dtos(dDate), 1, 6) + str(LASTDAY_NUMBER, 2))

   otherwise                  // any other keystroke
      loop

   endcase
   // if we changed months, lRedraw calendar
   if month(tdate) <> month(dDate) .or. lRedraw
      CalendBox(.f., nTop, nLeft, dDate, cMainColor, , datecoords)
   else
      // check if previous date was a holiday
      if lHolidays .and. holidays->( dbseek(tdate) )
         @ CURRENTDAY_ROW, CURRENTDAY_COL ssay str(day(tdate), 2) color 'w/r'
      else
         @ CURRENTDAY_ROW, CURRENTDAY_COL ssay str(day(tdate),2) ;
                                          color '+' + cMainColor
      endif

      // check if new date is a holiday
      if lHolidays .and. holidays->( dbseek(dDate) )
         @ newrow, newcol ssay str(day(dDate), 2) color '*+w/r'
      else
         @ newrow, newcol ssay str(day(dDate), 2) color '*+' + cMainColor
      endif

      // store new row/column coordinates for highlighted date
      CURRENTDAY_ROW := newrow
      CURRENTDAY_COL := newcol
   endif
enddo
GFRestEnv()
// close HOLIDAYS.DBF if we had to open it
if lOpenHolidays
   holidays->( dbCloseArea() )
endif
set(_SET_DELETED, olddelete)  // reset previous DELETED status
setkey(K_UP, bOldUp)
setkey(K_DOWN, bOldDown)
#ifdef MOUSE
if nKey == K_ENTER .or. nKey == K_LEFT_DBLCLICK
#else
if nKey == K_ENTER
#endif
   dRetVal := dDate
   // if no parameter was passed and there is an active GET,
   // manipulate it directly (this gets us around the problem
   // of passing by reference)
   if oGet <> NIL
      oGet:varPut( dDate )
   endif
else
   dRetVal := ctod('')
endif
return dRetVal


#ifdef MOUSE

function __FixMCol( nLeft )

local nMCol := GFLastCol()

if nMCol == nLeft +  2 .or. nMCol == nLeft +  5 .or. ;
   nMCol == nLeft +  8 .or. nMCol == nLeft + 11 .or. ;
   nMCol == nLeft + 14 .or. nMCol == nLeft + 17 .or. ;
   nMCol == nLeft + 20

   nMCol--
endif

return nMCol
#endif

* end function GetDate()
*--------------------------------------------------------------------*

* eof getdate.prg
