/*
    Program: GFALERT()
    System: GRUMPFISH LIBRARY
    Author: Ted Means and Greg Lief
    Copyright (c) 1993 Grumpfish, Inc.
    Dialect: CA-Clipper 5.x Version
    Purpose: Mouseable replacement for Clipper's ALERT() function
    Compile instructions: clipper gfalert /n /w
    If mouse support is desired, also compile with "/dMOUSE" option.
*/

#include "grump.ch"
#include "inkey.ch"

function gfAlert( cMessage, aOptions )
local aMessages
local nMessages
local nMsgWidth := 0
local nOptions
local nActive := 1
local lDone   := .F.
local aTriggers
local nPrompt
local nWidth
local nTop
local nLeft
local nRight
local nKey
local nScan
local n
local nMidrow := int( maxrow() / 2 )

// check for semi-colons in message, which would indicate multi-lines
if ( n := at(";", cMessage) ) > 0
   aMessages := {}
   do while n > 0
      aadd(aMessages, substr(cMessage, 1, n - 1))
      nMsgWidth := max(nMsgWidth, len(atail(aMessages)))
      cMessage := substr(cMessage, n + 1)
      n := at(";", cMessage)
   enddo
   nMsgWidth := max(nMsgWidth,len(cMessage))
   aadd(aMessages, cMessage)
else
   aMessages := { cMessage }
   nMsgWidth := len(cMessage)
endif
nMessages := len(aMessages)

if aOptions == NIL
   aOptions := { "Ok" }
endif
nOptions := len(aOptions)
nPrompt := 1 + len( aOptions[ 1 ] )

aTriggers := { upper( left( aOptions[ 1 ], 1 ) ) }

aeval( aOptions, {| o | aadd( aTriggers, upper( left( o, 1 ) ) ), ;
                        nPrompt := nPrompt + 4 + len( o )}, 2 )

nWidth  := max( nPrompt + 1, nMsgWidth )
nLeft   := int( ( maxcol() + 1 ) / 2 ) - 4 - ( int( nWidth ) / 2 )
nRight  := nLeft + nWidth + 8
nPrompt := int( ( maxcol() - nPrompt ) / 2 )
nTop    := nMidrow - int( (nMessages + 1) / 2) - 1

GFSaveEnv( {nTop, nLeft, nTop + nMessages + 4, nRight + 2}, 0 )

ColorSet( C_MESSAGE )

ShadowBox( nTop, nLeft, nTop + nMessages + 3, nRight, 2 )

for n := 1 to nMessages
   scrncenter(nTop + n, aMessages[n], maxcol() + 1 )
next

while .not. lDone
   setpos( nTop + nMessages + 2, nPrompt )

   for n := 1 to nOptions
      ColorSet( iif( n == nActive, C_MENU_SELECTED, C_MESSAGE ) )
      dispOut( " " + aOptions[ n ] + " " )

      if n < nOptions
         ColorSet( C_MESSAGE )
         dispOut( "  " )
      endif
   next
   
   nKey := ginkey( 0 )

   do case
      #ifdef MOUSE
      case nKey == K_RIGHT_DOWN
         nActive := 0
         lDone   := .T.

      case nKey == K_LEFT_DOWN
         if GFLastRow() == nTop + nMessages + 2
            nScan := nPrompt

            for n := 1 to nOptions
               if GFLastCol() >= nScan .and. ;
                  GFLastCol() <= nScan + len( aOptions[ n ] ) + 1
                  nActive := n
                  lDone := .T.
                  exit
               else
                  nScan += ( len( aOptions[ n ] ) + 4 )
               endif
            next
         endif
      #endif

      case nKey == K_ESC
         nActive := 0
         lDone   := .T.

      case nKey == K_ENTER
         lDone := .T.

      case nKey == K_RIGHT
         if ++nActive > nOptions
            nActive := 1
         endif

      case nKey == K_LEFT
         if --nActive < 1
            nActive := nOptions
         endif

      otherwise
         nScan := ascan( aTriggers, upper( chr( nKey ) ) )

         if nScan > 0
            lDone   := .T.
            nActive := nScan
         endif
   endcase
enddo

GFRestEnv()

return nActive

