/*
   Function: GFMCursor()
   System: GRUMPFISH LIBRARY
   Author: Joe Booth and Greg Lief
   CA-Clipper 5.x Version
   Compile instructions: clipper gfmcurs /n /w
   Changes the shape of the mouse cursor and returns the previous shape

   If you want to create your own mouse cursor shapes, first make an 8 x
   16 grid and map out your desired cursor shape upon it.  The following
   example demonstrates an hourglass cursor.

     1
     2 6 3 1
     8 4 2 6 8 4 2 1
     - - - - - - - -
     1 2 3 4 5 6 7 8
   1 X X X X X X X X        = 255
   2 X X X X X X X X        = 255
   3 X X X X X X X X        = 255
   4   X X X X X X          = 126
   5   X X X X X X          = 126
   6     X X X X            =  60
   7     X X X X            =  60
   8       X X              =  24
   9       X X              =  24
  10     X X X X            =  60
  11     X X X X            =  60
  12   X X X X X X          = 126
  13   X X X X X X          = 126
  14 X X X X X X X X        = 255
  15 X X X X X X X X        = 255
  16 X X X X X X X X        = 255

  You should then add the values of all elements in each row, using the
  numbers at the top of each column.  For example, row 7 above is
  comprised of elements in columns 32, 16, 8, and 4, whose sum is 60.

  Next, pass an array containing the sums of the 16 rows as the second
  parameter to the NewChar() function -- see below for examples.

  Finally, your NewChar() call should be immediately followed by a call
  to MouseChr() (see examples below).

*/

#define ASCII_CHAR  249
#define MOUSE
#include "grump.ch"

function GFMCursor( nCursor )
static nCurrCursor := MCURS_BLOCK
local nRetVal := nCurrCursor
if nCursor <> NIL
   do case
      case nCursor == MCURS_BLOCK
         NewChar(ASCII_CHAR,{255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255})
         MouseChr( 255 * 256, ASCII_CHAR )
      case nCursor == MCURS_ARROW
         NewChar(ASCII_CHAR,{0,0,0,128,192,224,240,248,252,240,216,12,12,6,6,0})
         MouseChr( 255 * 256, ASCII_CHAR )
      case nCursor == MCURS_IBEAM
         NewChar(ASCII_CHAR,{0,255,24,24,24,24,24,24,24,24,24,24,24,255,0})
         MouseChr( 255 * 256, ASCII_CHAR )
      case nCursor == MCURS_HOURGLASS
         NewChar(ASCII_CHAR,{255,255,255,126,126,60,60,24,24,60,60,126,126,255,255,255})
         MouseChr( 255 * 256, ASCII_CHAR )
      case nCursor == MCURS_CROSS
         NewChar(ASCII_CHAR,{0,24,24,24,24,24,255,255,24,24,24,24,24,24,0})
         MouseChr( 255 * 256, ASCII_CHAR )
   endcase
   nCurrCursor := nCursor
endif
return nRetVal
