/*
     Program: HBLINDS.PRG
     System: GRUMPFISH LIBRARY
     Author: Greg Lief
     Copyright (c) 1988-93, Greg Lief
     CA-Clipper 5.x version
     Compile instructions: clipper hblinds /n /w

     Procs & Fncts: HBLINDOPEN()
                  : HBLINDCLOS()
                  : HBLINDSAVE()

     NOTE: These three functions support 25 and 50 line display only!
*/

#include "grump.ch"
#include "fileio.ch"

/*
    Function: HBLINDOPEN()
    Purpose:  Draw title screen using horizontal blinds effect
    NOTE:     Title screen must be pre-saved using HBLINDSAVE()
    Syntax:   Hblindopen(<name of screen file>, [<delay>])
              Default delay is 15 - change it to suit your hardware
*/
function hblindopen(cFile, nDelay)
local nHandle
local aScreen[maxrow() + 1]
local nBufferLen := (maxcol() + 1) * 2
local nMaxRow := maxrow() + 1
local nMaxCol := maxcol() + 1
local xx
local yy
local cBuffer
default nDelay to 15
// sorry -- cannot currently accommodate 43-line mode (25 or 50 only)
// this is because 43 happens to be a prime number, and thus causes
// problems when we attempt to evenly divide the display intervals
if nMaxRow <> 43
   #ifdef MOUSE
   HideMouseCursor()
   #endif
   if file(cFile)
      nHandle := fopen(cFile, FO_READ)
      // verify that this file was saved in the same mode that we are in
      // because if it wasn't, all hell will break loose further down
      if fseek(nHandle, 0, FS_END) == nMaxRow * nMaxCol * 2
         fseek(nHandle, 0, FS_SET)       // reset to start of file
         for xx := 1 to nMaxRow
            cBuffer := space(nBufferLen)
            fread(nHandle, @cBuffer, 160)
            aScreen[xx] := cBuffer
         next
         fclose(nHandle)
         for xx := 1 to 5
            for yy := (nMaxRow / 5) to 1 step -1
               restscreen(yy*5 - xx, 0, yy*5 - xx, nMaxCol-1, aScreen[yy*5-xx+1])
            next
            yy := 1
            do while ++yy < nDelay
            enddo
         next
      endif
   endif
   #ifdef MOUSE
   ShowMouseCursor()
   #endif
endif
return NIL

* end function HBlindOpen()
*--------------------------------------------------------------------*

/*
    Function: HBLINDCLOS()
    Purpose:  Clear screen using horizontal blinds effect
    Syntax:   Hblindclos([<delay>])
              Default delay is 30 - change it to suit your hardware
*/
function hblindclos(nDelay)
local xx
local yy
local nMaxRow := maxrow() + 1
local nMaxCol := maxcol()
default nDelay to 30
for xx := 5 to 1 step -1
   for yy := 1 to (nMaxRow / 5)
      scroll(yy * 5 - xx, 0, yy * 5 - xx, nMaxCol, 0)
   next
   yy := 1
   do while ++yy < nDelay
   enddo
next
return NIL

* end function HBlindClos()
*--------------------------------------------------------------------*


/*
    Function: HBLINDSAVE()
    Purpose:  Save title screen to specified memory file
    Syntax:   HBlindSave(<name of screen file>)
*/
function hblindsave(cFile)
local cBuffer
local nHandle := fcreate(cFile, FC_NORMAL)
local xx
local lRetVal := .f.
local nMaxRow := maxrow()
local nMaxCol := maxcol()
if ferror() == 0
   #ifdef MOUSE
   HideMouseCursor()
   #endif
   lRetVal := .t.
   for xx := 0 to nMaxRow
      cBuffer := savescreen(xx, 0, xx, nMaxCol)
      if fwrite(nHandle, cBuffer) <> (nMaxCol + 1) * 2
         lRetVal := .f.
         exit
      endif
   next
   fclose(nHandle)
   #ifdef MOUSE
   ShowMouseCursor()
   #endif
endif
return lRetVal

* end function HBlindSave()
*--------------------------------------------------------------------*

* eof hblinds.prg
