/*
    Program: HSCRNSCRL()
    System: GRUMPFISH LIBRARY
    Author: Greg Lief
    Copyright (c) 1988-93, Greg Lief
    CA-Clipper 5.x Version
    Compile instructions: clipper vscroll /n /w

    Displays screen in horizontal column-by-column increments

    NOTE: screen must be pre-saved to the <screen file> using
    Save_Drape())

    Syntax: HSCRNSCRL(<screen file> [, <nDirection> ])

    Optional parameter <nDirection> determines whether display is left
    to right or vice versa.  1 = right to left, -1 = left to right.
    (Default is left-to-right.)  Thanks to Evan Davies for the
    inspiration!

    WILL HANDLE 25/43/50 ROW DISPLAYS
*/

#include "fileio.ch"
#include "grump.ch"

function HScrnScrl(cfile, ndirection)
local nMaxRow := maxrow() + 1
local nMaxCol := maxcol() + 1
local nHandle, aScreen[nMaxCol], buffer, xx, yy
local bufflen := nMaxRow * 2
local buffer2
local aLoopers
default ndirection to -1                // default is pull-down
if file(cfile)
   nHandle := fopen(cfile, FO_READ)
   if nHandle <> F_ERROR
      // verify that this file was saved in the same mode that we are in
      // because if it wasn't, all hell will break loose further down
      if ( xx := fseek(nHandle, 0, FS_END) ) == nMaxRow * nMaxCol * 2 .or. ;
                                         xx == (nMaxRow * nMaxCol * 2) + 4
         fseek(nHandle, (xx - nMaxRow * nMaxCol * 2), FS_SET) // reset to start
         for xx := 1 to nMaxCol
            buffer := space(bufflen)
            fread(nHandle, @buffer, bufflen)
            aScreen[xx] := buffer
         next
         fclose(nHandle)
         /*
             load aLoopers array based on direction -- structure is:
                  aLoopers[1] := starting row for FOR..NEXT loop
                  aLoopers[2] := ending row for FOR..NEXT loop
                  aLoopers[3] := leftmost column of region to be shifted
                  aLoopers[4] := rightmost column of region to be shifted
                  aLoopers[5] := column to be newly displayed
         */
         if ndirection == 1
            aLoopers := { 1, nMaxCol, 1, nMaxCol - 1, nMaxCol }
         else
            aLoopers := { nMaxCol, 1, 0, nMaxCol - 2, 0 }
         endif
         #ifdef MOUSE
         HideMouseCursor()
         #endif
         for xx := aLoopers[1] to aLoopers[2] step ndirection
            buffer2 := savescreen(0, aLoopers[3], nMaxRow - 1, aLoopers[4])
            restscreen(0, aLoopers[5],  nMaxRow - 1,  aLoopers[5], aScreen[xx])
            restscreen(0, aLoopers[3] - ndirection,  nMaxRow - 1, ;
                          aLoopers[4] - ndirection, buffer2)
         next
         #ifdef MOUSE
         ShowMouseCursor()
         #endif
      endif
   endif
endif
return NIL

* end function HScrnScrl()
*--------------------------------------------------------------------*

* eof hscroll.prg
