/*
   Function: IndexBar2()
   System: GRUMPFISH LIBRARY
   Author: Greg Lief
   Copyright (c) 1993 Greg Lief
   Comix-specific additions by David Yager
   Clipper 5.x version
   Compile instructions: clipper indexba2 /n /w
   Purpose:  Displays percentage complete when indexing multiple files
   Syntax:   IndexBar2(<aFiles> [, <nRow> ] )
             <aFiles>= contains information about the indexes to be created
                       Array structure:
                       { ;
                         { cAlias, cIndexFile, cIndexKey, lUnique }, ;
                         ... ;
                       }
             <nRow> = row at which to place window (default: centered)
   Returns:  Logical: .T. if all indexes are successfully created
   Special thanks to the Old Town Margarita Man for inspiration!

   NOTES: If you wish to make use of specialized CA-Clipper 5.2 indexing
          (e.g., order bags, conditional, etcetera), you should
          recompile this file with the /dCLIPPER52 command-line
          parameter.

          If you are using Comix, you should recompile this file with
          the /dCOMIX command-line parameter.

*/

#include "error.ch"
#include "grump.ch"

static nTotalRecs
static nGraphLen

// structure of nested arrays in aFiles
#define _ALIAS     1
#define _INDEXFILE 2
#define _INDEXKEY  3
#define _UNIQUE    4
// the next nine items are Clipper52 and Comix-specific
#define _CFOR      5
#define _BFOR      6
#define _ALL       7
#define _WHILE     8
#define _NEXT      9
#define _REC      10
#define _REST     11
#define _DESCEND  12
#define _TAGNAME  13
#ifdef COMIX
   #define _CMXFILE    14
   #define _ADDITIVE   15
   #define _USECURRENT 16
#endif
#define BASIC_OPTIONS   4
#define STD52_OPTIONS  13



function indexbar2(aFiles, nRow)
local cMessageColor := ColorSet(C_MESSAGE, .t.)
local bKey
local bOldHandler := errorblock( { | e | IndexError(e, bOldHandler) } )
local nFiles := len(aFiles)
local lRetVal := .f.
local x
local nOldRow
local nOldCol
local aAliases := {}
local nLeft := ( maxcol() / 2 ) - 30
local nRecsProcessed := 0
default nRow to ( maxrow() / 2 ) - 2
nTotalRecs := 0
nGraphLen  := 0
gfsaveenv(.t., 0)
for x := 1 to nFiles
   nTotalRecs += (aFiles[x][_ALIAS])->( reccount() )
next
dispbegin()
ColorSet(cMessageColor)
shadowbox(nRow, nLeft, nRow + 5, nLeft + 63, 2, "Index File Status")
@ nRow + 1, nLeft + 3  ssay 'Now processing '
@ nRow + 1, nLeft + 37 ssay 'Files processed:'
@ nRow + 2, nLeft + 2  ssay ''
@ nRow + 2, nLeft + 61 ssay ''
@ nRow + 3, nLeft + 2  ssay '' + replicate("", 9) + ';'
@ nRow + 4, nLeft + 2  ssay '0'
for x := 1 to 10
   @ nRow + 4, nLeft + x * 6 ssay str(10 * x, 3)
next
ColorSet(C_APICK_STATUSBAR)
@ nRow + 2, nLeft + 3 ssay replicate(chr(177), 58)
setpos(nRow + 2, nLeft + 3)
dispend()
setcolor(cMessageColor)

begin sequence

for x := 1 to nFiles
   nOldRow := row()
   nOldCol := col()
   @ nRow + 1, nLeft + 54 ssay ltrim(str(x, 3)) + '/' + ltrim(str(nFiles))
   @ nRow + 1, nLeft + 18 ssay padr( ;
                               upper(if(aFiles[x][_INDEXFILE] == NIL,;
                                        aFiles[x][_TAGNAME], ;
                                        aFiles[x][_INDEXFILE] + indexext())), ;
                                     12)
   setpos(nOldRow, nOldCol)
   bKey := makeblock(aFiles[x][_INDEXKEY])  // see GRUMP.CH for MakeBlock()
   dbClearIndex()

   if (aFiles[x][_ALIAS])->( lastrec() ) > 0

      (aFiles[x][_ALIAS])->( dbGoTop() )
      ColorSet(C_APICK_INDICATOR)

      #ifdef COMIX

      if len(aFiles[x]) > STD52_OPTIONS
         (aFiles[x][_ALIAS])->( ordCondSet(aFiles[x][_CFOR],  ;
                                           aFiles[x][_BFOR],  ;
                                           aFiles[x][_ALL],   ;
                                           aFiles[x][_WHILE], ;
                     { || gfshowbar(NIL, ++nRecsProcessed) }, ;
                                           , recno(),         ;
                                           aFiles[x][_NEXT],  ;
                                           aFiles[x][_REC],   ;
                                           aFiles[x][_REST],  ;
                                           aFiles[x][_DESCEND],;
                                           aFiles[x][_CMXFILE], ;
                                           aFiles[x][_ADDITIVE], ;
                                           aFiles[x][_USECURRENT]) )

         #ifdef CLIPPER52  // with Comix

         (aFiles[x][_ALIAS])->( ordCreate(aFiles[x][_INDEXFILE],      ;
                                          aFiles[x][_TAGNAME],        ;
                                          aFiles[x][_INDEXKEY], bkey, ;
                                          aFiles[x][_UNIQUE] ) )
         (aFiles[x][_ALIAS])->( ordListClear() )

         #else    //  using 5.01 with Comix

         (aFiles[x][_ALIAS])->( dbCreateIndex(if(aFiles[x][_CMXFILE] == 0,;
                                                 aFiles[x][_INDEXFILE],   ;
                                                 aFiles[x][_TAGNAME] ),   ;
                                                 aFiles[x][_INDEXKEY], bkey, ;
                                                 aFiles[x][_UNIQUE] ) )
         (aFiles[x][_ALIAS])->( dbClearIndex() )

         #endif // CLIPPER52

      #else

      if .f.

      #endif // COMIX

      #ifdef CLIPPER52

      elseif len(aFiles[x]) > BASIC_OPTIONS
         (aFiles[x][_ALIAS])->( ordCondSet(aFiles[x][_CFOR],  ;
                                           aFiles[x][_BFOR],  ;
                                           aFiles[x][_ALL],   ;
                                           aFiles[x][_WHILE], ;
                     { || gfshowbar(NIL, ++nRecsProcessed) }, ;
                                           , recno(),         ;
                                           aFiles[x][_NEXT],  ;
                                           aFiles[x][_REC],   ;
                                           aFiles[x][_REST],  ;
                                           aFiles[x][_DESCEND]) )
         (aFiles[x][_ALIAS])->( ordCreate(aFiles[x][_INDEXFILE],      ;
                                          aFiles[x][_TAGNAME],        ;
                                          aFiles[x][_INDEXKEY], bkey, ;
                                          aFiles[x][_UNIQUE] ) )
         (aFiles[x][_ALIAS])->( ordListClear() )

      #endif // CLIPPER52

      else

         (aFiles[x][_ALIAS])->( dbCreateIndex(aFiles[x][_INDEXFILE], ;
                             "(" + aFiles[x][_INDEXKEY] + ")",       ;
                             {|| gfshowbar(bKey, ++nRecsProcessed)}, ;
                             aFiles[x][_UNIQUE]) )
         (aFiles[x][_ALIAS])->( dbClearIndex() )

      endif

   else
      #ifdef COMIX

      if len(aFiles[x]) > STD52_OPTIONS
         (aFiles[x][_ALIAS])->( ordCondSet(aFiles[x][_CFOR],  ;
                                           aFiles[x][_BFOR],  ;
                                           aFiles[x][_ALL],   ;
                                           aFiles[x][_WHILE], ;
                                                         NIL, ;
                                           , recno(),         ;
                                           aFiles[x][_NEXT],  ;
                                           aFiles[x][_REC],   ;
                                           aFiles[x][_REST],  ;
                                           aFiles[x][_DESCEND],;
                                           aFiles[x][_CMXFILE], ;
                                           aFiles[x][_ADDITIVE], ;
                                           aFiles[x][_USECURRENT]) )

         #ifdef CLIPPER52  // with Comix

         (aFiles[x][_ALIAS])->( ordCreate(aFiles[x][_INDEXFILE],      ;
                                          aFiles[x][_TAGNAME],        ;
                                          aFiles[x][_INDEXKEY], bkey, ;
                                          aFiles[x][_UNIQUE] ) )
         (aFiles[x][_ALIAS])->( ordListClear() )

         #else // using 5.01 with Comix

         (aFiles[x][_ALIAS])->( dbCreateIndex(if(aFiles[x][_CMXFILE] == 0,;
                                                 aFiles[x][_INDEXFILE],   ;
                                                 aFiles[x][_TAGNAME] ),   ;
                                                 aFiles[x][_INDEXKEY], bkey, ;
                                                 aFiles[x][_UNIQUE] ) )
         (aFiles[x][_ALIAS])->( dbClearIndex() )

         #endif // CLIPPER52

      #else

      if .f.

      #endif

      #ifdef CLIPPER52

      elseif len(aFiles[x]) > BASIC_OPTIONS
         (aFiles[x][_ALIAS])->( ordCondSet(aFiles[x][_CFOR],  ;
                                           aFiles[x][_BFOR],  ;
                                           aFiles[x][_ALL],   ;
                                           aFiles[x][_WHILE], ;
                                                         NIL, ;
                                           , recno(),         ;
                                           aFiles[x][_NEXT],  ;
                                           aFiles[x][_REC],   ;
                                           aFiles[x][_REST],  ;
                                           aFiles[x][_DESCEND]) )
         (aFiles[x][_ALIAS])->( ordCreate(aFiles[x][_INDEXFILE],      ;
                                          aFiles[x][_TAGNAME],        ;
                                          aFiles[x][_INDEXKEY], bkey, ;
                                          aFiles[x][_UNIQUE] ) )
         (aFiles[x][_ALIAS])->( ordListClear() )

      #endif

      else

         (aFiles[x][_ALIAS])->( dbCreateIndex(aFiles[x][_INDEXFILE], ;
                                aFiles[x][_INDEXKEY], bKey, aFiles[x][_UNIQUE]) )
         (aFiles[x][_ALIAS])->( dbClearIndex() )

      endif

   endif

   setcolor(cMessageColor)
next
for x := 1 to nFiles
   #ifdef CLIPPER52
      if len(aFiles[x]) > BASIC_OPTIONS .and. aFiles[x][_TAGNAME] <> NIL
         (aFiles[x][_ALIAS])->( ordListAdd(aFiles[x][_INDEXFILE]) )
      else
         (aFiles[x][_ALIAS])->( dbSetIndex(aFiles[x][_INDEXFILE]) )
      endif
   #else
      (aFiles[x][_ALIAS])->( dbSetIndex(aFiles[x][_INDEXFILE]) )
   #endif
next
lRetVal := .t.

end sequence

GFRestEnv()
errorblock(bOldHandler)
return lRetVal

* end function IndexBar()
*--------------------------------------------------------------------*


/*
    Function: GFShowBar()
    Syntax:   GFShowBar(<index key>)
              <index key> = self-explanatory
    Purpose:  Display status graph whilst indexing
    Example:  See above -- NOT MEANT FOR USE AS A STANDALONE
    Returns:  the index key for each record
*/
static function gfshowbar(bKey, nRecs)
if nRecs <= nTotalRecs
   // draw more of the graph if the relative position has changed
   do while nGraphLen < int(nRecs / nTotalRecs * 58)
      dispout(chr(219))
      nGraphLen++
   enddo
endif

#ifdef COMIX
   return .t.
#else
   #ifdef CLIPPER52
      return if(bKey == NIL, .t., eval(bKey))
   #else
      return eval(bKey)
   #endif
#endif

* end static function GFShowBar()
*--------------------------------------------------------------------*


/*
    Function: IndexError()
    Purpose:  Trap various errors in the indexing process
*/
static function IndexError(oRats, bOldHandler)
if oRats:genCode == EG_NOALIAS .or. oRats:genCode == EG_NOTABLE
   err_msg({ "Missing alias... process aborted (INDEXBAR2)" } )
   break
elseif oRats:genCode == EG_SYNTAX .or. oRats:genCode == EG_NOVAR .or. ;
       oRats:genCode == EG_NOFUNC
   err_msg({ "Invalid index key... process aborted (INDEXBAR2)" } )
   break
endif
return eval(bOldHandler, oRats)

* end static function IndexError()
*--------------------------------------------------------------------*

* eof indexbar.prg
