/*
   Function: LITE_MENU2()
   System:   Grumpfish Library
   Author:   Greg Lief
   Copyright (c) 1988-93 Greg Lief
   Dialect:  CA-Clipper 5.x
   Compile with: clipper litemen2 /n/w/a
   Purpose:  Replacement for MENU TO... and ACHOICE()

   Warning:  You must include the following line in your program if
             you want to use this function:

             #include "grumpm.ch"

   Caveat:   Pretty minor, actually.  In the same fashion that Clipper
             carries around a PUBLIC variable GETLIST for GETs, this
             function needs to use the variable MENULIST.  Therefore,
             to avoid compiler warnings, you should include this line
             of code in your program somewhere:

             memvar menulist
*/

#include "grump.ch"
#include "inkey.ch"

#define  DUMMY           254

// structure of menu prompt array
#define  ROW             1
#define  COL             2
#define  PROMPT          3
#define  MESSAGE         4
#define  MESSAGECOLOR    5
#define  WHEN            6
#define  ACTION          7
#define  TRIGGERCOLOR    8
#define  VARIABLE        9
#define  TRIGGERPOSITION 10

#define  FORWARD         1
#define  BACKWARD       -1

#ifdef MOUSE
   // structure of hot zone array
   #define HZ_TOP         1
   #define HZ_LEFT        2
   #define HZ_BOTTOM      3
   #define HZ_RIGHT       4
   #define HZ_ACTION      5
#endif

static unavacolor := '+n/n'   // color for unavailable menu options


function lite_menu2(aMenu, cPlainColor, nSel, cVarName, bEvent, ;
                    nTimeOut, bExit, trigcolor, cLeft, cRight,  ;
                    lCenter, lClearmsgs, nMessRow, nMessCol, aHotZones)
local aTemp := aclone(aMenu)
local nElements := len(aTemp)
local xx
local nKey := 0
local cTrigLetters := ''
local lFallout := .f.
local nPtr
local bAction
local cHiliteColor
local direction := FORWARD
local nStart
local lMessages := .f.

#ifdef MOUSE
   local nMouseRow
   local nMouseCol
   local nHotZones := if( valtype(aHotZones) == "A", len(aHotZones), 0)
#endif

gfsaveenv(, 0)

// if no time-out was specified, use a ridiculous default
default nTimeOut to 100000000

// set default colors for unselected, selected, and unavailable options
default cPlainColor to setcolor()
xx := at(',', cPlainColor)
nPtr := rat(',', cPlainColor)
if xx == nPtr     // this means only unselected & selected colors were passed
   cHiliteColor := substr(cPlainColor, xx + 1)
else
   cHiliteColor := substr(cPlainColor, xx + 1, nPtr - xx - 1)
   /*
      if a color string was passed that contains two commas, then use
      whatever lies to the right of the second comma as the color for
      unavailable menu options -- but if we are using the current
      color setting, instead use the default (+N/N) as defined above
   */
   if cPlainColor <> setcolor()
      unavacolor := substr(cPlainColor, nPtr + 1)
   endif
endif

// establish default color for highlighting trigger letters
default trigcolor to "+" + cPlainColor

/*
   determine initial highlighted item default to 1 -- also perform
   error-checking to ensure they didn't specify an invalid selection
*/
if nSel == NIL .or. (nSel < 1 .or. nSel > nElements)
   nSel := 1
endif

// display the options and build the string of trigger letters
dispbegin()
for xx := 1 to nElements

   // determine whether we are using messages or not
   if ! lMessages .and. aTemp[xx, MESSAGE] <> NIL
      lMessages := .t.
      // set defaults for various message-related items
      default lClearmsgs to .t.

      // if user didn't specify whether or not to center the message,
      // use the default SET MESSAGE CENTER setting (however, message
      // column is specified, then assume they don't want centering)
      if nMessCol <> NIL
         lCenter := .f.
      else
         nMessCol := 0
         default lCenter to set(_SET_MCENTER)
      endif

      // if user didn't specify message row, use the system default
      // (or use the bottom row if the default is still zero)
      if nMessRow == NIL .and. ( nMessRow := set(_SET_MESSAGE) ) == 0
         nMessRow := maxrow()
      endif
   endif

   // if there are any code blocks in the place of menu prompts,
   // evaluate them now to create the character string prompts
   if valtype(aMenu[xx, PROMPT]) == "B"
      aTemp[xx, PROMPT] := eval(aMenu[xx, PROMPT])
   endif

   // next make sure that any WHEN clause for this option is
   // satisfied, and that the option can therefore be selected
   if aTemp[xx, WHEN] == NIL .or. eval( aTemp[xx, WHEN] )
      /*
         the default is to add the first non-space character.
         However, if there is a tilde embedded in this menu
         option, use the letter directly following it.
      */
      if (nPtr := at("~", aTemp[xx, PROMPT]) ) > 0
         cTrigLetters += upper(substr(aTemp[xx, PROMPT], nPtr + 1, 1))
      else
         cTrigLetters += upper(left(ltrim(aTemp[xx, PROMPT]), 1))
         // determine how much white space precedes the actual prompt
         nPtr := len(aTemp[xx, PROMPT]) - len(ltrim(aTemp[xx, PROMPT])) + 1
      endif
      /*
         Add actual trigger letter position to nested array for this
         menu option.  I discovered the hard way that this is necessary
         in the event that the menu prompt begins with a blank space.
      */
      // but first, a little kludge... users with old GRUMPM.CH
      // files might not have the ninth element in the array
      if len(aTemp[xx]) < VARIABLE
         aadd(aTemp[xx], NIL )
      endif
      aadd(aTemp[xx], nPtr)
   else
      // add dummy placeholder for unavailable options
      cTrigLetters += chr(DUMMY)
   endif
   // strip out any tildes now
   aTemp[xx, PROMPT] := strtran(aTemp[xx, PROMPT], "~", "")
   ShowOption(aTemp[xx], cPlainColor, trigcolor)
next
dispend()

/*
   if the trigger letter string is full of dummy placeholders, that means
   there are no selectable menu options at this time, so fall out!
*/
if cTrigLetters == replicate(chr(DUMMY), nElements)
   nSel := 0
else
   // commence main key-grabbing loop
   #ifdef MOUSE
   while nKey <> K_ENTER .and. nKey <> K_ESC .and. nKey <> K_RIGHT_DOWN
   #else
   while nKey <> K_ENTER .and. nKey <> K_ESC
   #endif
      // make sure that current option is available for selection
      if aTemp[nSel, WHEN] == NIL .or. eval(aTemp[nSel, WHEN])

         // then display current option in highlight color
         @ aTemp[nSel, ROW], aTemp[nSel, COL] ssay ;
                 aTemp[nSel, PROMPT] color cHiliteColor

         // if we are using messages, either display the message (if
         // there is one) or clear the message row
         if lMessages
            if aTemp[nSel, MESSAGE] == NIL .and. lClearmsgs
               scroll(nMessRow, nMessCol, nMessRow, maxcol(), 0)
            elseif lCenter
               if lClearmsgs
                  @ nMessRow, 0 ssay ;
                            padc(aTemp[nSel, MESSAGE], maxcol()+1)     ;
                            color if(aTemp[nSel, MESSAGECOLOR] == NIL, ;
                                  cPlainColor, aTemp[nSel, MESSAGECOLOR])
               elseif aTemp[nSel, MESSAGE] <> NIL
                  scrncenter(nMessRow, aTemp[nSel, MESSAGE],         ;
                             if(aTemp[nSel, MESSAGECOLOR] == NIL,    ;
                             cPlainColor, aTemp[nSel, MESSAGECOLOR])  )
               endif
            elseif lClearmsgs
               @ nMessRow, nMessCol ssay ;
                      padr(aTemp[nSel, MESSAGE], maxcol()+1)        ;
                      color if(aTemp[nSel, MESSAGECOLOR] == NIL,    ;
                            cPlainColor, aTemp[nSel, MESSAGECOLOR])
            else
               @ nMessRow, nMessCol ssay aTemp[nSel, MESSAGE] color ;
                               if(aTemp[nSel, MESSAGECOLOR] == NIL, ;
                               cPlainColor, aTemp[nSel, MESSAGECOLOR])
            endif
         endif
         if lFallout
            exit
         else
            /*
               begin keypress wait loop -- necessary to accommodate
               recurring event and keyboard inactivity timeout
            */
            #ifdef MOUSE
            ShowMouseCursor()
            #endif
            nStart := seconds()
            do while (nKey := inkey()) == 0 .and. seconds() - nStart < nTimeOut
               if bEvent <> NIL
                  eval(bEvent)
               endif
            enddo
            #ifdef MOUSE
            HideMouseCursor()
            #endif

            // we timed out!
            if nKey == 0
               // if no exitevent was specified, use screen blanker
               if bExit == NIL
                  blankscr3(-1)
               else
                  eval(bExit)
               endif
            endif

            #ifdef MOUSE
            nMouseRow := GFLastRow()
            nMouseCol := GFLastCol()
            if nKey == K_LEFT_DBLCLICK
               if nMouseRow == aTemp[ nSel, ROW ] .and. ;
                  nMouseCol >= aTemp[ nSel, COL ] .and. ;
                  nMouseCol <= aTemp[ nSel, COL ] +     ;
                          len( aTemp[ nSel, PROMPT ] ) - 1
                  nKey := K_ENTER
               else
                  nKey := K_LEFT_DOWN
               endif
            endif
            #endif

            do case
               #ifdef MOUSE
               case nKey == K_LEFT_DOWN
                  ShowOption(aTemp[nSel], cPlainColor, trigcolor)
                  for xx := 1 to nElements
                     if aTemp[xx][ROW] == nMouseRow              .and. ;
                               nMouseCol >= aTemp[xx][COL]       .and. ;
                               nMouseCol <= aTemp[xx][COL] +           ;
                                            len(aTemp[xx][PROMPT]) - 1
                        nSel := xx
                        exit
                     endif
                  next
                  // if mouse click was not on a menu prompt,
                  // scan hot zone array if one was passed
                  if xx > nElements .and. nHotZones > 0
                     for xx := 1 to nHotZones
                        if nMouseRow >= aHotZones[xx][HZ_TOP]      .and. ;
                             nMouseRow <= aHotZones[xx][HZ_BOTTOM] .and. ;
                             nMouseCol >= aHotZones[xx][HZ_LEFT]   .and. ;
                             nMouseCol <= aHotZones[xx][HZ_RIGHT]
                           eval(aHotZones[xx][HZ_ACTION])
                           exit
                        endif
                     next
                  endif
               #endif

               /*
                  First, check for left and right arrow keypresses.
                  If the LEFT and RIGHT clauses were used in conjunction
                  with the MENU TO command, we must stuff characters into
                  the buffer now.  This would generally be used with a
                  pull-down menu system.
               */
               case nKey == K_LEFT .and. cLeft <> NIL
                  keyboard cLeft

               case nKey == K_RIGHT .and. cRight <> NIL
                  keyboard cRight

               /*
                  next, check for action block attached to last keypress
                  if there is one, evaluate it and pass it the name of
                  the MENU TO variable along with the current highlighted
                  option (e.g., SEL[5] if you are on the fifth option)
               */
               case ( bAction := setkey(nKey) ) <> NIL
                  eval(bAction, procname(1), procline(1), cVarName + ;
                       "[" + ltrim(str(nSel)) + "]")

               // go down one line, observing wrap-around conventions
               case nKey == K_DOWN .or. nKey == K_RIGHT
                  direction := FORWARD
                  ShowOption(aTemp[nSel], cPlainColor, trigcolor)
                  if nSel == nElements
                     nSel := 1
                  else
                     nSel += direction
                  endif

               // go up one line, observing wrap-around conventions
               case nKey == K_UP .or. nKey == K_LEFT
                  direction := BACKWARD
                  ShowOption(aTemp[nSel], cPlainColor, trigcolor)
                  if nSel == 1
                     nSel := nElements
                  else
                     nSel--
                  endif

               // jump to top option
               case nKey == K_HOME .or. nKey == K_PGUP
                  // no point in changing color if we're already there
                  if nSel <> 1
                     ShowOption(aTemp[nSel], cPlainColor, trigcolor)
                     nSel := 1
                     direction := FORWARD
                  endif

               // jump to bottom option
               case nKey == K_END .or. nKey == K_PGDN
                  // no point in changing color if we're already there
                  if nSel <> nElements
                     ShowOption(aTemp[nSel], cPlainColor, trigcolor)
                     nSel := nElements
                     direction := BACKWARD
                  endif

               // first letter - jump to appropriate option
               case upper(chr(nKey)) $ cTrigLetters
                  ShowOption(aTemp[nSel], cPlainColor, trigcolor)
                  nSel := at(upper(chr(nKey)), cTrigLetters)
                  lFallout := .t.

            endcase
         endif
      else
         // keep moving in current direction
         nSel += DIRECTION
         // wrap-around
         if nSel == 0
            nSel := nElements
         elseif nSel > nElements
            nSel := 1
         endif
      endif
   enddo
   // if there is an action block attached to this selection, run it
   #ifdef MOUSE
   if nKey <> K_ESC .and. nKey <> K_RIGHT_DOWN
   #else
   if nKey <> K_ESC
   #endif
      if aTemp[nSel][ACTION] <> NIL
         // reset cursor because next module might expect it to be on
         gfrestenv()
         /*
           if there is a variable associated with this menu option,
           we must save the value returned from the ACTION block
           NOTE: we must ensure that array actually contains the
           VARIABLE element, which was a late addition
         */
         if len(aTemp[nSel]) >= VARIABLE .and. ;
                    aTemp[nSel][VARIABLE] <> NIL
            eval(aTemp[nSel][VARIABLE], eval(aTemp[nSel][ACTION]))
         else
            eval(aTemp[nSel][ACTION])
         endif
         // now that that's over with, turn the cursor back off
         gfsaveenv(, 0)
      endif
   else
      nSel := 0  // since they Esc'd out, return a zero
   endif
endif
gfrestenv()
return nSel

* end function Lite_Menu2()
*--------------------------------------------------------------------*


/*
  Function: ShowOption()
  Purpose:  Display current prompt in mixed colors
*/
static function ShowOption(aItem, cPlainColor, cTrigColor)
local nPtr
// first make sure that this option is available for selection
if aItem[WHEN] == NIL .or. eval( aItem[WHEN] )
   if aItem[TRIGGERPOSITION] <> 1
      @ aItem[ROW], aItem[COL] ssay aItem[PROMPT] color cPlainColor
      @ aItem[ROW], aItem[COL] + aItem[TRIGGERPOSITION] - 1 ssay ;
          substr(aItem[PROMPT], aItem[TRIGGERPOSITION], 1) ;
          color if( empty(aItem[TRIGGERCOLOR]), cTrigColor, aItem[TRIGGERCOLOR] )
   else
      @ aItem[ROW], aItem[COL] ssay left(aItem[PROMPT], 1) ;
          color if( empty(aItem[TRIGGERCOLOR]), cTrigColor, aItem[TRIGGERCOLOR] )
      dispout(substr(aItem[PROMPT], 2), cPlainColor)
   endif
else
   // display unavailable menu options in gray on black
   @ aItem[ROW], aItem[COL] ssay aItem[PROMPT] color unavacolor
endif
return nil

* end static function ShowOption()
*--------------------------------------------------------------------*

* eof litemen2.prg
