/*
    Program: MEMEDIT()
    System: GRUMPFISH LIBRARY
    Author: Greg Lief
    Copyright (c) 1988-93, Greg Lief
    CA-Clipper 5.x Version
    Compile instructions: clipper memedit /n /w

    Procs & Fncts: MEMEDIT()
                   ESCFUNC()

    Calls: SHADOWBOX()   (function in SHADOWBO.PRG)

    Generic memo editing function
*/

#include "grump.ch"
#include "inkey.ch"
#include "memoedit.ch"
#include "setcurs.ch"

// the following three variables are file-wide static because they must
// be visible in EscFunc() below
static nMsgrow     // row at which to display "MEMO NOT UPDATED" msg
static nMsgcol     // column at which to display "MEMO NOT UPDATED" msg


function memedit(cField, nTop, nLeft, nBottom, nRight, cTitle, lEditit)
local lRetVal := .t.
local cMemo
local lOldExact := set(_SET_EXACT, .T.)
local vartype
local bOldIns  := setkey( K_INS, {|| setcursor( ;
                  if(readinsert(! readInsert()), SC_NORMAL, SC_SPECIAL1))} )
local lOldIns  := set(_SET_INSERT)
default nTop to 5
default nLeft to 10
default nBottom to maxrow() - 5
default nRight to maxcol() - 10
default cTitle to []
default lEditit to .t.

// initialize the file-wide statics (see above)
nMsgrow := nTop
nMsgcol := nRight

gfsaveenv( .t. )
ColorSet(C_MEMEDIT_BOX)
Shadowbox(nTop, nLeft, nBottom, nRight, 1, cTitle, , , ;
          if(lEditit, 'Ctrl-W to save,', ' to move, or') + ' Esc to exit')
ColorSet(C_MEMEDIT_WINDOW)
scroll(nTop + 1, nLeft + 1, nBottom - 1, nRight - 1, 0)
// determine if this memo is a field or variable
do case
   case len(cField) <= 10 .and. ! empty( vartype := fieldpos(cField) )  // field
      cMemo := fieldget(vartype)
   case ( vartype := memvarblock(cField) ) <> NIL  // PRIVATE/PUBLIC
      cMemo := eval(vartype)
      vartype := .t.
   otherwise                                       // LOCAL/STATIC
      cMemo := cField
      vartype := .f.
endcase
if lEditit
   // set initial cursor size based on current mode
   setcursor( if(readInsert(), SC_SPECIAL1, SC_NORMAL) )
endif
cMemo := memoedit(cMemo, nTop + 1, nLeft + 1, nBottom - 1, nRight - 1, ;
                  lEditit, if(lEditit, 'escfunc', ''), , 3)
// the following block is relevant only when we are editing
if lEditit

   #ifdef MOUSE
   if lastkey() <> K_ESC .and. lastkey() <> K_RIGHT_DOWN
   #else
   if lastkey() <> K_ESC
   #endif

      do case

         // we edited a field
         case valtype(vartype) == "N"
            if rlock()
               fieldput(fieldpos(cField) , cMemo)
               unlock
            else
               err_msg( { NETERR_MSG } )
               lRetVal := .f.
            endif

         // we edited a private or public variable
         case vartype
            eval( memvarblock(cField) , cMemo)

         // we edited a local or static variable
         otherwise
            cField := cMemo
      endcase
   else
      lRetVal := .f.
   endif
endif
gfrestenv()
set(_SET_EXACT, lOldExact)
setkey(K_INS, bOldIns)
set(_SET_INSERT, lOldIns)
return lRetVal

* end function MemEdit()
*--------------------------------------------------------------------*


function EscFunc( nStat, line, col)
local ret_val := ME_DEFAULT
#ifdef MOUSE
if ( lastkey() == K_ESC .or. lastkey() == K_RIGHT_DOWN ) .and. ;
                  (nStat == 1 .or. nStat == 2)
#else
if lastkey() == K_ESC .and. (nStat == 1 .or. nStat == 2)
#endif
   gfsaveenv(, 0)
   @ nMsgrow, nMsgcol - 18 ssay 'MEMO NOT UPDATED'
   tone(MUSIC_ALERT, 1)
   tone(MUSIC_ALERT, 1)
   inkey(2)
   gfrestenv()
   ret_val := K_ESC
endif
return ret_val

* end function EscFunc()
*--------------------------------------------------------------------*

* eof memedit.prg
