/*
   Functions: CHECKMOVE(), MOUSEMOVE()
   System: GRUMPFISH LIBRARY
   Author: Greg Lief and Ted Means
   CA-Clipper 5.x Version
   Compile instructions: clipper moving /n /w

   Called by: POPCALC()
              POPSTOP()

   Handles screen movement of desktop utility windows
   Returns Logical Value: .T. if window was moved, .F. if not

   NOT FOR USE AS STAND-ALONE FUNCTIONS!!!
*/

#include "inkey.ch"
#ifdef MOUSE
   #include "grump.ch"
#endif

function checkmove(nkey, topoffset, leftoffset, ntop, nleft, oldscrn)
local buffer, ret_val := .f.
if (nkey == K_UP    .and. ntop > 0) .or. ;
   (nkey == K_DOWN  .and. ntop < maxrow() - topoffset) .or. ;
   (nkey == K_LEFT  .and. nleft > 0) .or. ;
   (nkey == K_RIGHT .and. nleft < maxcol() - leftoffset) .or. ;
   nkey == K_HOME .or. nkey == K_END .or. ;
   nkey == K_PGUP .or. nkey == K_PGDN .or. ;
   nkey == K_CTRL_LEFT .or. nkey == K_CTRL_RIGHT

   dispbegin()
   // save screen image at current location
   buffer := savescreen(ntop, nleft, ntop+topoffset - 1, nleft+leftoffset - 2)
   restscreen(ntop, nleft, ntop + topoffset, nleft + leftoffset, oldscrn)

   // determine new screen coordinates for calculator box
   // first check for 'big moves' (home, end, pgup, pgdn, ^left, ^right)
   if nkey == K_HOME .or. nkey == K_END .or. nkey == K_PGUP .or. ;
            nkey == K_PGDN .or. nkey == K_CTRL_LEFT .or. nkey == K_CTRL_RIGHT
      ntop := if(nkey == K_HOME .or. nkey == K_PGUP, 0, ;
              if(nkey == K_PGDN .or. nkey == K_END, maxrow() - topoffset, ntop))
      nleft := if(nkey == K_HOME .or. nkey == K_CTRL_LEFT, 0, ;
               if(nkey == K_END .or. nkey == K_CTRL_RIGHT, ;
               maxcol() - leftoffset, nleft))
   else
      ntop -= if(nkey == K_DOWN, -1, if(nkey == K_UP, 1, 0))
      nleft -= if(nkey == K_RIGHT, -1, if(nkey == K_LEFT, 1, 0))
   endif
   oldscrn := savescreen(ntop, nleft, ntop + topoffset, nleft + leftoffset)
   gfattr(ntop+1, nleft+leftoffset-1, ntop+topoffset-1, nleft+leftoffset, 8)
   gfattr(ntop+topoffset-1, nleft+2, ntop+topoffset, nleft+leftoffset, 8)
   restscreen(ntop, nleft, ntop+topoffset-1, nleft+leftoffset-2, buffer)
   ret_val := .t.
   dispend()
endif
return ret_val
* end function CheckMove()
*--------------------------------------------------------------------*



#ifdef MOUSE

procedure mousemove( topoffset, leftoffset, ntop, nleft, oldscrn )
local nKey    := 0
local nRow    := GFLastRow()
local nCol    := GFLastCol()
local nMaxRow := maxrow() - ( ntop + topoffset - nRow )
local nMinRow := nRow - ntop
local nMaxCol := maxcol() - ( nleft + leftoffset - nCol )
local nMinCol := nCol - nleft
local aPos
local buffer

enableDrag( .T. )       // Enable mouse dragging

while ( nKey := inkey() ) != K_LEFT_UP
   aPos := GetMousePos()

   if aPos[ 1 ] == nRow .and. aPos[ 2 ] == nCol
      loop
   elseif aPos[ 1 ] > nMaxRow .or. aPos[ 1 ] < nMinRow .or. ;
          aPos[ 2 ] > nMaxCol .or. aPos[ 2 ] < nMinCol
      SetMousePos( nRow, nCol )
      loop
   endif

   dispbegin()
   buffer := savescreen( ntop, nleft, ntop + topoffset - 1, nleft + leftoffset - 2 )
   restscreen( ntop, nleft, ntop + topoffset, nleft + leftoffset, oldscrn )

   ntop  += ( aPos[ 1 ] - nRow ) ; nRow := aPos[ 1 ]
   nleft += ( aPos[ 2 ] - nCol ) ; nCol := aPos[ 2 ]

   oldscrn := savescreen( ntop, nleft, ntop + topoffset, nleft + leftoffset )
   gfattr( ntop + 1, nleft + leftoffset - 1, ntop + topoffset - 1, nleft + leftoffset, 8 )
   gfattr( ntop + topoffset - 1, nleft + 2, ntop + topoffset, nleft + leftoffset, 8 )
   restscreen( ntop, nleft, ntop + topoffset - 1, nleft + leftoffset - 2, buffer )
   dispend()
end

enableDrag( .F. )          // Disable mouse dragging

return
* end function MouseMove()
*--------------------------------------------------------------------*

#endif

* eof moving.prg
