/*
   Program: SETFILT.PRG
   System: GRUMPFISH LIBRARY
   Author: Greg Lief
   Copyright (c) 1988-93, Greg Lief
   CA-Clipper 5.x Version
   Compile instructions: clipper setfilt /n /w

   User-friendly interactive query builder
*/

#include "grump.ch"
#include "inkey.ch"
#include "dbstruct.ch"

// the following three items are declared here because they must
// be visible within several functions herein

static mfilter            // the filter condition being built
static mefilter           // english translation of same
static fieldinfo          // array of info for the current field

// manifest constants to make things more readable
#define FieldName  fieldinfo[DBS_NAME]
#define FieldType  fieldinfo[DBS_TYPE]
#define FieldLen   fieldinfo[DBS_LEN]

// manifest constants for return values from AddFilt() to NewFilt()
#define DISCARDED  1
#define DONE      12
#xtranslate ShowFilt() => memoedit(mefilter, 04, 01, 17, 37, .f., .f.)

external soundex

function setfilt(aDescrip)
local malias
local run_it_now
local oldscore := set(_SET_SCOREBOARD, .f.)
local maincolor := ColorSet(C_APICK_BOXOUTLINE, .T.)
local browse
local column
local mcount
local buffer
local oldcolor
local key
local oldfkeys_
local lMainLoop := .t.
local nMaxRow  := maxrow()
#ifdef MOUSE
local aHotSpots := { { nMaxRow,  0,  0, K_UP    }, ;
                     { nMaxRow,  1,  1, K_DOWN  }, ;
                     { nMaxRow, 11, 12, K_F7    }, ;
                     { nMaxRow, 22, 23, K_F8    }, ;
                     { nMaxRow, 34, 35, K_F9    }, ;
                     { nMaxRow, 44, 46, K_F10   }, ;
                     { nMaxRow, 60, 61, K_ENTER }, ;
                     { nMaxRow, 72, 74, K_ESC   } }

#endif

if empty( malias := upper(alias()) )
   err_msg( { "No database open" } )
else
   // must disable F7, F8, F9, F10 because they are used in my main
   // keypress loop below
   oldfkeys_ := { setkey(K_F7, NIL), setkey(K_F8, NIL), ;
                  setkey(K_F9, NIL), setkey(K_F10, NIL) }

   GFSaveEnv(.t., 0)       // shut off cursor
   scroll()
   if ! file('queries.dbf')
      waiton('initializing query file... please wait')
      dbcreate('queries.dbf', { { "DESCRIP",   "C", 50, 0 } , ;
                              { "MFILTER",   "M", 10, 0 } , ;
                              { "MEFILTER",  "M", 10, 0 } , ;
                              { "QUERY_FILE","C",  8, 0 } } )
      waitoff()
   endif
   use queries new

   // give user choice of existing queries first
   // create new browse object
   browse := TBrowseDB(5, 15, 19, 64)
   browse:headSep := ""
   browse:colorSpec := maincolor + ', I, W/B, N/W'
   column := TBColumnNew( "Query Description", { | | queries->descrip } )
   browse:addColumn(column)

   @ nMaxRow,03 ssay 'move'
   @ nMaxRow,14 ssay 'count'
   @ nMaxRow,25 ssay 'delete'
   @ nMaxRow,37 ssay 'view'
   @ nMaxRow,48 ssay 'new query'
   @ nMaxRow,63 ssay 'select'
   @ nMaxRow,76 ssay 'exit'
   setcolor('I')
   @ nMaxRow,00 ssay chr(24)+chr(25)
   @ nMaxRow,11 ssay 'F7'
   @ nMaxRow,22 ssay 'F8'
   @ nMaxRow,34 ssay 'F9'
   @ nMaxRow,44 ssay 'F10'
   @ nMaxRow,60 ssay chr(17) + ""
   @ nMaxRow,72 ssay 'Esc'
   ColorSet(C_APICK_BOXOUTLINE)
   set filter to trim(queries->query_file) == malias
   go top
   shadowbox(04, 14, 20, 65, 3)
   do while lMainLoop
      do while (key := inkey()) = 0 .and. ! browse:stabilize()
      enddo
      if browse:stable
         key := ginkey(0, "KEY")
      endif

      #ifdef MOUSE
      if key == K_LEFT_DBLCLICK
         if GFLastRow() == browse:nTop + browse:rowPos + 1 .and. ;
            GFLastCol() >= browse:nLeft .and. GFLastCol() <= browse:nRight
            key := K_ENTER
         else
            key := K_LEFT_DOWN
         endif
      endif

      do case
         case key == K_RIGHT_DOWN
            key := K_ESC

         case key == K_LEFT_DOWN
            if GFLastRow() >= browse:nTop + 2 .and. ;
               GFLastRow() <= browse:nBottom  .and. ;
               GFLastCol() >= browse:nLeft .and. GFLastCol() <= browse:nRight
               browse:rowPos := GFLastRow() - browse:nTop - 1
               browse:refreshAll()
            else
               aeval( aHotSpots, {| a | iif( GFLastRow() == a[ 1 ] .and. ;
                                             GFLastCol() >= a[ 2 ] .and. ;
                                             GFLastCol() <= a[ 3 ],      ;
                                             key := a[ 4 ], )} )
            endif
      endcase
      #endif

      do case
         case key == K_UP
            browse:up()
         case key == K_LEFT
            browse:left()
         case key == K_RIGHT
            browse:right()
         case key == K_DOWN
            browse:down()
         case key == K_CTRL_PGUP
            browse:goTop()
         case key == K_CTRL_PGDN
            browse:goBottom()
         case key == K_PGUP .or. key == K_HOME
            browse:pageUp()
         case key == K_PGDN .or. key == K_END
            browse:pageDown()
         case key == K_ESC
            lMainLoop := .f.
         case key == K_ENTER
            if eof()
               err_msg( { "No query to select!" } )
            else
               if ! empty(queries->mfilter)
                  select(select(malias))
                  waiton('Searching records.. please wait')
                  dbsetfilter(&("{ || " + queries->mfilter + "}"), ;
                              queries->mfilter )
                  go top
                  if eof()
                     err_msg( { 'No records meet those criteria' } )
                     set filter to
                     select queries
                     waitoff()
                  else
                     waitoff()
                     lMainLoop := .f.
                  endif
               else
                  select(select(malias))
                  go top
                  lMainLoop := .f.
               endif
            endif

         case key == K_F7     // count records for this query
            if eof()
               err_msg( { 'No query to count' } )
            else
               select(select(malias))
               waiton('Now counting records.. please wait')
               go top
               // if the filter condition is empty, then the "filter" is
               // a moot point -- show 'em total # of records
               if empty(queries->mfilter)
                  mcount := lastrec()
               else
                  mfilter := &("{ | | " + queries->mfilter + "}" )
                  count for eval(mfilter) to mcount
               endif
               oldcolor := ColorSet(C_WAITMESSAGE)
               @ 12, 23 ssay padr( str(mcount, 8) + ' record' + ;
                                  if(mcount > 1, 's', '') + ;
                                  ' for this criteria', 34)
               ginkey(0)
               waitoff()
               setcolor(oldcolor)
               select queries
            endif

         case key == K_F8                     // delete query
            if eof()
               err_msg( { 'No query to delete' } )
            else
               if yes_no('Are you sure you want to delete this query')
                  if rlock()
                     delete
                     use queries exclusive
                     if ! neterr()
                        waiton('deleting query.... please wait')
                        pack
                        copy structure to tempqry
                        use tempqry exclusive
                        append from queries
                        ferase('queries.dbf')
                        ferase('queries.dbt')
                        use
                        frename('tempqry.dbf', 'queries.dbf')
                        frename('tempqry.dbt', 'queries.dbt')
                        use queries
                        set filter to trim(queries->query_file) == malias
                        go top
                        waitoff()
                        browse:refreshAll()
                     else
                        err_msg( { NETERR_MSG } )
                     endif
                  else
                     err_msg( { NETERR_MSG } )
                  endif
                  unlock
               endif
            endif

         case key == K_F9                     // view criteria
            if eof()
               err_msg( { 'No query to view' } )
            else
               oldcolor := ColorSet(C_ERRORMESSAGE)
               buffer := shadowbox(07, 25, 17, 54, 4, 'Query Criteria')
               @ 17,30 ssay ' press Esc to exit '
               memoedit(queries->mefilter, 8, 26, 16, 53, .f.)
               setcolor(oldcolor)
               byebyebox(buffer)
            endif

         case key == K_F10                    // create new query
            gfsaveenv(.t.)
            select(select(malias))
            run_it_now := NewFilt(malias, aDescrip)
            gfrestenv()
            go top
            if run_it_now
               select(select(malias))
               waiton('Searching records.. please wait')
               if ! empty(mfilter)
                  dbsetfilter(&("{ || " + mfilter + "}"), mfilter )
               endif
               go top
               if eof()
                  err_msg( { 'No records meet those criteria' } )
                  set filter to
                  select queries
               else
                  waitoff()
                  lMainLoop := .f.
               endif
            endif
            browse:refreshAll()

      endcase
   enddo

   GFRestEnv()
   select queries
   use
   if len(malias) > 0
      select(select(malias))
   endif

   // reset F7, F8, F9, F10 keys to their previous values
   for key := 1 to 4
      setkey(-(key + 5), oldfkeys_[key])
   next
endif
return nil

* end function SetFilt()
*--------------------------------------------------------------------*


/*
   Function: NewFilt()
*/
static function NewFilt(malias, aDescrip)
local botrow
local returncode
local lRunItNow := .f.
local oldcolor := setcolor()
local lFirstLoop := .t.
local browse
local column
local key
local mdescrip := space(50)
local marker := recno()
local aFields := dbstruct()
local ele := 1
local getlist := {}
local lMainLoop := .t.
local xx
local openparen := 0       // number of open parentheses in filter criteria
local nMaxwidth := 0
#ifdef MOUSE
local aHotSpots := { { 21, 2, 2, K_UP       }, ;
                     { 21, 3, 3, K_DOWN     }, ;
                     { 22, 2, 3, K_ENTER,   }, ;
                     { 23, 2, 4, K_ESC      } }
#endif

// initialize selection array based on whether a layman's description
// array was passed as a parameter to SETFILT()
if aDescrip == NIL
   aDescrip := array( len(aFields) )
   aeval(aFields, { | f, ele | aDescrip[ele] := f[DBS_NAME] } )
endif
// must determine maximum width for sizing TBrowse window
nMaxwidth := 0
aeval(aDescrip, { | f | nMaxwidth := max(nMaxwidth, len(f) ) } )

// initialize filter condition and english translation
mfilter := []
mefilter := 'ALL RECORDS'

// use phantom record to get initial value for this field
go bottom
skip

setcolor('w/n')
dispbegin()
scroll()
colorset(C_APICK_BOXOUTLINE)
@0,33 ssay " Query Builder "
exbox(3, 0, 18, 38, 2, 0, '', .f., 'Selection Criteria')
@ 04,01 ssay mefilter
botrow := min(fcount() + 2, 23)
SINGLEBOX(20, 00, 24, 25)
@ 21,05 ssay 'Move Highlight Bar'
@ 22,06 ssay 'To Select An Item'
@ 23,06 ssay 'To End Selection'
@ 21,02 ssay chr(24)+chr(25) color 'I'
@ 22,02 ssay chr(17)+"" color 'I'
@ 23,02 ssay 'Esc' color 'I'

browse := TBrowseNew( 2, 61 - (nMaxwidth / 2), botrow - 1, ;
                      61 + (nMaxwidth / 2) - (nMaxwidth % 2) )
browse:colorSpec := colorset(C_APICK_BOXOUTLINE, .t.) + ", I"
browse:skipBlock := { | SkipCnt | Gilligan(@ele, SkipCnt, len(aFields)) }
column := TBColumnNew( "", { | | aDescrip[ele] } )
column:width := nMaxwidth
browse:addColumn(column)
ShadowBox(1, browse:nLeft - 1, botrow, browse:nRight + 1, 4, 'Fields')
dispend()
do while lMainLoop
   do while (key := inkey()) == 0 .and. ! browse:stabilize()
   enddo
   if browse:stable
      ShowFilt()
      key := ginkey(0, "KEY")
   endif

   #ifdef MOUSE
   if key == K_LEFT_DBLCLICK
      if GFLastRow() == browse:nTop + browse:rowPos - 1 .and. ;
         GFLastCol() >= browse:nLeft .and. GFLastCol() <= browse:nRight
         key := K_ENTER
      else
         key := K_LEFT_DOWN
      endif
   endif

   do case
      case key == K_RIGHT_DOWN
         key := K_ESC

      case key == K_LEFT_DOWN
         if GFLastRow() >= browse:nTop  .and. GFLastRow() <= browse:nBottom .and. ;
            GFLastCol() >= browse:nLeft .and. GFLastCol() <= browse:nRight
            browse:rowPos := GFLastRow() - browse:nTop + 1
            browse:refreshAll()
         else
            aeval( aHotSpots, {| a | iif( GFLastRow() == a[ 1 ] .and. ;
                                          GFLastCol() >= a[ 2 ] .and. ;
                                          GFLastCol() <= a[ 3 ],      ;
                                          key := a[ 4 ], )} )
         endif
   endcase
   #endif

   do case
      case key == K_UP
         browse:up()
      case key == K_DOWN
         browse:down()
      case key == K_ENTER .and. aFields[ele][DBS_TYPE] <> "M"
         // dump this particular array element to a mini-array!
         // note that fieldinfo is declared as an external static
         // at the top of this program because we need to have it
         // visible in the hot-key function VIEW_VALS
         fieldinfo := aFields[ele]
         // note that OPENPAREN may need to be altered in AddFilt()
         if (returncode := AddFilt(fieldinfo, aDescrip[ele], @openparen)) == DONE
            lMainLoop := .f.
         endif
         if lFirstLoop .and. returncode > DISCARDED
            lFirstLoop := .f.
         endif

         #ifdef MOUSE
         if lMainLoop
            ShowMouseCursor()
         endif
         #endif

      case key == K_ESC
         lMainLoop := .f.
         // check for unclosed booleans (AND, OR) at the end of
         // current filter condition
         if ! empty(mfilter)
            do case
               case right(mfilter, 6) == ".AND. "
                  mfilter := substr(mfilter, 1, len(mfilter) - 7)
                  mefilter := substr(mefilter, 1, len(mefilter) - 7)
               case right(mfilter, 5) == ".OR. "
                  mfilter := substr(mfilter, 1, len(mfilter) - 6)
                  mefilter := substr(mefilter, 1, len(mefilter) - 6)
            endcase
         endif
   endcase
enddo

go marker
colorset(C_APICK_BOXOUTLINE)
ShowFilt()
select queries
ColorSet(C_MESSAGE)
shadowbox(09, 07, 15, 72, 2)
@ 10,09 ssay 'You may now enter a description of up to 50 characters for'
@ 11,09 ssay 'this query.  If you wish to run this query immediately without'
@ 12,09 ssay 'saving it, leave the description empty and press Enter.  If '
@ 13,09 ssay 'you want to exit without saving this query, press Esc.'
@ 14,15 get mdescrip
setcursor(1)
#ifdef MOUSE
   ShowMouseCursor()
#endif
read
setcursor(0)

#ifdef MOUSE
if lastkey() <> K_ESC .and. lastkey() <> K_RIGHT_DOWN
#else
if lastkey() <> K_ESC
#endif
   if len(trim(mdescrip)) == 0   // left it blank -- do not save it
      lRunItNow := .t.          // to run query immediately - see querybrow()
   else
      // force this record to be added with the following 'endless loop'
      do while .t.
         append blank
         if ! neterr()
            queries->descrip    := mdescrip
            queries->query_file := malias
            queries->mfilter    := mfilter
            queries->mefilter   := mefilter
            exit
         endif
      enddo
   endif
endif
setcolor(oldcolor)
return lRunItNow

* end static function NewFilt()
*--------------------------------------------------------------------*


/*
   Function: ADDFILT()
*/
static function AddFilt(fieldinfo, cPseudo, openparen)

// arrays to be used when selecting fields and operators
static op_string := { '=', '<', '>', '<=', '>=', '<>' }
static operators := { 'Equal to', 'Less than', 'Greater than', ;
       'Less than or equal to', 'Greater than or equal to', 'Not equal to', ;
       'Contains', 'Does not contain', 'Sounds like' }
static op_choices := { .t., .t., .t., .t., .t., .t., .t., .t., .t., .t. }
local booleans := { 'Discard', ' .AND. ', ' .OR. ', '( ... .AND.', ;
                     '( ... .OR.', ' .AND. (', ' .OR. (', ') .AND. ', ;
                     ') .OR. ', ') .AND. (', ') .OR. (', '<<done>>' }
static bl_choices := { .t., .t., .t., .t., .t., .t., .t., .t., .t., .t., .t. }
local mvalue
local mfield
local mefield
local wid
local boxbott
local op
local mop
local meop
local mboolean
local pic_len
local mpic
local mplain := ColorSet(C_APICK_BOXOUTLINE, .T.)  + ',I'
local oldaltv
local menhanced := '+' + mplain
local getlist := {}
gfsaveenv(.t.)
mvalue := fieldget(fieldpos(FieldName))
do case
   case FieldType == 'C'
      if len(mvalue) > 35
         mpic := "@S35"
         pic_len := 35
      else
         mpic := replicate("X", (pic_len := len(mvalue)) )
      endif

   case FieldType == 'D'
      pic_len := 8
      mpic := "@D"

   case FieldType == 'N'
      op := str(mvalue)
      if "." $ op
         mpic := replicate('9', at(".", op) - 1) + "."
         mpic += replicate('9', len(op) - len(mpic))
      else
         mpic := replicate('9', len(op))
      endif
      pic_len := len(mpic)

   case FieldType == 'L'
      pic_len := 1
      mpic := "Y"
endcase
mop := ' = '
meop := ' equal to '
if FieldType <> 'L'       // get operator for non-logical fields
   colorset(C_YESNO)
   boxbott := if(FieldType == 'C', 13, 10)
   // note: trimming field pseudo-name to 24 characters
   op := 0
   // only permit access to soundex() and substr() choices if this
   // is a character variable -- logical enough, eh?
   op_choices[7] := op_choices[8] := op_choices[9] := (FieldType = 'C')
   aeval( operators, {| o, e | iif( op_choices[ e ], , o := "~" + o )} )
   do while op == 0
      op := apick(3, 48, boxbott - 2, 71, operators )
   enddo
   meop := ' ' + trim(operators[op]) + ' '
   if op < 7
      mop := ' ' + op_string[op]
   endif
endif
wid := max(len(cPseudo) + len(meop) + pic_len, 30)
ColorSet(C_MESSAGE)
ShadowBox(05, 68 - wid, 08, 72, 4)
// establish ALT_V as hot-key for viewing values in database.
oldaltv := setkey(K_ALT_V, { || View_Vals() } )
#ifdef MOUSE
   // enable left click for viewing values in database
   setkey(K_LEFT_DOWN, { || MViewVals() } )
#endif
@ 07, 42 ssay '(Alt-V for available values)'
#ifdef MOUSE
   @ 07, 43 ssay 'Alt-V' color "I"
#endif
@ 06, 70-wid ssay cPseudo + meop
@ 06, col() get mvalue picture mpic
setcursor(1)
#ifdef MOUSE
   ShowMouseCursor()
#endif
read
setcursor(0)
setkey( K_ALT_V, oldaltv)     // restore ALT_V to its previous state
#ifdef MOUSE
   setkey(K_LEFT_DOWN, NIL)
#endif
do case
   case FieldType == 'N'
      mfield := FieldName + mop + ltrim(str(mvalue))
      mefield := cPseudo + meop + ltrim(str(mvalue))
   case FieldType == 'L'
      mfield := if(mvalue, FieldName, '! ' + FieldName)
      mefield := if(mvalue, cPseudo, 'not ' + cPseudo)
   case FieldType == 'D'
      mfield := FieldName + mop + "ctod('" + dtoc(mvalue) + "')"
      mefield := cPseudo + meop + dtoc(mvalue)
   otherwise
      // criteria 7 and 8 ('contains' and 'sounds like') are special cases
      // and must be processed a bit differently than the first six
      do case
         case op == 7        // 'contains'
            mfield := '[' + trim(mvalue) + '] $ ' + FieldName
         case op == 8        // 'does not contain'
            mfield := '! [' + trim(mvalue) + '] $ ' + FieldName
         case op == 9        // 'sounds like'
            mfield := 'soundex(' + FieldName + ') = soundex([' + ;
                     trim(mvalue) + '])'
         otherwise
            mfield := FieldName + mop + "[" + trim(mvalue) + "]"
      endcase
      mefield := cPseudo + meop + "'" + trim(mvalue) + "'"
endcase
ColorSet(C_YESNO)
mboolean := 0
// only allow selection of leading open parenthesis on first pass
bl_choices[4] := bl_choices[5] := empty(mfilter)
// only allow selection of something involving a closed parenthesis
// if there is at least one open parenthesis already...
bl_choices[8] := bl_choices[9] := bl_choices[10] := bl_choices[11] := (openparen > 0)
// force them to make a selection
aeval( bl_choices, {| b, e | iif( b, , booleans[ e ] := "~" + booleans[ e ] )} )
do while mboolean == 0
   mboolean := apick(11, 55, 22, 66, booleans )
enddo
if mboolean == len(booleans)
   // add the required number of closed parenthesis to balance it out
   mfilter += mfield + replicate(')', openparen)
   mefilter := strtran(mefilter,'ALL RECORDS', '') + mefield + ;
               replicate(')', openparen)
elseif mboolean > 1    // selection 1 means they want to discard this condition
   if mboolean > 3 .and. mboolean < 7
      openparen++
   elseif mboolean == 8 .or. mboolean == 9
      openparen--
   endif
   if empty(mfilter) .and. (mboolean == 4 .or. mboolean == 5)
      mfilter  := "( " + mfield + if(mboolean == 4, " .AND. ", " .OR. ")
      mefilter := "(" + mefield + if(mboolean == 4, " AND ", " OR ")
   else
      mfilter  += mfield + booleans[mboolean]
      mefilter := strtran(mefilter, 'ALL RECORDS', '') + mefield + ;
                  booleans[mboolean]
   endif
   setcolor(mplain)
   scroll(23, 02, 23, 24, 0)
endif
gfrestenv()
return mboolean

* end static function AddFilt()
*--------------------------------------------------------------------*


#ifdef MOUSE

/*
   Function: MViewVals()
   Purpose:  Enables user to pop-up values in database by clicking
             on left mouse button
   Note:     User must position mouse cursor over the "Alt-V" message
*/
static function MViewVals()
local bOldKey := setkey(K_LEFT_DOWN, NIL)
local aPos := GetMousePos()
if aPos[1] == 7 .and. aPos[2] > 41 .and. aPos[2] < 70
   View_Vals()
endif
setkey(K_LEFT_DOWN, bOldKey)
return nil

* end static function MViewVals()
*--------------------------------------------------------------------*

#endif


/*
    Function: VIEW_VALS
    Hot key (Alt-V) to pop up field values for quick reference
*/
static function view_vals
local browse
local column
local key
local buffer
local marker := recno()
local oldblock := setkey( K_ALT_V, NIL )

#ifdef MOUSE
local nMRow
local nMCol
#endif

gfsaveenv(, 0, colorset(C_APICK_BOXOUTLINE))
buffer := shadowbox(10, 70 - min(FieldLen, 70), 20, 71, 2)
browse := TBrowseDB(11, 71 - min(FieldLen, 70), 19, 70)
browse:headSep := ""
browse:colorSpec := colorset(C_APICK_BOXOUTLINE, .t.) + ',' + ;
                    colorset(C_APICK_INDICATOR, .t.)
column := TBColumnNew( "Value", FieldBlock(FieldName) )
browse:addColumn(column)
go top
do while key <> K_ESC .and. key <> K_ENTER
   do while (key := ginkey()) = 0 .and. ! browse:stabilize()
   enddo
   if browse:stable
      key := ginkey(0, "KEY")
   endif

   #ifdef MOUSE
   if key == K_LEFT_DBLCLICK
      if GFLastRow() == nMRow .and. GFLastCol() == nMCol
         key := K_ENTER
      else
         key := K_LEFT_DOWN
      endif
   endif

   do case
      case key == K_RIGHT_DOWN
         key := K_ESC

      case key == K_LEFT_DOWN
         nMRow := GFLastRow()
         nMCol := GFLastCol()

         if nMRow >= browse:nTop .and. nMRow <= browse:nBottom
            browse:rowPos := nMRow - browse:nTop - 1
            browse:refreshAll()
         // clicked on top row of box... move up
         elseif nMRow == browse:nTop - 1
            key := K_PGUP
         // clicked on bottom row of box... move down
         elseif nMRow == browse:nBottom + 1
            key := K_PGDN
         endif

   endcase
   #endif

   do case
      case key == K_UP
         browse:up()
      case key == K_DOWN
         browse:down()
      case key == K_CTRL_PGUP
         browse:goTop()
      case key == K_CTRL_PGDN
         browse:goBottom()
      case key == K_PGUP .or. key == K_HOME
         browse:pageUp()
      case key == K_PGDN .or. key == K_END
         browse:pageDown()
   endcase
enddo
if key <> K_ESC
   getactive():varPut( fieldget(fieldpos(FieldName)) )
endif
go marker
byebyebox(buffer)
gfrestenv()
#ifdef MOUSE
ShowMouseCursor()
#endif
setkey(K_ALT_V, oldblock)   // reset Alt-V for next time
return NIL

* end static function View_Vals()
*--------------------------------------------------------------------*

* eof setfilt.prg
