/*
    Function: TTY2()
    System:   GRUMPFISH LIBRARY
    Author:   Greg Lief, with help from Dennis Levin
    Copyright (c) 1988-93, Greg Lief
    CA-Clipper 5.x Version
    Compile instructions: clipper tty /n /w
    Displays message on screen "teletype" style until keypress
*/

#include "grump.ch"

function tty2(nRow, cMsg, nDelay, cColor)
local nLen
local nLeft
local nCol
local nStart
local cTemp
local yy
local nMaxCol := maxcol()
if nRow <= maxrow()
   GFSaveEnv( { nRow, 0, nRow, maxcol() } )

   // use default message if a null string was passed
   default cMsg to 'Press any key to continue'

   // truncate the message if longer than 70 characters
   cMsg := substr(cMsg, 1, 70)

   default nDelay to 0
   setcolor(cColor)
   nLen := len(cMsg) + 10            // determine how many dots to use
   nLeft := int((nMaxCol + 1 - nLen) / 2)
   @ nRow, nLeft ssay replicate(chr(46), nLen)
   nCol := nLeft + nLen - 6        // start five columns from the right
   nStart := 1                       // substring ptr for when we hit the left

   while ginkey() == 0
      cTemp := substr(cMsg, nStart, nLen + nLeft - nCol - 5) + ;
               if(nStart > 1, replicate(chr(46), nStart - 1), '')
      @ nRow, nCol ssay cTemp
      if nDelay == 0            // use sound
         tone(MUSIC_TTY, 1)
      else
         for yy := 1 to nDelay
         next
      endif
      if nCol == nLeft + 5                    // we're at the left side
         if ++nStart > nLen - 10              // time to go back to the right
            @ nRow, nCol ssay chr(46)         // clear remainder of message
            nCol := nLeft + nLen - 5          // reset column pointer
            nStart := 1                       // reset substring pointer
         endif
      else
         nCol--
      endif
   end

   GFRestEnv()
endif
return NIL

* end function TTY2()
*--------------------------------------------------------------------*

* eof tty2.prg
