/*
    Program: VSCRNSCRL()
    System:  GRUMPFISH LIBRARY
    Author:  Greg Lief
    Copyright (c) 1988-93, Greg Lief
    CA-Clipper 5.x Version
    Compile instructions: clipper vscroll /n/w/a

    Displays screen in vertical row-by-row increments

    NOTE: screen must be pre-saved to the <screen file> using
    HBlindSave())

    Syntax: VSCRNSCRL(<screen file> [, <nDirection> ])

    Optional parameter <nDirection> determines whether the display
    will be pull-down or pop-up.  Pass 1 for pop-up, -1 for pull-down.
    (Default is pull-down.)  Thanks to Evan Davies for the suggestion!

    WILL HANDLE 25/43/50 ROW DISPLAYS
*/

#include "fileio.ch"
#include "grump.ch"


function VScrnScrl(cfile, ndirection)
local nMaxRow := maxrow() + 1
local nMaxCol := maxcol() + 1
local handle, aScreen[nMaxRow], buffer, xx, yy
local bufflen := nMaxCol * 2
local aLoopers
default ndirection to -1                // default is pull-down
if file(cfile)
   handle := fopen(cfile, FO_READ)
   if handle <> F_ERROR
      // verify that this file was saved in the same mode that we are in
      // because if it wasn't, all hell will break loose further down
      if fseek(handle, 0, FS_END) == nMaxRow * nMaxCol * 2
         fseek(handle, 0, FS_SET)       // reset to start of file
         for xx := 1 to nMaxRow
            buffer := space(bufflen)
            fread(handle, @buffer, bufflen)
            aScreen[xx] := buffer
         next
         fclose(handle)
         /*
             load aLoopers array based on direction -- structure is:
                  aLoopers[1] := starting row for FOR..NEXT loop
                  aLoopers[2] := ending row for FOR..NEXT loop
                  aLoopers[3] := row to be displayed within loop
         */
         if ndirection == 1                     // pop-up
            aLoopers := { 1, nMaxRow, nMaxRow }
         else                                   // pull-down
            aLoopers := { nMaxRow, 1, 0 }
         endif
         #ifdef MOUSE
         HideMouseCursor()
         #endif
         for xx := aLoopers[1] to aLoopers[2] step ndirection
            scroll(0, 0, nMaxRow, nMaxCol, ndirection)
            restscreen(aLoopers[3], 0, aLoopers[3], nMaxCol, aScreen[xx])
         next
         #ifdef MOUSE
         ShowMouseCursor()
         #endif
      endif
   endif
endif
return NIL

* end function VScrnScrl()
*--------------------------------------------------------------------*

* eof vscroll.prg
