/*
     Function: WaitOn()
     System: GRUMPFISH LIBRARY
     Author: Greg Lief
     Copyright (c) 1988-93, Greg Lief
     CA-Clipper 5.x Version
     Compile instructions: clipper waiton /n/w/a

     Display a wait message on the screen -- ideal for any tedious
     process.  If first parameter passed to this function is of logical
     type, it will assume that your tedious process has ended and will
     thus restore the affected portion of the screen.

     Calls: ShadowBox()    (function contained in SHADOWBO.PRG)
            ByeByeBox()    (function contained in BYEBYE.PRG)
*/

#include "grump.ch"

function waiton(msgs, lsound, ccolor, nTop, nLeft)
local x := 0
local nLength
#ifdef MOUSE
local lResetMouse := .f.
local aPos
#endif
local a
static waitsound
static aWaitStack := {}

// if the first parameter passed was of logical type, that means
// that the tedious process is over and we are to restore the screen
if valtype(msgs) == "L"
   // make sure that WAITON() was called first -- if it was not,
   // the aWaitStack array will be empty
   if ! empty(aWaitStack)
      if waitsound .or. (lsound <> NIL .and. lsound)
         tone(MUSIC_WAITON[2], 1)
         tone(MUSIC_WAITON[1], 1)
      endif

      dispbegin()

      a := atail(aWaitStack)

      #ifdef MOUSE

      // move mouse cursor out of screen drawing area if necessary
      aPos := GetMousePos()
      if aPos[1] >= a[1] .and. aPos[1] <= a[3] .and. ;
                         aPos[2] >= a[2] .and. aPos[2] <= a[4]
         lResetMouse := .t.
         SetMousePos(a[1]-1, a[2]-1)
      endif
      #endif

      ByeByeBox(a)

      dispend()

      #ifdef MOUSE

      if lResetMouse
         SetMousePos(aPos[1], aPos[2])
      endif

      #endif

      truncate(aWaitStack)
   endif
else
   // establish message -- use default if not passed
   default msgs to { 'Now printing... please wait' }
   default lsound to .t.

   // if message passed was a character string, convert to array
   if valtype(msgs) == "C"
      msgs := { msgs }
   endif
   nLength := len(msgs)

   gfsaveenv( , 0)             // shut off cursor
   // use color default if not passed
   ColorSet(IF(ccolor == NIL, C_WAITMESSAGE, ccolor))
   // use default coordinates (centered on screen) if not passed
   default nTop to maxrow() / 2 - len( msgs )

   // determine width of longest message in array
   aeval(msgs, { | a | x := max(len(a), x) } )

   default nLeft to int(maxcol() - 3 - x) / 2

   dispbegin()

   #ifdef MOUSE
   // move mouse cursor out of screen drawing area if necessary
   aPos := GetMousePos()
   if aPos[1] >= nTop .and. aPos[1] <= nTop + nLength + 2 .and. ;
                      aPos[2] >= nLeft .and. aPos[2] <= nLeft + 5 + x
      lResetMouse := .t.
      SetMousePos(nTop - 1, nLeft - 1)
   endif
   #endif

   aadd(aWaitStack, shadowbox(nTop, nLeft, nTop + nLength + 1, nLeft + 3 + x, 1))
   waitsound := lsound
   if ccolor == NIL
      ColorSet( C_WAITMESSAGEBLINK )
   endif
   setpos(nTop, 0)
   aeval(msgs, { | line | setpos(row() + 1, nLeft + 2), dispout(line) } )

   dispend()

   #ifdef MOUSE

   if lResetMouse
      SetMousePos(aPos[1], aPos[2])
   endif

   #endif

   if waitsound
      tone(MUSIC_WAITON[1], 1)
      tone(MUSIC_WAITON[2], 1)
   endif

   gfrestenv()

endif

return NIL

* end function WaitOn()
*--------------------------------------------------------------------*

* eof waiton.prg
