/*
    Program: YES_NO()
    System: GRUMPFISH LIBRARY
    Author: Greg Lief
    Copyright (c) 1988-93, Greg Lief
    CA-Clipper 5.x Version
    Compile instructions: clipper yesno /n /w
    Compile with "/dMOUSE" for mouseability

    Display a Yes/No question and wait for user to respond
*/

#include "grump.ch"

function Yes_No(cMsg, cMsg2, cBoxColor, cTextColor, cTitle)
local nWidth
local oldscrn
local lOneLine := .F.
local nMaxRow := maxrow()
local nTop := nMaxRow - 6
local lRetval
local cKey
local nKey
local cBuffer

#ifdef MOUSE
   local nMsgRow
   local nMsgCol
   local aPos
   local lResetMouse := .f.
#endif

GFSaveEnv(, 0)                  // shut off cursor
default cMsg2 to []

// determine how many lines will be displayed, and top row of box
if empty(cMsg2)
   lOneLine := .T.
   nTop++
   cMsg += '? Y/N '
   nWidth := int(max(maxcol() - 5 - len(cMsg), 0)) / 2
else
   cMsg2 += '? Y/N '
   nWidth := int(maxcol() - 5 - max(len(cMsg), len(cMsg2))) / 2
endif

// establish default box and text colors
if empty(cBoxColor)     // either not passed or null string -- use default
   cBoxColor := ColorSet(C_YESNO, .t.)
endif
default cTextColor to cBoxColor

dispbegin()

#ifdef MOUSE
// move mouse cursor out of screen drawing area if necessary
aPos := GetMousePos()
if aPos[1] >= nTop .and. aPos[1] <= nMaxRow .and. aPos[2] >= nWidth ;
                   .and. aPos[2] <= maxcol() - nWidth + 1
   lResetMouse := .t.
   SetMousePos(nTop - 1, nWidth - 1)
endif
#endif

// if title message was not passed, check current
// date format to determine language
if cTitle == NIL
   cBuffer := left(set(_SET_DATEFORMAT), 8)
   if cBuffer == "dd.mm.yy"   // SET DATE GERMAN
      cTitle := "Frage"
   else
      cTitle := "Question"
   endif
endif

oldscrn := shadowbox(nTop, nWidth, nMaxRow - 1, maxcol() - nWidth, 3, ;
                     cTitle, , cBoxColor)
ScrnCenter(nMaxRow - if(lOneLine, 3, 4), cMsg, cTextColor)

#ifdef MOUSE

if lResetMouse
   SetMousePos(aPos[1], aPos[2])
endif

if lOneLine
   nMsgRow := row()
   nMsgCol := col() - 4
endif
#endif

dispend()

if ! lOneLine
   ScrnCenter(row() + 1, cMsg2, cTextColor)

   #ifdef MOUSE
   nMsgRow := row()
   nMsgCol := col() - 4
   #endif
endif

while lRetval == NIL
   nKey := ginkey( 0 )

   #ifdef MOUSE
   do case
      case nKey == K_LEFT_DOWN
         if GFLastRow() == nMsgRow
            if GFLastCol() == nMsgCol
               lRetVal := .T.
            elseif GFLastCol() == nMsgCol + 2
               lRetVal := .F.
            endif
         endif

      case nKey == K_RIGHT_DOWN
         lRetVal := .F.

      otherwise
   #endif
         cKey := upper( chr( nKey ) )
         if cKey $ "YN"
            lRetval := (cKey == "Y")
         endif
   #ifdef MOUSE
   endcase
   #endif
end

byebyebox(oldscrn)
GFRestEnv()
return lRetval

* end function Yes_No()
*--------------------------------------------------------------------*

* eof yesno.prg
