/*
    Program: YES_NO2()
    System: GRUMPFISH LIBRARY
    Author: Greg Lief
    Copyright (c) 1988-93, Greg Lief
    CA-Clipper 5.x Version
    Compile instructions: clipper yesno2 /n /w
    Compile with "/dMOUSE" for mouseability

    Syntax:     Yes_No(<cQuestion>, [<nTopRow>, <cYesMsg>, <cNoMsg>,
                <lInitVal>, <cBoxColor>, <cOptColor>, <lTab>, <lSpacebar>])

    Parameters: <cQuestion> -- message to display (character string)
                <nTopRow>   -- top row of box (numeric, default 20)
                <cYesMsg>   -- yes prompt (character, default " YES ")
                <cNoMsg>    -- no prompt (character, default " NO ")
                <lInitVal>  -- which value to highlight first
                               (.T. = Yes, .F. - No, default Yes)
                <cBoxColor> -- color to draw box and question (character,
                               string, default +W/RB)
                <cOptColor> -- color to draw prompts (character string,
                               default W/RB for unselected, "I" selected)
                <lTab>      -- flag to allow TAB to emulate arrow keys
                               (inactive by default)
                <lSpacebar> -- flag to allow SPACEBAR to emulate arrow
                               keys (inactive by default)

    Returns:  True if user selected YES, False if user selected NO or Escaped
*/

#include "grump.ch"
#include "inkey.ch"

#ifdef MOUSE
// structure of aCoords nested arrays
#define A_TOP   1
#define A_LEFT  2
#define A_RIGHT 3
#endif

function Yes_No2(aMsgs, nTop, cYesMsg, cNoMsg, lInitVal, ;
                 cBoxColor, cOptColor, lTab, lSpacebar)
local boxleft
local optslen
local oldscrn
local optionstr
local key
local mlen
local x := 0

#ifdef MOUSE
   local aCoords[ 2 ]
   local aPos
   local lResetMouse := .f.
#endif

GFSaveEnv( , 0)      // shut off cursor

// if message passed was a character string, convert to array
if valtype(aMsgs) == "C"
   aMsgs := { aMsgs }
endif
mlen := len(aMsgs)

// establish variables if not passed as parameters
default nTop to maxrow() / 2 - mlen - 1
default cYesMsg to " YES "
default cNoMsg to " NO "
default lInitVal to YES
default lTab to NO
default lSpacebar to NO

// determine width of longest message in array
aeval(aMsgs, { | a | x := max(len(a), x) } )

// determine width of box, and accordingly, left column of box
optslen := len(cYesMsg) + len(cNoMsg) + 3   // allow 3 spaces btwn yes & no
boxleft := int((maxcol() - 5 - max(x, optslen)) / 2)

dispbegin()

#ifdef MOUSE
// move mouse cursor out of screen drawing area if necessary
aPos := GetMousePos()
if aPos[1] >= nTop .and. aPos[1] <= nTop + 4 + mlen .and. aPos[2] >= boxleft ;
                   .and. aPos[2] <= maxcol() - boxleft + 1
   lResetMouse := .t.
   SetMousePos(nTop - 1, boxleft - 1)
endif
#endif

// save environment and get on with it
ColorSet(if(cBoxColor == NIL, C_YESNO, cBoxColor))
oldscrn := shadowbox(nTop, boxleft, nTop + 3 + mlen, maxcol() - boxleft, 2)
for x := 1 to mlen
   SCRNCENTER(nTop + x, aMsgs[x])
next

#ifdef MOUSE

if lResetMouse
   SetMousePos(aPos[1], aPos[2])
endif

#endif

dispend()

optionstr := upper(left(ltrim(cYesMsg), 1)) + upper(left(ltrim(cNoMsg), 1))

// main keypress loop
do while .t.
   // draw yes and no options in appropriate colors... if the
   // options color parameter was not passed, go with the standard
   // YESNO color for unselected, and GETS color for selected
   ColorSet(if(lInitVal, C_GETS, IF(cOptColor == NIL, C_YESNO, cOptColor)))
   setpos(nTop + mlen + 2, int(( maxcol() + 1 - optslen) / 2))

   #ifdef MOUSE
   if aCoords[ 1 ] == NIL
      aCoords[ 1 ] := { row(), col(), col() + len( cYesMsg ) - 1 }
   endif
   aPos := GetMousePos()
   // move mouse cursor out of the way of the prompt we are about to draw
   if aPos[1] == aCoords[1][A_TOP] .and. aPos[2] >= aCoords[1][A_LEFT] ;
                                   .and. aPos[2] <= aCoords[1][A_RIGHT]
      lResetMouse := .t.
      SetMousePos(aCoords[1][A_TOP], aCoords[1][A_LEFT] - 1)
   endif
   #endif

   dispout(cYesMsg)
   ColorSet(if(! lInitVal, C_GETS, IF(cOptColor == NIL, C_YESNO, cOptColor)))
   setpos(nTop + mlen + 2, col() + 3)

   #ifdef MOUSE
   if aCoords[ 2 ] == NIL
      aCoords[ 2 ] := { row(), col(), col() + len( cNoMsg ) - 1 }
   endif

   // move mouse cursor out of the way of the prompt we are about to draw
   if ! lResetMouse
      aPos := GetMousePos()
      if aPos[1] == aCoords[2][A_TOP] .and.       ;
              aPos[2] >= aCoords[2][A_LEFT] .and. ;
              aPos[2] <= aCoords[2][A_RIGHT]
         lResetMouse := .t.
         SetMousePos(aCoords[2][A_TOP], aCoords[2][A_LEFT] - 1)
      endif
   endif
   #endif

   dispout(cNoMsg)

   #ifdef MOUSE
   if lResetMouse
      SetMousePos(aPos[1], aPos[2])
      lResetMouse := .f.
   endif
   #endif

   key := ginkey( 0 )

   do case
      #ifdef MOUSE
      case key == K_RIGHT_DOWN
         lInitVal := NO
         exit

      case key == K_LEFT_DOWN
         if GFLastRow() == aCoords[ 1 ][ A_TOP   ] .and. ;
            GFLastCol() >= aCoords[ 1 ][ A_LEFT  ] .and. ;
            GFLastCol() <= aCoords[ 1 ][ A_RIGHT ]
            
            lInitVal := YES
            exit
         endif

         if GFLastRow() == aCoords[ 2 ][ A_TOP   ] .and. ;
            GFLastCol() >= aCoords[ 2 ][ A_LEFT  ] .and. ;
            GFLastCol() <= aCoords[ 2 ][ A_RIGHT ]
            
            lInitVal := NO
            exit
         endif
      #endif

      case key == K_ESC
         lInitVal := NO
         exit

      case key == K_ENTER
         exit

      case key == K_LEFT .or. key == K_RIGHT
         lInitVal := ! lInitVal

      case key == K_TAB .and. lTab
         lInitVal := ! lInitVal

      case key == SPACEBAR .and. lSpacebar
         lInitVal := ! lInitVal

      case upper( chr( key ) ) $ optionstr
         lInitVal := ( at( upper( chr( key ) ), optionstr ) == 1 )
         exit

   endcase
enddo

// restore environment and get out of here
GFRestEnv()
ByeByeBox(oldscrn)
return lInitVal

* end function Yes_No2()
*--------------------------------------------------------------------*

* eof yesno2.prg
